/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.google;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IGeoResourceInfo;
import net.refractions.udig.catalog.IResolveChangeEvent;
import net.refractions.udig.catalog.IResolveDelta;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.internal.CatalogImpl;
import net.refractions.udig.catalog.internal.ResolveChangeEvent;
import net.refractions.udig.catalog.internal.ResolveDelta;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.resource.ImageDescriptor;
import org.geotools.data.wfs.WFSDataStore;
import org.geotools.data.wms.WebMapServer;
import org.geotools.data.wms.xml.WMSSchema;
import org.geotools.xml.wfs.WFSSchema;

import com.vividsolutions.jts.geom.Envelope;

/**
 *  summary sentence.
 * <p>
 * Paragraph ...
 * </p><p>
 * Responsibilities:
 * <ul>
 * <li>
 * <li>
 * </ul>
 * </p><p>
 * Example:<pre><code>
 * GoogleResource x = new GoogleResource( ... );
 * TODO code example
 * </code></pre>
 * </p>
 * @author David Zwiers, Refractions Research
 */
abstract class GoogleResource extends IGeoResource {

    public static GoogleResource getResource(OGCLayer layer) {
        if (layer.getServertype().equalsIgnoreCase("WMS")) { //$NON-NLS-1$
            return getWMSResource(layer);
        } 
        if (layer.getServertype().equalsIgnoreCase("WFS")) { //$NON-NLS-1$
            return getWFSResource(layer);
        }
        return null;
    }
    
    public static GoogleResource getWMSResource(OGCLayer layer) {
        return new GoogleWMSResource(layer);
    }
    public static GoogleResource getWFSResource(OGCLayer layer) {
        return new GoogleWFSResource(layer);
    }
    
    protected Throwable msg = null;
    protected IGeoResourceInfo info = null;
    private URL id = null;
    
    public GoogleResource(OGCLayer layer) {
        id = layer.getId(); //$NON-NLS-1$
        info = new IGeoResourceInfo(layer.getTitle(), layer.getName(), layer.getDescription(), getSchema(), 
                new Envelope(), null, new String[] {layer.getServertype(), layer.getName(), layer.getTitle()}, getIcon() );
    }
    
    protected abstract URI getSchema();
    protected abstract ImageDescriptor getIcon();
    
    /*
     * Required adaptions:
     * <ul>
     * <li>IGeoResourceInfo.class
     * <li>IService.class
     * </ul>
     * 
     * @see net.refractions.udig.catalog.IResolve#resolve(java.lang.Class, org.eclipse.core.runtime.IProgressMonitor)
     */
    public <T> T resolve( Class<T> adaptee, IProgressMonitor monitor ) throws IOException{
        if(adaptee == null)
            return null;

        if(real != null)
            return real.resolve(adaptee,monitor);
        
        if(adaptee.isAssignableFrom(IGeoResourceInfo.class))
            return adaptee.cast( this.getInfo(monitor));
        
        loadReal(monitor);
        if(real != null)
            return real.resolve(adaptee,monitor);
        return null;
    }
    
    public IGeoResourceInfo getInfo(IProgressMonitor monitor) throws IOException{
        if(real!=null)
            return real.getInfo(monitor);
        return info;
    }
    protected IService service = null;
    protected IGeoResource real = null;
    protected abstract void loadReal(IProgressMonitor monitor) throws IOException;
    public IService parent(IProgressMonitor monitor) throws IOException{
        if(service == null)
            loadReal(monitor);
//System.out.println("SERVICE == NULL? "+(service==null));
        if(service!=null)
            return service;
        return null;
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#canResolve(java.lang.Class)
     */
    public <T> boolean canResolve( Class<T> adaptee ) {
        if(adaptee == null)
            return false;
        return (adaptee.isAssignableFrom(IGeoResourceInfo.class) || 
                adaptee.isAssignableFrom(IService.class));
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#getStatus()
     */
    public Status getStatus() {
        return msg==null?Status.CONNECTED:Status.BROKEN;
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#getMessage()
     */
    public Throwable getMessage() {
        return msg;
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#getIdentifier()
     */
    public URL getIdentifier() {
        if( real!=null )
            return real.getIdentifier();
        return id;
    }

static class GoogleWMSResource extends GoogleResource{

    /**
     * Construct <code>GoogleWMSResource</code>.
     *
     * @param layer
     * @throws MalformedURLException
     */
    public GoogleWMSResource( OGCLayer layer ) {
        super(layer);
        url = layer.getOnlineresource();
    }
    private URL url = null;

    /*
     * @see net.refractions.udig.catalog.google.GoogleResource#getSchema()
     */
    protected URI getSchema() {
        return WMSSchema.NAMESPACE;
    }

    /*
     * @see net.refractions.udig.catalog.google.GoogleResource#getIcon()
     */
    protected ImageDescriptor getIcon() {
        return null;
    }
    @Override
    public <T> boolean canResolve( Class<T> adaptee ) {
        if(adaptee == null)
            return false;
        return super.canResolve(adaptee) ||
            adaptee.isAssignableFrom(WebMapServer.class) ||
            adaptee.isAssignableFrom(org.geotools.data.ows.Layer.class);
        
    }

    /*
     * @see net.refractions.udig.catalog.google.GoogleResource#loadReal(org.eclipse.core.runtime.IProgressMonitor)
     */
    protected void loadReal( IProgressMonitor monitor ) throws IOException {
        getInfo(monitor); // load me
        if(info == null || url == null|| info.getName() == null)
            return;
        
        URL id = getIdentifier();
        if(id==null)
            return;
        
        if(service == null){
        	List<IService> services = CatalogPlugin.getDefault().getServiceFactory().aquire(url);
        	Iterator<IService> srv = services.iterator();
        	while(srv.hasNext()){
        		IService s = srv.next();
        		if(s.canResolve(WebMapServer.class)){
        			service = s;
        		}
        	}

            if(service == null){
                return;
            }
        }
        if(real == null){
	        CatalogPlugin.getDefault().getLocalCatalog().add(service);
	        // search for the child
	        List<? extends IGeoResource> resL = service.members(monitor);
	        if(resL==null){
	            return;
	        }
	        Iterator<? extends IGeoResource> res = resL.iterator();
	        while(res.hasNext() && real == null){
	            IGeoResource restmp = res.next();
	            if(restmp!=null && restmp.getIdentifier()!=null && info.getName().equals(restmp.getIdentifier().getRef()))
	                real = restmp;
	        }
        }
        IResolveDelta delta = new ResolveDelta( this, real, null);
        ((CatalogImpl)CatalogPlugin.getDefault().getLocalCatalog()).fire( new ResolveChangeEvent( this, IResolveChangeEvent.Type.POST_CHANGE, delta )  );
    }
}


static class GoogleWFSResource extends GoogleResource{

    /**
     * Construct <code>GoogleWMSResource</code>.
     *
     * @param layer
     * @throws MalformedURLException
     */
    public GoogleWFSResource( OGCLayer layer ) {
        super(layer);
        url = layer.getOnlineresource();
    }
    private URL url = null;

    /*
     * @see net.refractions.udig.catalog.google.GoogleResource#getSchema()
     */
    protected URI getSchema() {
        return WFSSchema.NAMESPACE;
    }

    /*
     * @see net.refractions.udig.catalog.google.GoogleResource#getIcon()
     */
    protected ImageDescriptor getIcon() {
        return null;
    }

    /*
     * @see net.refractions.udig.catalog.google.GoogleResource#loadReal(org.eclipse.core.runtime.IProgressMonitor)
     */
    protected void loadReal( IProgressMonitor monitor ) throws IOException {
        getInfo(monitor); // load me
        if(info == null || url == null|| info.getName() == null)
            return;
        
        URL id = getIdentifier();
        if(id==null)
            return;
        
        if(service == null){

        	List<IService> services = CatalogPlugin.getDefault().getServiceFactory().aquire(url);
        	Iterator<IService> srv = services.iterator();
        	while(srv.hasNext()){
        		IService s = srv.next();
        		if(s.canResolve(WFSDataStore.class)){
        			service = s;
        		}
        	}

            if(service == null){
                return;
            }
        }

        if(real == null){
	        // search for the child
	        List<? extends IGeoResource> resL = service.members(monitor);
	        if(resL==null)
	            return;
	        Iterator<? extends IGeoResource> res = resL.iterator();
	        while(res.hasNext() && real == null){
	            IGeoResource restmp = res.next();
	            if(restmp!=null && restmp.getIdentifier()!=null && info.getName().equals(restmp.getIdentifier().getRef()))
	                real = restmp;
	        }
        }
        IResolveDelta delta = new ResolveDelta( this, real, null );
        ((CatalogImpl)CatalogPlugin.getDefault().getLocalCatalog()).fire( new ResolveChangeEvent( this, IResolveChangeEvent.Type.POST_CHANGE, delta )  );
    }
}

}