/**
 * 
 */
package net.refractions.udig.catalog.internal.db2.ui;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.nio.channels.FileChannel;

import net.refractions.udig.catalog.db2.DB2Plugin;
import net.refractions.udig.catalog.internal.db2.Policy;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;

/**
 * @author jones
 */
public class DB2Preferences extends PreferencePage implements IWorkbenchPreferencePage {
    private static final String JDBC_DRIVER = "db2jcc.jar"; //$NON-NLS-1$
    private static final String LICENSE = "db2jcc_license_cu.jar"; //$NON-NLS-1$
    Text jdbcDriver;
    Text license;
    private Color red;
    private Color black;
    private Button browserDriver;
    private Button browseLicense;
    private Listener listener;
    private Shell shell;
    Control control;
    /**
     * @param title
     * @param image
     */
    public DB2Preferences() {
        super(Policy.bind("DB2Preferences.title")); //$NON-NLS-1$
    }

    public DB2Preferences( String file ) {
        super("file"); //$NON-NLS-1$
    }

    public DB2Preferences( String file, ImageDescriptor desc ) {
        super("file", desc); //$NON-NLS-1$
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
     */
    protected Control createContents( Composite parent ) {
        this.shell = parent.getShell();
        final Composite comp = new Composite(parent, SWT.NONE);
        FormLayout layout = new FormLayout();
        layout.marginBottom = 0;
        layout.marginTop = 0;
        layout.marginRight = 0;
        layout.marginLeft = 0;
        layout.spacing = 5;
        comp.setLayout(layout);

        createJDBCdriverUI(comp);
        createLicenseUI(comp);

        comp.setTabList(new Control[]{jdbcDriver, browserDriver, license, browseLicense});
        return comp;
    }

    /**
     * @param comp
     */
    private void createJDBCdriverUI( final Composite comp ) {
        Label label = new Label(comp, SWT.FLAT);
        label.setText(Policy.bind("DB2Preferences.driverLabel")); //$NON-NLS-1$

        jdbcDriver = new Text(comp, SWT.SINGLE | SWT.BORDER);

        jdbcDriver.addModifyListener(new ModifyListener(){
            public void modifyText( org.eclipse.swt.events.ModifyEvent e ) {
                File srcDriver = new File(jdbcDriver.getText());
                if (isInstalled() )
                    acceptance(Policy.bind("DB2Preferences.fileExists"), jdbcDriver); //$NON-NLS-1$
                else if (!srcDriver.exists() )
                	error(Policy.bind("DB2Preferences.fileNotFound"), jdbcDriver); //$NON-NLS-1$
                else
                	error(Policy.bind("DB2Preferences.notValid"), jdbcDriver); //$NON-NLS-1$

            };
        });
        String storedValue = getPreferenceStore().getString(JDBC_DRIVER);
        if (storedValue.trim().length() == 0 ) {
            storedValue = null;
        }
        jdbcDriver.setText(storedValue == null ? JDBC_DRIVER : storedValue);

        browserDriver = new Button(comp, SWT.PUSH);
        browserDriver.setText(Policy.bind("DB2Preferences.browse")); //$NON-NLS-1$

        browserDriver.addSelectionListener(new SelectionListener(){

            public void widgetSelected( SelectionEvent e ) {
                widgetDefaultSelected(e);
            }

            public void widgetDefaultSelected( SelectionEvent e ) {
                FileDialog fileDialog = new FileDialog(comp.getShell(), SWT.OPEN);
                fileDialog.setFilterExtensions(new String[]{"*.jar"}); //$NON-NLS-1$
                fileDialog.setFilterNames(new String[]{Policy.bind("DB2Preferences.archive")}); //$NON-NLS-1$
                String result = fileDialog.open();
                if (result != null)
                    jdbcDriver.setText(result);
            }

        });

        layout(null, label, jdbcDriver, browserDriver);
    }

    /**
     * @param comp
     */
    private void createLicenseUI( final Composite comp ) {
        Label label = new Label(comp, SWT.FLAT);
        label.setText(Policy.bind("DB2Preferences.licenceLabel")); //$NON-NLS-1$

        license = new Text(comp, SWT.SINGLE | SWT.BORDER);
        license.addModifyListener(new ModifyListener(){
            public void modifyText( org.eclipse.swt.events.ModifyEvent e ) {
                File file = new File(license.getText());
                if (isInstalled() )
                    acceptance(Policy.bind("DB2Preferences.fileExists"), license); //$NON-NLS-1$
                else if (!file.exists() )
                	error(Policy.bind("DB2Preferences.fileNotFound"), license); //$NON-NLS-1$
                else
                	error(Policy.bind("DB2Preferences.notValid"), license); //$NON-NLS-1$

            };
        });
        String storedValue = getPreferenceStore().getString(LICENSE);
        if (storedValue.trim().length() == 0 ) {
            storedValue = null;
        }
        license.setText(storedValue == null ? LICENSE : storedValue);

        browseLicense = new Button(comp, SWT.PUSH);
        browseLicense.setText(Policy.bind("DB2Preferences.browse")); //$NON-NLS-1$
        browseLicense.addSelectionListener(new SelectionListener(){

            public void widgetSelected( SelectionEvent e ) {
                widgetDefaultSelected(e);
            }

            public void widgetDefaultSelected( SelectionEvent e ) {
                FileDialog fileDialog = new FileDialog(comp.getShell(), SWT.OPEN);
                fileDialog.setFilterExtensions(new String[]{"*.jar"}); //$NON-NLS-1$
                fileDialog.setFilterNames(new String[]{Policy.bind("DB2Preferences.archive")}); //$NON-NLS-1$
                String result = fileDialog.open();
                if (result != null)
                    license.setText(result);
            }

        });

        layout(jdbcDriver, label, license, browseLicense);
    }
    /**
     * Sets the layout of the labels with respect to each other and the above control... if it
     * exists.
     */

    private void layout( Control above, Label label, Text location, Button browse ) {
        // setLayout for label
        int labelWidth = this.convertWidthInCharsToPixels(16);
        int buttonWidth = this.convertWidthInCharsToPixels(9);
        int height = this.convertHeightInCharsToPixels(1) + 4;
        FormData layoutData = new FormData(labelWidth, height);
        if (above != null)
            layoutData.top = new FormAttachment(above);
        else
            layoutData.top = new FormAttachment(0);
        layoutData.left = new FormAttachment(0);
        label.setLayoutData(layoutData);

        // setLayout for text area
        layoutData = new FormData();
        if (above != null)
            layoutData.top = new FormAttachment(above);
        else
            layoutData.top = new FormAttachment(0);

        layoutData.left = new FormAttachment(label, 0, SWT.RIGHT);
        layoutData.right = new FormAttachment(85, 0);
        location.setLayoutData(layoutData);

        // setLayout for browse button
        layoutData = new FormData(buttonWidth, height + 2);
        layoutData.top = new FormAttachment(location, 0, SWT.CENTER);
        layoutData.left = new FormAttachment(location);
        layoutData.right = new FormAttachment(100);

        browse.setLayoutData(layoutData);
    }

    protected void performDefaults() {
        if( jdbcDriver!=null )
            jdbcDriver.setText(JDBC_DRIVER);
        if( license!=null )
            license.setText(LICENSE);
        renameFile(getFile(".flag", true), ".flag", false); //$NON-NLS-1$ //$NON-NLS-2$
        super.performDefaults();

        if (listener != null)
            listener.handleEvent(null);
    }

    public boolean performOk() {
        if (jdbcDriver.getText().trim().length() > 0) {
            File srcDriver = null;
            try {
                srcDriver = new File(jdbcDriver.getText());
                if (!srcDriver.exists()) {
                    if (jdbcDriver.getText().equals(JDBC_DRIVER))
                        return true;
                    error(Policy.bind("DB2Preferences.fileNotFound"), jdbcDriver); //$NON-NLS-1$
                    return false;
                }
            } catch (Exception e) {
                error(Policy.bind("DB2Preferences.copyError"), jdbcDriver); //$NON-NLS-1$
                return false;
            }
            File srcLicense = null;
            try {
                srcLicense = new File(license.getText());
                if (!srcLicense.exists()) {
                    if (license.getText().equals(LICENSE))
                        return true;
                    error(Policy.bind("DB2Preferences.fileNotFound"), license); //$NON-NLS-1$
                    return false;
                }
            } catch (Exception e) {
                error(Policy.bind("DB2Preferences.copyError"), license); //$NON-NLS-1$
                return false;
            }

            try {
                File destDriver = getFile(JDBC_DRIVER, false);
                if (destDriver != null)
                    renameFile(destDriver, "." + destDriver.getName(), false); //$NON-NLS-1$
                destDriver.createNewFile();

                copyfile(srcDriver, destDriver);
            } catch (Exception e) {
                error(Policy.bind("DB2Preferences.23"), jdbcDriver); //$NON-NLS-1$
                return false;
            }
            try {
                File destLicense = getFile(LICENSE, false);
                if (destLicense != null)
                    renameFile(destLicense, "." + destLicense.getName(), false); //$NON-NLS-1$
                destLicense.createNewFile();

                copyfile(srcLicense, destLicense);
            } catch (Exception e) {
                error(Policy.bind("DB2Preferences.copyError"), license); //$NON-NLS-1$
                return false;
            }

            getPreferenceStore().putValue(JDBC_DRIVER, jdbcDriver.getText());
            getPreferenceStore().putValue(LICENSE, license.getText());

            if (listener != null)
                listener.handleEvent(null);

            boolean restart = MessageDialog
                    .openQuestion(
                            getShell(),
                            Policy.bind("DB2Preferences.restartTitle"), Policy.bind("DB2Preferences.restartNeeded") + //$NON-NLS-1$ //$NON-NLS-2$
                                    Policy.bind("DB2Preferences.restartQuestion")); //$NON-NLS-1$
            if (restart) {
                PlatformUI.getWorkbench().restart();
            }
        }

        return true;
    }

    @Override
    public Shell getShell() {
        return shell;
    }

    /**
     * @return
     * @throws IOException
     */
    private static File getFile( String name, boolean backup ) {
        URL url = Platform.getBundle("net.refractions.udig.libs.db2").getEntry("/libs"); //$NON-NLS-1$ //$NON-NLS-2$
        if (url == null)
            return new File("RandomCrazyPlaceholderlkjaljflkasdjfkjlkjasdfiwjkl"); //$NON-NLS-1$
        URL localURL;
        try {
            localURL = Platform.asLocalURL(url);
        } catch (IOException e) {
            return new File("RandomCrazyPlaceholderlkjaljflkasdjfkjlkjasdfiwjkl"); //$NON-NLS-1$
        }
        String prefix = backup ? "." : ""; //$NON-NLS-1$ //$NON-NLS-2$
        File destDriver = new File(localURL.getFile() + prefix + name);
        return destDriver;
    }

    /**
     * @param flag
     */
    private void renameFile( File flag, String newname, boolean deleteOnFail ) {
        if (flag != null) {
            File dest = new File(flag.getParentFile(), newname);

            try {
                if (dest.exists())
                    dest.delete();
                flag.renameTo(dest);
            } catch (Exception e) {
                try {
                    copyfile(flag, dest);
                } catch (IOException e1) {
                    DB2Plugin.log(Policy.bind("DB2Preferences.renameError"), e1); //$NON-NLS-1$
                    if (deleteOnFail) {
                        flag.delete();
                    }
                }
            }
        }
    }

    private void copyfile( File src, File dest ) throws IOException {
        FileChannel srcChannel = null;
        FileChannel dstChannel = null;
        try {
            // Create channel on the source
            srcChannel = new FileInputStream(src).getChannel();

            // Create channel on the destination
            dstChannel = new FileOutputStream(dest).getChannel();

            // Copy file contents from source to destination
            dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
        } finally {
            // Close the channels
            if (srcChannel != null)
                srcChannel.close();
            if (dstChannel != null)
                dstChannel.close();
        }
    }

    private void error( String message, Text field ) {
        setErrorMessage(message); //$NON-NLS-1$
        field.setForeground(getRed());
        field.setToolTipText(message);
    }

    private void acceptance( String message, Text field ) {
        setErrorMessage(null); //$NON-NLS-1$
        field.setForeground(getBlack());
        field.setToolTipText(message);
    }

    private Color getRed() {
        if (red == null)
            red = new Color(getShell().getDisplay(), 255, 0, 0);
        return red;
    }

    private Color getBlack() {
        if (black == null)
            black = new Color(getShell().getDisplay(), 0, 0, 0);
        return black;
    }

    public void dispose() {
        super.dispose();
        if (red != null) {
            red.dispose();
            red = null;
        }
        if (black != null) {
            black.dispose();
            black = null;
        }
    }

    /*
     * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
     */
    public void init( IWorkbench workbench ) {
        // this.workbench = workbench;
    }

    public void setListener( Listener listener ) {
        this.listener = listener;
    }

    @Override
    protected IPreferenceStore doGetPreferenceStore() {
        return PlatformUI.getPreferenceStore();
    }

	public static boolean isInstalled() {
		try{
			Class clazz=Class.forName("com.ibm.db2.jcc.DB2Connection"); //$NON-NLS-1$
			if( clazz!=null)
				return true;
			else
				return false;
		}catch (ClassNotFoundException e){
			return false;
		}
	}

}
