/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.internal.postgis;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.ServiceExtension;

import org.geotools.data.postgis.PostgisDataStoreFactory;

/**
 * Provides ...TODO summary sentence
 * <p>
 * TODO Description
 * </p>
 * @author David Zwiers, Refractions Research
 * @since 0.6
 */
public class PostGISServiceExtension implements ServiceExtension {

    /**
     * 
     * @param id
     * @param params
     * @return x
     */
    public IService createService( URL id, Map<String,Serializable> params ) {
        if(params!=null && params.containsKey(getFactory().getParametersInfo()[2].key) && params.get(getFactory().getParametersInfo()[2].key) instanceof String){
            String val = (String)params.get(getFactory().getParametersInfo()[2].key);
            params.remove(val);
            params.put(getFactory().getParametersInfo()[2].key,new Integer(val));
        }
        if(!getFactory().canProcess(params))
            return null;
        if(id == null){
            String host = (String)params.get(getFactory().getParametersInfo()[1].key);
            Integer port = (Integer)params.get(getFactory().getParametersInfo()[2].key);
            String db = (String)params.get(getFactory().getParametersInfo()[3].key);
            try {
                return new PostGISServiceImpl(new URL("http://"+host+".postgis.jdbc:"+(port.toString())+"/"+db),params); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
            } catch (MalformedURLException e) {
                // log this?
                e.printStackTrace();
                return null;
            } 
        }
        return new PostGISServiceImpl(id,params);
    }

    /**
     * This is a guess ...
     * 
     * @see net.refractions.udig.catalog.ServiceExtension#createParams(java.net.URL)
     * @param url
     * @return x
     */
    public Map<String,Serializable> createParams( URL url ) {
        if (!isPostGIS(url)) {
            return null;
        }
    
        Map<String,Serializable> params = new HashMap<String,Serializable>();
        params.put(getFactory().getParametersInfo()[0].key,"postgis"); // dbtype //$NON-NLS-1$
        String host = url.getHost();
        if(host != null && !"".equals(host)){ //$NON-NLS-1$
            if(host.endsWith("postgis.jdbc")){ //$NON-NLS-1$
                host = host.substring(0,host.length()-12);
            }
        }
        params.put(getFactory().getParametersInfo()[1].key,host); // host
        Integer the_port = url.getPort() == -1 ? new Integer(5432) : new Integer(url.getPort());
        params.put(getFactory().getParametersInfo()[2].key,the_port); // port
        String the_database = url.getPath() == null ? "" : url.getPath(); //$NON-NLS-1$
        params.put(getFactory().getParametersInfo()[3].key,the_database); // database
        String userInfo = url.getUserInfo() == null ? "" : url.getUserInfo(); //$NON-NLS-1$
        params.put(getFactory().getParametersInfo()[4].key,userInfo); // user
        params.put(getFactory().getParametersInfo()[5].key,""); // pass //$NON-NLS-1$
        return params;
    }
    
    private static PostgisDataStoreFactory factory = null;
    /**
     * 
     * @return x
     */
    protected static PostgisDataStoreFactory getFactory(){
        if(factory == null){
            //factory = new PostgisDataStoreFactory();
        	factory = new UDIGPostGISDataStoreFactory();
        }
        return factory;
    }
    
    /** A couple quick checks on the url */ 
    private static final boolean isPostGIS( URL url ){
        return url.toString().contains("postgis.jdbc"); //$NON-NLS-1$
    }
}
