package net.refractions.udig.catalog.internal.postgis;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import org.geotools.data.DataSourceException;
import org.geotools.data.Transaction;
import org.geotools.data.jdbc.ConnectionPool;
import org.geotools.data.jdbc.JDBCDataStoreConfig;
import org.geotools.data.jdbc.JDBCUtils;
import org.geotools.data.postgis.PostgisDataStore;
import org.geotools.data.postgis.PostgisFeatureLocking;
import org.geotools.feature.FeatureType;

/**
 * Subclass of PostgisDataStore which optimizes interaction with underlying 
 * postgis database. 
 * 
 * @author Justin Deoliveira,Refractions Research Inc.,jdeolive@refractions.net
 *
 */
public class UDIGPostGISDataStore extends PostgisDataStore {

	public UDIGPostGISDataStore(ConnectionPool connectionPool, JDBCDataStoreConfig config, int optimizeMode) throws IOException {
		super(connectionPool, config, optimizeMode);
		// TODO Auto-generated constructor stub
	}

	protected UDIGPostGISDataStore(ConnectionPool connPool, String schema, String namespace, int optimizeMode) throws IOException {
		super(connPool, schema, namespace, optimizeMode);
		// TODO Auto-generated constructor stub
	}

	protected UDIGPostGISDataStore(ConnectionPool connPool, String schema, String namespace) throws IOException {
		super(connPool, schema, namespace);
		// TODO Auto-generated constructor stub
	}

	protected UDIGPostGISDataStore(ConnectionPool connPool, String namespace) throws IOException {
		super(connPool, namespace);
		// TODO Auto-generated constructor stub
	}

	protected UDIGPostGISDataStore(ConnectionPool connPool) throws IOException {
		super(connPool);
		// TODO Auto-generated constructor stub
	}
	
	/**
	 * Returns the type names of the database which are located in particular
	 * namespace or schema. Uses the postgres system tables as well as the 
	 * postgis geometry_columns table to perform the typename lookup.
	 * 
	 * @return A list of namespace qualified feature type names of the form
	 * [namespace].[tablename].
	 */
	@Override
	public String[] getTypeNames() throws IOException {
		Connection conn = null;
		
		String namespace = config.getNamespace();
		try {
			conn = getConnection(Transaction.AUTO_COMMIT);
			
			PreparedStatement st = null;
			
			if (namespace != null && !namespace.trim().equals("")) { //$NON-NLS-1$
				st = conn.prepareStatement(
					"SELECT distinct a.relname "  //$NON-NLS-1$
					+ "FROM pg_class a, pg_attribute b, pg_namespace c, pg_type d " //$NON-NLS-1$
				   + "WHERE a.oid = b.attrelid " //$NON-NLS-1$
				   	 + "AND b.atttypid = d.oid "  //$NON-NLS-1$
				   	 + "AND a.relnamespace = c.oid "  //$NON-NLS-1$
				   	 + "AND c.nspname = ? " //$NON-NLS-1$
				   	 + "AND d.typname = ? " //$NON-NLS-1$
				   	 + "AND a.relname in (SELECT f_table_name FROM geometry_columns)" //$NON-NLS-1$
				);
				st.setString(1, namespace);
				st.setString(2, "geometry"); //$NON-NLS-1$
			}
			else {
				st = conn.prepareStatement(
					"SELECT distinct a.relname "  //$NON-NLS-1$
					+ "FROM pg_class a, pg_attribute b, pg_type d " //$NON-NLS-1$
				   + "WHERE a.oid = b.attrelid " //$NON-NLS-1$
				   	 + "AND b.atttypid = d.oid "  //$NON-NLS-1$
				   	 + "AND d.typname = ? " //$NON-NLS-1$
				   	 + "AND a.relname in (SELECT f_table_name FROM geometry_columns)" //$NON-NLS-1$
				);
				st.setString(1, "geometry"); //$NON-NLS-1$
			}
			
			ResultSet rs = st.executeQuery(); 
			ArrayList names = new ArrayList();
			while(rs.next()) {
				String table = rs.getString(1);
				if (allowTable(table)){
					names.add(table);
				}
					
			}
			
			return (String[])names.toArray(new String[names.size()]);
		}
		catch (SQLException sqlException) {
	        JDBCUtils.close(conn, Transaction.AUTO_COMMIT, sqlException);
	        conn = null;
	
	        String message = Policy.bind("UDIGPostGISDataStore.tableQueryError") //$NON-NLS-1$
	            + sqlException.getMessage();
	        throw new DataSourceException(message, sqlException);
	    } 
	    finally {
	        JDBCUtils.close(conn, Transaction.AUTO_COMMIT, null);
	    }
	}

	 @Override
	public PostgisFeatureLocking createFeatureLockingInternal(
		PostgisDataStore ds, FeatureType type
	) throws IOException {
		
		 return new UDIGPostGISFeatureLocking(ds,type);
	}
}