/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.catalog.internal.postgis.ui;

import java.io.IOException;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.IResolve;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.internal.postgis.Policy;
import net.refractions.udig.catalog.internal.postgis.PostGISGeoResource;
import net.refractions.udig.catalog.internal.postgis.PostGISServiceExtension;
import net.refractions.udig.catalog.internal.postgis.PostGISServiceImpl;
import net.refractions.udig.catalog.internal.postgis.PostgisPlugin;
import net.refractions.udig.catalog.ui.IDataWizard;
import net.refractions.udig.catalog.ui.UDIGImportPage;
import net.refractions.udig.catalog.ui.wizard.DataBaseRegistryWizardPage;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.geotools.data.DataStoreFactorySpi;
import org.geotools.data.DataStoreFactorySpi.Param;
import org.geotools.data.postgis.PostgisConnectionFactory;
import org.geotools.data.postgis.PostgisDataStoreFactory;

/**
 * Enter Postgis connection parameters.
 * 
 * @author dzwiers
 * @since 0.3
 */
public class PostGisWizardPage extends DataBaseRegistryWizardPage 
	implements UDIGImportPage {
    
	private static final String POSTGIS_WIZARD = "POSTGIS_WIZARD"; //$NON-NLS-1$
    private static final String POSTGIS_RECENT = "POSTGIS_RECENT"; //$NON-NLS-1$
    private IDialogSettings settings;
    private static final int COMBO_HISTORY_LENGTH = 15;
    ArrayList<DataBaseConnInfo> dbData;
    
    public final String IMAGE_KEY = "PostGisWizardPageImage"; //$NON-NLS-1$

    /** <code>wkb</code> field */
    protected Button wkb = null;
    /** <code>looseBBox</code> field */
    protected Button looseBBox = null;
    
    public PostGisWizardPage() {
        super(Policy.bind("PostGisWizardPage.title")); //$NON-NLS-1$
        settings = PostgisPlugin.getDefault().getDialogSettings().getSection(POSTGIS_WIZARD);
        if (settings == null) {
            settings = PostgisPlugin.getDefault().getDialogSettings().addNewSection(POSTGIS_WIZARD);
        }
    }

    public void createControl( Composite arg0 ) {
        super.createControl(arg0);
        
        
        String[] recentPostGiss = settings.getArray(POSTGIS_RECENT);
        ArrayList<String> hosts = new ArrayList<String>();
        dbData = new ArrayList<DataBaseConnInfo>();
        if (recentPostGiss == null) {
            recentPostGiss = new String[0];
        } else {
            for( String recent : recentPostGiss ) {
                DataBaseConnInfo dbs = new DataBaseConnInfo(recent);
                dbData.add(dbs);
                hosts.add(dbs.getHost());
            }
        }
        if( hosts.size() > 0 ) {
            ((CCombo)host).setItems(hosts.toArray(new String[0]));
            ((CCombo)host).addModifyListener(new ModifyListener(){
                public void modifyText( ModifyEvent e ) {
                    if(e.widget!=null) {
                        for( DataBaseConnInfo db : dbData ) {
                            if(db.getHost().toString().equalsIgnoreCase(((CCombo)host).getText())) {
                                port.setText(db.getPort());
                                user.setText(db.getUser());
                                pass.setText(db.getPass());
                                ((CCombo)database).setText(db.getDb());
                                break;
                            }
                        }
                    }
                }
            });
        }
        
        //For Drag 'n Drop as well as for general selections
        // look for a url as part of the selction
        Map<String,Serializable> params = defaultParams(); // based on selection
        Param[] dbParams = factory.getParametersInfo();
        String selectedHost = (String)params.get(dbParams[1].key);
        
        if( selectedHost != null ){
            ((CCombo)host).setText(params.get(dbParams[1].key).toString());
            port.setText(params.get(dbParams[2].key).toString());
            ((CCombo)database).setText(params.get(dbParams[3].key).toString());
            user.setText(params.get(dbParams[4].key).toString());
            pass.setText(params.get(dbParams[5].key).toString());
        }
    }

    /**
	 * Can process needs to return true *iff* we are sure this is a PostGis object.
	 * <p>
	 * 
	 * </p>
	 */
	public boolean canProcess( Object data ) {
	    return toCapabilitiesURL(data) != null;
	}

	protected Group createAdvancedControl( Composite arg0 ) {
        advanced = new Group(arg0, SWT.SHADOW_NONE);
        advanced.setLayout(new GridLayout(1, false));

        wkb = new Button(advanced, SWT.CHECK);
        wkb.setLayoutData(new GridData(SWT.LEFT, SWT.DEFAULT, false, false));
        wkb.setSelection(false);
        wkb.addSelectionListener(this);
        wkb.setText(Policy.bind("PostGisWizardPage.button.wkb.text")); //$NON-NLS-1$
        wkb.setToolTipText(Policy.bind("PostGisWizardPage.button.wkb.tooltip")); //$NON-NLS-1$
        wkb.setSelection(true);

        looseBBox = new Button(advanced, SWT.CHECK);
        looseBBox.setLayoutData(new GridData(SWT.LEFT, SWT.DEFAULT, false, false));
        looseBBox.setSelection(false);
        looseBBox.addSelectionListener(this);
        looseBBox.setText(Policy.bind("PostGisWizardPage.button.looseBBox.text")); //$NON-NLS-1$
        looseBBox.setToolTipText(Policy.bind("PostGisWizardPage.button.looseBBox.tooltip")); //$NON-NLS-1$
        looseBBox.setSelection(true);

        port.setText("5432"); //$NON-NLS-1$

        return advanced;
    }

    protected DataStoreFactorySpi getDataStoreFactorySpi() {
        return factory;
    }

    Connection realConnection;

	public String getId() {
		return "net.refractions.udig.catalog.ui.postgis"; //$NON-NLS-1$
	}
    
	/** Can be called during createControl */
    protected Map<String,Serializable> defaultParams(){
        IStructuredSelection selection = ((IDataWizard) getWizard()).getSelection();
        return toParams( selection );
    }
    /** Retrieve "best" PostGIS guess of parameters based on provided context */
    protected Map<String,Serializable> toParams( IStructuredSelection context){
    	if( context==null )
    		return Collections.EMPTY_MAP;
        for( Iterator itr = context.iterator(); itr.hasNext(); ) {
            Map<String,Serializable> params = toParams( itr.next() );
            if( !params.isEmpty() ) return params;
        }
        return Collections.EMPTY_MAP;
    }
    /**
     * Get the connection parameters based on the provided context.
     * <p>
     * Context is often data from DnD, note this should accept anything that
     * even thinks it is a url. Compare and contrast with canProcess that must
     * only say true to urls it is sure of.
     * </p>
     * @param context Often from DnD
     * @return Map of connection parameters, or EMPTY_MAP if unknown
     */
    protected Map<String,Serializable> toParams( Object data ){
        if( data instanceof PostGISServiceImpl ){
            PostGISServiceImpl postgis = (PostGISServiceImpl) data;
            return postgis.getConnectionParams();
        }
        URL url = toCapabilitiesURL( data );
        if( url == null ){
            // so we are not sure it is a postgis url
            // lets guess
            url = CatalogPlugin.locateURL(data);
        }
        if( url != null ) {
            // well we have a url - lets try it!            
            List<IResolve> list = CatalogPlugin.getDefault().getLocalCatalog().find( url , null);
            for( IResolve resolve : list ){
                if( resolve instanceof PostGISServiceImpl) {
                    // got a hit!
                    PostGISServiceImpl postgisService = (PostGISServiceImpl) data;
                    return postgisService.getConnectionParams();
                }
                else if (resolve instanceof PostGISGeoResource ){
                    PostGISGeoResource layer = (PostGISGeoResource) resolve;
                    PostGISServiceImpl postgis;
                    try {
                        postgis = (PostGISServiceImpl) layer.parent( null );
                        return postgis.getConnectionParams();
                    } catch (IOException e) {
                        toCapabilitiesURL( layer.getIdentifier() );
                    }                    
                }
            }
            return createParams( url );            
        }        
        return Collections.EMPTY_MAP;
    }
    /** 'Create' params given the provided url, no magic occurs */
    protected Map<String,Serializable> createParams( URL url ){
        PostGISServiceExtension serviceFactory = new PostGISServiceExtension();
        Map params = serviceFactory.createParams( url );
        if( params != null) return params;
        
        Map<String,Serializable> params2 = new HashMap<String,Serializable>();
        Param[] dbParams = factory.getParametersInfo();
        
        params2.put(dbParams[0].key, "postgis"); //$NON-NLS-1$
        params2.put(dbParams[1].key, url.getHost());
        String dbport = ((Integer)url.getPort()).toString();
        try {
            params2.put(dbParams[2].key, new Integer(dbport));
        } catch (NumberFormatException e) {
            params2.put(dbParams[2].key, new Integer(5432));
        }

        String the_database = url.getPath() == null ? "" : url.getPath(); //$NON-NLS-1$
        params2.put(dbParams[3].key,the_database); // database
        String userInfo = url.getUserInfo() == null ? "" : url.getUserInfo(); //$NON-NLS-1$
        params2.put(dbParams[4].key,userInfo); // user
        params2.put(dbParams[5].key,""); // pass //$NON-NLS-1$
        
        return params2;
    }

    /**
     * Convert "data" to a PostGIS url
     * <p>
     * Candidates for conversion are:
     * <ul>
     * <li>URL - from browser DnD
     * <li>PostGISServiceImpl - from catalog DnD
     * <li>IService - from search DnD
     * </ul>
     * </p>
     * <p>
     * No external processing should be required here, it is enough to guess and let
     * the ServiceFactory try a real connect.
     * </p>
     * @param data IService, URL, or something else
     * @return URL considered a possibility for a PostGIS connection, or null
     */
    protected URL toCapabilitiesURL( Object data ) {
        if( data instanceof IResolve ){
            return toCapabilitiesURL( (IResolve) data );
        }
        else if( data instanceof URL ){
            return toCapabilitiesURL( (URL) data );
        }
        else if( data instanceof String ){
            return toCapabilitiesURL( (String) data );
        }
        else if( CatalogPlugin.locateURL(data) != null ){
            return toCapabilitiesURL( CatalogPlugin.locateURL(data) );
        }
        else {
            return null; // no idea what this should be
        }
    }
    protected URL toCapabilitiesURL( IResolve resolve ){
        if( resolve instanceof IService ){
            return toCapabilitiesURL( (IService) resolve );
        }
        return toCapabilitiesURL( resolve.getIdentifier() );        
    }
    protected URL toCapabilitiesURL( IService resolve ){
        if( resolve instanceof PostGISServiceImpl ){
            return toCapabilitiesURL( (PostGISServiceImpl) resolve );
        }
        return toCapabilitiesURL( resolve.getIdentifier() );        
    }
    /** No further QA checks needed - we know this one works */
    protected URL toCapabilitiesURL( PostGISServiceImpl postgis ){
        return postgis.getIdentifier();                
    }
    /** Quick sanity check to see if url is a PostGIS url */
    protected URL toCapabilitiesURL( URL url ){
        if (url == null) return null;

        String path = (url.getPath() != null) ? url.getPath().toLowerCase()
                : null;
        
        String query = (url.getQuery() != null) ? url.getQuery().toLowerCase()
                : null;
        
        String protocol = url.getProtocol() != null ? url.getProtocol().toLowerCase()
                : null;

        if (!"http".equals(protocol) //$NON-NLS-1$
                && !"https".equals(protocol)) { //$NON-NLS-1$ 
            return null;
        }
        if (url.toExternalForm().indexOf("postgis.jdbc") != -1) { //$NON-NLS-1$
            return url;
        }
        return null;
    }
    /** Quick sanity check to see if url is a PostGIS url String */
    protected URL toCapabilitiesURL( String string ){
        if (string == null) return null;

        if( !string.contains("postgis.jdbc") ) { //$NON-NLS-1$
            return null;
        }
        //jdbc.postgresql://host:port/database
        int startindex = string.indexOf("//") + 2; //$NON-NLS-1$
        int hostEnd = string.indexOf(":", startindex); //$NON-NLS-1$
        int portEnd = string.indexOf("/", hostEnd); //$NON-NLS-1$
        
        //int databaseEnd = string.indexOf(" ", databaseStart);
        String the_host = string.substring(startindex, hostEnd);
        String the_port;
        String the_database;
        int databaseStart = 0;
        if( portEnd < 1 ) {
            the_port = string.substring(hostEnd + 1);
            the_database = ""; //$NON-NLS-1$
        } else {
            the_port = string.substring(hostEnd + 1, portEnd);
            databaseStart = string.indexOf("/", portEnd); //$NON-NLS-1$
            the_database = string.substring(portEnd);
        }
        Integer intPort;
        if( !the_port.equalsIgnoreCase("") ) { //$NON-NLS-1$
            intPort = new Integer(the_port);
        } else {
            intPort = new Integer(5432);
        }
        
        //URL(String protocol, String host, int port, String file)
        URL url = null;
        try {
            url = new URL("postgis", the_host, intPort, the_database); //$NON-NLS-1$
        } catch (MalformedURLException e) {
            // TODO Catch e
            PostgisPlugin.log("bad url", e); //$NON-NLS-1$
        }
        return url;
    }
    
    @Override
    protected boolean excludeDB(String db) {
    	return "template0".equals(db) || "template1".equals(db); //$NON-NLS-1$ //$NON-NLS-2$
    }
    
    @Override
    protected boolean excludeSchema(String schema) {
    	return "information_schema".equals(schema) || //$NON-NLS-1$
    		"pg_catalog".equals(schema); //$NON-NLS-1$
    }
    
    protected Connection getConnection() {
        final String portText = port.getText();
        final String hostText = ((CCombo) host).getText();
        final String userText = user.getText();
        final String passText = pass.getText();
        final String olddb = ((CCombo) database).getText();

        if (dirty || realConnection == null) {
            dirty = false;

            try {
                getContainer().run(true, true, 
            		new IRunnableWithProgress(){

                    public void run( IProgressMonitor monitor ) throws InvocationTargetException,
                            InterruptedException {
                        if (realConnection != null)
                            try {
                                realConnection.close();
                            } catch (SQLException e1) {
                                // it's dead anyhow
                            }

                        String db = olddb == null || "".equals(olddb) ? "template1" : olddb; //$NON-NLS-1$ //$NON-NLS-2$

                        PostgisConnectionFactory conFac = new PostgisConnectionFactory(hostText,
                                portText, db);
                        conFac.setLogin(userText, passText);
                        DriverManager.setLoginTimeout(3);
                        try {
                            realConnection = conFac.getConnection();
                            if (realConnection != null) {
                                database.getDisplay().asyncExec(new Runnable(){
                                    public void run() {
                                        database.notifyListeners(SWT.FocusIn, new Event());

                                    }
                                });
                            }
                        } catch (SQLException e) {
                            //throw new InvocationTargetException(e, e.getLocalizedMessage());
                        	throw (InvocationTargetException) new InvocationTargetException(e,e.getLocalizedMessage());
                        }
                    }
                	}
                );
            } 
            catch (InvocationTargetException e2) {
                throw new RuntimeException(e2.getLocalizedMessage(), e2);
            } 
            catch (InterruptedException e2) {
                
            }
        }

        return realConnection;
    }

    public void dispose() {
        super.dispose();
        if (connection != null)
            try {
                connection.close();
            } catch (SQLException e) {
                // it's dead anyhow
            }
    }
    private Connection connection = null;

    private boolean dirty=true;

    @Override
    protected void widgetSelectedInternal(SelectionEvent e) {
    	if (e.widget != null && e.widget.equals(database)) {
    		dirty = true;
    		populateSchema();
    	}
    }
    
    public void modifyText( ModifyEvent e ) {
        if (e.widget != null) {
            if (e.widget.equals(host) || e.widget.equals(port) || e.widget.equals(user)
                    || e.widget.equals(pass)) {
                dirty=true;
                setErrorMessage(null);
            }
        }
    }

    protected boolean isDBCombo() {
        return true;
    }
    
    protected boolean isHostCombo() {
        return true;
    }

    protected boolean hasSchema() {
        return true;
    }

    String getHost() {
        return ((CCombo)host).getText();
    }

    String getPort() {
        return port.getText();
    }

    String getUser() {
        return user.getText();
    }

    String getPass() {
        return pass.getText();
    }

    String getDB() {
        return ((CCombo) database).getText();
    }

    String getSchema() {
    	return schema != null ? schema.getText() : ""; //$NON-NLS-1$
    }
    
    @Override
    public boolean isPageComplete() {
        return factory.canProcess(getParams());
    }

    /*
     * @see net.refractions.udig.catalog.ui.UDIGImportPage#getResources(org.eclipse.core.runtime.IProgressMonitor)
     */
    public List<IResolve> getResources( IProgressMonitor monitor ) throws Exception {
        if (!isPageComplete())
            return null;

//        if (monitor != null) {
//        	String msg = Policy.bind("PostGisWizardPage.loadingServiceMessage");
//        	monitor.subTask(msg);
//        }
        
        PostGISServiceExtension creator = new PostGISServiceExtension();
        IService service = creator.createService(null, getParams());
        service.getInfo(monitor); // load

        List<IResolve> servers = new ArrayList<IResolve>();
        servers.add(service);
        
        /*
         * Success! Store the URL in history.
         */
        saveWidgetValues();
        
        return servers;
    }

    private static PostgisDataStoreFactory factory = new PostgisDataStoreFactory();

    /**
     * Returns the parameters
     */
    public Map<String, Serializable> getParams() {
        Map<String, Serializable> params = new HashMap<String, Serializable>();
        Param[] dbParams = factory.getParametersInfo();
        params.put(dbParams[0].key, "postgis"); //$NON-NLS-1$
        params.put(dbParams[1].key, getHost());
        String dbport = getPort();
        try {
            params.put(dbParams[2].key, new Integer(dbport));
        } catch (NumberFormatException e) {
            params.put(dbParams[2].key, new Integer(5432));
        }
        params.put(dbParams[3].key, getSchema()); //$NON-NLS-1$
        String db = getDB();
        params.put(dbParams[4].key, db);

        String userName = getUser();
        params.put(dbParams[5].key, userName);
        String password = getPass();
        params.put(dbParams[6].key, password);

        if (wkb.getSelection())
            params.put(dbParams[7].key, Boolean.TRUE);
        if (looseBBox.getSelection())
            params.put(dbParams[8].key, Boolean.TRUE);

        
        return params;
    }

    /**
     * Saves the widget values
     */
    private void saveWidgetValues() {
        // Update history
        if (settings != null) {
            String[] recentPOSTGISs = settings.getArray(POSTGIS_RECENT);
            if (recentPOSTGISs == null) {
                recentPOSTGISs = new String[0];
            }
            String dbs = new DataBaseConnInfo(((CCombo)host).getText(), port.getText(), 
                    user.getText(), pass.getText(), ((CCombo)database).getText()).toString();
            recentPOSTGISs = addToHistory(recentPOSTGISs, dbs);
            settings.put(POSTGIS_RECENT, recentPOSTGISs);
        }
    }
    
    /**
     * Adds an entry to a history, while taking care of duplicate history items
     * and excessively long histories.  The assumption is made that all histories
     * should be of length <code>COMBO_HISTORY_LENGTH</code>.
     *
     * @param history the current history
     * @param newEntry the entry to add to the history
     * @return the history with the new entry appended
     * Stolen from org.eclipse.team.internal.ccvs.ui.wizards.ConfigurationWizardMainPage
     */
    private String[] addToHistory(String[] history, String newEntry) {
        ArrayList<String> l = new ArrayList<String>(Arrays.asList(history));
        addToHistory(l, newEntry);
        String[] r = new String[l.size()];
        l.toArray(r);
        return r;
    }
    
    /**
     * Adds an entry to a history, while taking care of duplicate history items
     * and excessively long histories.  The assumption is made that all histories
     * should be of length <code>COMBO_HISTORY_LENGTH</code>.
     *
     * @param history the current history
     * @param newEntry the entry to add to the history
     * Stolen from org.eclipse.team.internal.ccvs.ui.wizards.ConfigurationWizardMainPage
     */
    private void addToHistory(List<String> history, String newEntry) {
        history.remove(newEntry);
        history.add(0,newEntry);
    
        // since only one new item was added, we can be over the limit
        // by at most one item
        if (history.size() > COMBO_HISTORY_LENGTH)
            history.remove(COMBO_HISTORY_LENGTH);
    }
}
