/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.internal.wfs;

import java.io.Serializable;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.ServiceExtension;
import net.refractions.udig.catalog.internal.wfs.UDIGWFSDataStoreFactory.UDIGWFSDataStore;

import org.geotools.data.wfs.WFSDataStoreFactory;

/**
 * Provides ...TODO summary sentence
 * <p>
 * TODO Description
 * </p>
 * @author David Zwiers, Refractions Research
 * @since 0.6
 */
public class WFSServiceExtension implements ServiceExtension {

    private static WFSDataStoreFactory wfsDSFactory;
    /**
     * TODO summary sentence for getWFSDSFactory ...
     * 
     * @return x
     */
    public static WFSDataStoreFactory getWFSDSFactory(){
        if(wfsDSFactory == null)
            wfsDSFactory = new WFSDataStoreFactory();
        return wfsDSFactory;
    }
    
    /**
     * TODO summary sentence for createService ...
     * 
     * @param id
     * @param params
     * @return x
     */
    public IService createService( URL id, Map<String,Serializable> params ) {
        if(params == null || !params.containsKey(WFSDataStoreFactory.URL.key))
            return null;
        if(id == null){
            URL base = (URL)params.get(WFSDataStoreFactory.URL.key);
            base = base == null?null:UDIGWFSDataStore.createGetCapabilitiesRequest(base);
            return new WFSServiceImpl(base,params);
        }
        return new WFSServiceImpl(id,params);
    }

    /**
     * TODO summary sentence for createParams ...
     * 
     * @see net.refractions.udig.catalog.ServiceExtension#createParams(java.net.URL)
     * @param url
     * @return x
     */
    public Map<String,Serializable> createParams( URL url ) {
        if (!isWFS(url)) {
            return null;
        }
        
        // wfs check
        Map<String,Serializable> params = new HashMap<String,Serializable>();
        params.put(WFSDataStoreFactory.URL.key,url);
        params.put( WFSDataStoreFactory.TIMEOUT.key, 3000);
        
        // don't check ... it blocks
        // (XXX: but we are using that to figure out if the service will work?)
        return params;
    }
    
    /** A couple quick checks on the url */ 
    private static final boolean isWFS( URL url ){
        String PATH = url.getPath();
        String QUERY = url.getQuery();
        String PROTOCOL = url.getProtocol();
        
        if( !"http".equals(PROTOCOL)){ //$NON-NLS-1$
            return false;
        }
        if( QUERY != null && QUERY.toUpperCase().indexOf( "SERVICE=" ) != -1){ //$NON-NLS-1$
            // we have a service! it better be wfs            
            return QUERY.toUpperCase().indexOf( "SERVICE=WFS") != -1; //$NON-NLS-1$
        }
        if( PATH != null && PATH.toUpperCase().indexOf( "GEOSERVER/WFS" ) != -1){ //$NON-NLS-1$
            return true;
        }  
        return true; // try it anyway
    }

}
