/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.internal.wms;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.ServiceExtension;

/**
 * Provides ...TODO summary sentence
 * <p>
 * TODO Description
 * </p>
 * @author David Zwiers, Refractions Research
 * @since 0.6
 */
public class WMSServiceExtension implements ServiceExtension {

    /**
     * TODO summary sentence for createService ...
     * 
     * @param id
     * @param params
     * @return x
     */
    public IService createService( URL id, Map<String,Serializable> params ) {
        if(params == null)
            return null;
        
        if ((!params.containsKey(WMSServiceImpl.WMS_URL_KEY) && id == null) &&
        	 !params.containsKey(WMSServiceImpl.WMS_WMS_KEY)) {
        	return null; // nope we don't have a WMS_URL_KEY
        }
        
        if(params.containsKey(WMSServiceImpl.WMS_URL_KEY)){
            URL base = null; // base url for service
			
            if(params.get(WMSServiceImpl.WMS_URL_KEY) instanceof URL){
                base = (URL)params.get(WMSServiceImpl.WMS_URL_KEY); // use provided url for base
            }
            else {
                try {
                    base = new URL((String)params.get(WMSServiceImpl.WMS_URL_KEY)); // upcoverting string to url for base
                } catch (MalformedURLException e1) {
                    // log this?
                    e1.printStackTrace();
                    return null;
                }
                params.remove(params.get(WMSServiceImpl.WMS_URL_KEY));
                params.put(WMSServiceImpl.WMS_URL_KEY,base);
            }
            // params now has a valid url
            
            if(id == null){
                id = base;
//                try {
//					// try to ensure that the id includes at least a SERVICE=WMS in query
//					if( base.getQuery() == null || base.getQuery().trim().length()==0 ){
//						base = new URL(base,"?SERVICE=WMS"); // append SERVICE=WMS :-)	 //$NON-NLS-1$
//					}
//					else {
//						// we have a query, ensure it has SERVICE=WMS
//						if( base.getQuery().toUpperCase().contains("SERVICE=WMS") ){ //$NON-NLS-1$
//							// NOP: base already is good		
//						} else {
//							base = new URL(base,"?"+base.getQuery()+"&SERVICE=WMS");  //$NON-NLS-1$//$NON-NLS-2$
//						}
//					}                    
//                } catch (MalformedURLException e) {
//                    WmsPlugin.log( null,e);
//                }
//                return new WMSServiceImpl(base,params);
            }
            return new WMSServiceImpl(id,params);

        }
        return null;
    }

    /**
     * TODO summary sentence for createParams ...
     * 
     * @see net.refractions.udig.catalog.ServiceExtension#createParams(java.net.URL)
     * @param url
     * @return
     */
    public Map<String,Serializable> createParams( URL url ) {
        if( !isWMS(url)) {
            return null;
        }
        
        // wms check
        Map<String,Serializable> params2 = new HashMap<String,Serializable>();
        params2.put(WMSServiceImpl.WMS_URL_KEY,url);
        
//        TODO this strikes me as a bad place to check for WMS. Also note the wasted IO as the 
//        result is not saved
        
//        try {
//            new WebMapServer(url);
            return params2;
//        } catch (IOException e) {
//            // didn't work
//            // log this?
//        }
//        return null;
    }

    public static boolean isWMS( URL url ){
        if (url == null) {
            return false;
        }
        
        String PATH = url.getPath();
        String QUERY = url.getQuery();
        String PROTOCOL = url.getProtocol();
        if( PROTOCOL.indexOf("http") == -1 ){ //$NON-NLS-1$ supports 'https' too.
            return false;
        }
        if( QUERY != null && QUERY.toUpperCase().indexOf( "SERVICE=" ) != -1){ //$NON-NLS-1$
            // we have a service! It better be WMS            
            return QUERY.toUpperCase().indexOf( "SERVICE=WMS") != -1; //$NON-NLS-1$
        }
        else if( PATH != null && PATH.toUpperCase().indexOf( "GEOSERVER/WMS" ) != -1){ //$NON-NLS-1$
            return true;
        }                
        return true; // try it anyway
    }
}
