package net.refractions.udig.catalog.memory.internal;

import java.io.IOException;
import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IResolveChangeEvent;
import net.refractions.udig.catalog.IResolveDelta;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.IServiceInfo;
import net.refractions.udig.catalog.internal.CatalogImpl;
import net.refractions.udig.catalog.internal.ResolveChangeEvent;
import net.refractions.udig.catalog.internal.ResolveDelta;
import net.refractions.udig.catalog.memory.MemoryService;

import org.eclipse.core.runtime.IProgressMonitor;


public class MemoryServiceImpl extends IService {

	/** info object **/
    private ScratchServiceInfo info;
	
	/** the data store **/
	private AnotherMemoryDataStore ds;
	
    private List<MemoryGeoResourceImpl> memberList;
    
	public MemoryServiceImpl() {
	}
	
	@Override
	public Map<String, Serializable> getConnectionParams() {
		return new MemoryServiceExtensionImpl().createParams(getIdentifier());
	}

	public <T> boolean canResolve(Class<T> adaptee) {
		if (adaptee == null) return false;
		
		return adaptee.isAssignableFrom(IServiceInfo.class) 
                || adaptee.isAssignableFrom(List.class) 
                || adaptee.isAssignableFrom(AnotherMemoryDataStore.class);
	}
	
	@Override
	public <T> T resolve(Class<T> adaptee, IProgressMonitor monitor) throws IOException {
		  if (adaptee == null) return null;
	        
	        if (adaptee.isAssignableFrom(IServiceInfo.class)) {
	            if (info == null)
	                info = new ScratchServiceInfo();
	            
	            return adaptee.cast(info);
	        }
	        if (adaptee.isAssignableFrom(List.class)) {
	           return adaptee.cast(members(monitor)); 
	        }
	        if (adaptee.isAssignableFrom(AnotherMemoryDataStore.class)) {
				return adaptee.cast(getDS());
	        }
			
	        return null;
	}
	
	@Override
	public List<? extends IGeoResource> members(IProgressMonitor monitor) throws IOException {
	    if(this.memberList == null) {
	        synchronized(getDS()){
	            if(this.memberList == null) {
	                AnotherMemoryDataStore ds = getDS();
	                String[] types = ds.getTypeNames();
	                
	                this.memberList = new ArrayList<MemoryGeoResourceImpl>();
	                for (int i = 0; i < types.length; i++) {
	                    this.memberList.add(new MemoryGeoResourceImpl(types[i],this));
	                }
	            }
	        }
	    }
	    return this.memberList;
	}
	
	public Status getStatus() {
		return Status.CONNECTED;
	}

	public Throwable getMessage() {
		return null;
	}

	public URL getIdentifier() {
		return MemoryService.URL;
	}
	
	AnotherMemoryDataStore getDS() {
		if (ds == null) {
			synchronized(AnotherMemoryDataStore.class){
                if(ds == null){
                    ds = new AnotherMemoryDataStore();
                    ds.addListener(new MemoryServiceListener() {
                        public void schemaChanged() {
                            MemoryServiceImpl.this.memberList = null;
                        }
                    });
                }
            }
            IResolveDelta delta = new ResolveDelta( this, IResolveDelta.Kind.CHANGED );
            ((CatalogImpl)CatalogPlugin.getDefault().getLocalCatalog()).fire( new ResolveChangeEvent( this, IResolveChangeEvent.Type.POST_CHANGE, delta )  );
		}
		
		return ds;
	}
	
	class ScratchServiceInfo extends IServiceInfo {
		/*
	     * @see net.refractions.udig.catalog.IServiceInfo#getTitle()
	     */
        public String getTitle() {
            return Policy.bind("catalog.memory.service.title"); //$NON-NLS-1$
        }
        
        @Override
        public String getDescription() {
            return getTitle();
        }
		
	}
}