/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.ui;

import net.refractions.udig.catalog.CatalogPlugin;

import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

/**
 * Provides Tree view of the Registry.
 * <p>
 * Supports the following:
 * <ul>
 * <li>List
 * <li>IService
 * <li>IGeoReference
 * <li>ICatalog
 * <li>String
 * </ul>
 * </p>
 * <p>
 * To display a message or status please use a Collections.singletonList( "hello world" ).
 * </p>
 * @author jeichar
 * @since 0.3
 */
public class CatalogTreeViewer extends TreeViewer {
    /**
     * Construct <code>CatalogTreeViewer</code>.
     *
     * @param parent
     */
    public CatalogTreeViewer( Composite parent ) {
        this(parent, SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER ); // no border by default                
    }
    
    /**
     * Construct <code>CatalogTreeViewer</code>.
     * <p>
     * You will need to set your input:<pre><code>
     * CatalogTreeViewer viewer = new CatalogTreeViewer( parent, SWT.DEFAULT );
     * viewer.setInput( CatalogPlugin.getDefault().getLocalCatalog() );
     * </code></pre>
     * </p>
     * @param parent Parent component
     * @param style The other constructor uses SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER
     */
    public CatalogTreeViewer( Composite parent, int style ) {
        super(parent, style );        
        setContentProvider( new ResolveContentProvider() );                
        setLabelProvider( new ResolveLabelProvider() );
        setInput( CatalogPlugin.getDefault().getLocalCatalog() );
    }
}