package net.refractions.udig.catalog.ui;

import java.util.List;

import net.refractions.udig.catalog.internal.ui.ConnectionPage;
import net.refractions.udig.catalog.internal.ui.Policy;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

/**
 * A wizard page which displays a set of IConnectionErrorHandler objects.
 * 
 * <p>
 * This page is intended for use in a wizard extending 
 * @see net.refractions.udig.catalog.ui.IDataWizard which uses an 
 * @see net.refractions.udig.catalog.ui.UDIGImportPage to connect to a service.
 * </p>
 * 
 * @author Justin Deoliveira
 *
 */
public class ConnectionErrorPage extends WizardPage {

	/** list of handlers **/
	List<IConnectionErrorHandler> handlers;
	
	/** the current handler **/
	IConnectionErrorHandler handler;
	
	public ConnectionErrorPage(List<IConnectionErrorHandler> handlers) {
		super(Policy.bind("ConnectionErrorPage.pageName")); //$NON-NLS-1$
	
		this.handlers = handlers;
		
		setDescription(Policy.bind("ConnectionErrorPage.pageDescription")); //$NON-NLS-1$
		setTitle(Policy.bind("ConnectionErrorPage.pageTitle")); //$NON-NLS-1$
	}

    @Override
    public boolean isPageComplete() {
        return handler.isComplete();
    }
    
	@Override
	public boolean canFlipToNextPage() {
		if (handler != null)
			return handler.canRecover();
		
		return false;
	}
	
	public void createControl(Composite parent) {
		
		Composite root = new Composite(parent, SWT.NONE);
		root.setLayout(new GridLayout(1, false));
		
		handler = handlers.get(0);
		handler.createControl(root);
		
		GridData data 
			= new GridData();
		data.widthHint = getContainer().getShell().getSize().x-100;
		data.heightHint = getContainer().getShell().getSize().y-250;
		
		handler.getControl().setLayoutData(data);
		setControl(root);
	}
	
    @Override
    public IWizardPage getNextPage() {
        if (handler.canRecover() && handler.isComplete()) {
            ConnectionPage page = (ConnectionPage)getPreviousPage();
            return page.getNextPage();
        }
        
        return super.getNextPage();
    }
}