/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.ui;

import java.util.Map;

/**
 * Define user interface needs for a for an opperation.
 * <p>
 * This represents a suitable compromise that can be maintained both
 * by uDig and Toolkit providers.
 * </p><p>
 * Responsibilities:
 * <ul>
 * <li>Name
 * <li>Description
 * <li>Params
 * </ul>
 * </p>
 * @author jgarnett
 * @since 0.3
 */
public interface IOp {
    /** <code>EXTENSION_POINT</code> field */
    public static final String EXTENSION_POINT="net.refractions.udig.registry.ui.featureType";  //$NON-NLS-1$
    
    /**
     * Returns a unique identifier for this operation, or <code>null</code> if it has
     * none.
     *
     * @return the operation id, or <code>null</code> if none
     */
    public String getId();

    /**
     * Returns the name of this operation.
     * 
     * @return the name, or <code>null</code> if none
     */
    public String getName();
    
    /**
     * Returns the operations description if it has one.
     * @return the description, or <code>null</code> if none
     */
    public String getDescription();
    
    /**
     * Return (default) parameters to an end-user to customize.
     * <p>
     * The end-user should have a chance to look at and edit these values before
     * run() is called.
     * </p>
     * @return the params map, or <code>null</code> if none
     */
    public Map getParams();
    
    /**
     * Runs this opperation.
     * <p>
     * Each opperation implementation must define the steps needed to carry
     * out its function.
     * <p>
     * It is expected that implementations demand specific pre and
     * post conditions (often in the style of an AbstractSuperclass implementing
     * run(), and providing an op( ... ) method for subclasses to implement.
     * </p>
     */
    //public void run();
  
}