/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.ui;

import java.io.IOException;

import net.refractions.udig.catalog.IResolve;
import net.refractions.udig.ui.ImageCache;

import org.eclipse.jface.util.ListenerList;
import org.eclipse.jface.viewers.ILabelDecorator;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;

/**
 * Decorate labels with actual titles from the info objects.
 * 
 * @author jgarnett
 * @since 0.6.0
 */
public class ResolveTitlesDecorator implements ILabelDecorator {
	
    ListenerList listeners = new ListenerList();
    
    //static GlyphBuilder glyphBuilder = new GlyphBuilder();

    static ImageCache cache = new ImageCache();
    
    /**
     * Construct <code>ResolveDecorator</code>.
     *
     */
    public ResolveTitlesDecorator() {
        // start thread later
    }
    
    /*
     * @see org.eclipse.jface.viewers.ILabelDecorator#decorateImage(org.eclipse.swt.graphics.Image, java.lang.Object)
     */
    public Image decorateImage( Image image, Object element ) {
        return null;
    }

    /*
     * @see org.eclipse.jface.viewers.ILabelDecorator#decorateText(java.lang.String, java.lang.Object)
     */
    public String decorateText( String text, Object element ) {
        try { 
            IResolve resolve = (IResolve) element;
            return CatalogUIPlugin.title( resolve );            
        }
        catch (IOException e) {                
            return null; // TODO: Log EARNING when info is unavailable?
        }
        catch (Throwable t ){
            return null; // TODO: Log SEVER decorator would otherwise hang ui
        }        
    }        
    
    /**
     * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
     */
    public void dispose() {
        // should clean up after hack
        listeners.clear();
        cache.dispose();
    }

    /**
     * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object,
     *      java.lang.String)
     */
    public boolean isLabelProperty( Object element, String property ) {
        return true; 
    }

    /**
     * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
     */
    public void removeListener( ILabelProviderListener listener ) {
        listeners.remove(listener);
    }

    /*
     * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
     */
    public void addListener( ILabelProviderListener listener ) {
        listeners.add( listener );
    }
}
