/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.ui;

import java.util.List;

import net.refractions.udig.catalog.IResolve;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.wizard.IWizardPage;

/**
 * A udig import wizard page is responsible for providing the connection
 * information to a specific data store. 
 * 
 *<p> 
 * An import page may dynamically contribute pages to the containing wizard 
 * by returning a page from getNextPage().  
 *</p>
 *
 * @author jeichar
 * @since 0.9.0
 */
public interface UDIGImportPage extends IWizardPage {
    
	/** extension point id **/
	public static final String XPID = 
		"net.refractions.udig.catalog.ui.dataWizards"; //$NON-NLS-1$
	
	/**
	 * Returns a processed and useful list of IResolves (all known implementations
     * use one or more IService).
     * <p>
     * Called by next/finish, if exception is thrown message will be set, and
     * we will remain on this page.
     * </p>
	 * @return a processed and useful list of URLs and java.util.Map object.
	 */
	List<IResolve> getResources( IProgressMonitor monitor ) 
		throws Exception;
	
	/**
	 * Determines if an object can be processed by the wizard page to obtain 
	 * a set of resources.`
	 * 
	 * @param object The object to be "processed" or "adapted" into a set of 
	 * resources.
	 * 
	 * @return true if the object can be processed, otherwise false. 
	 */
	boolean canProcess(Object object);
	
	/**
	 * Returns the id of the wizard page. This id must match the id declared
	 * in the associated extension.
	 * 
	 * @return The id of the page.
	 */
	String getId();
	
}
