package net.refractions.udig.catalog.ui;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;

/**
 * A descriptor for an extension of net.refractions.udig.
 * @author jdeolive
 *
 */
public class UDIGImportPageDescriptor {
	IConfigurationElement element;
	
	public UDIGImportPageDescriptor(IConfigurationElement element) {
		this.element = element;
	}
	
	public UDIGImportPage createImportPage() throws CoreException {
		UDIGImportPage page = 
			(UDIGImportPage) element.createExecutableExtension("class");  //$NON-NLS-1$
		
		page.setTitle(getLabel());
		page.setDescription(getDescription());
		page.setImageDescriptor(getDescriptionImage());
		return page;
	}

	public String getLabel() {
		return element.getAttribute("name"); //$NON-NLS-1$
	}
	
	public String getDescription() {
		String desc = element.getAttribute("description"); //$NON-NLS-1$
		if (desc == null)
			return ""; //$NON-NLS-1$
		
		return desc.trim();
	}
	
	public String getId() {
		return element.getAttribute("id"); //$NON-NLS-1$
	}
	
	public ImageDescriptor getImage() {
		String ns = element.getNamespace();
		String banner = element.getAttribute("icon"); //$NON-NLS-1$
		
		if (banner == null)
			return null;
		
		return AbstractUIPlugin.imageDescriptorFromPlugin(ns,banner); //$NON-NLS-1$
	}
	
	public ImageDescriptor getDescriptionImage() {
		String ns = element.getNamespace();
		String banner = element.getAttribute("banner"); //$NON-NLS-1$
		
		if (banner == null)
			return null;
		
		return AbstractUIPlugin.imageDescriptorFromPlugin(ns,banner); //$NON-NLS-1$
	}
	
	public String getServiceType() {
		return element.getAttribute("type"); //$NON-NLS-1$
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj) 
			return true;
		
		if (obj instanceof UDIGImportPageDescriptor) {
			UDIGImportPageDescriptor descriptor = 
				(UDIGImportPageDescriptor)obj;
			
			return getId() != null && 
				getId().equals(descriptor.getId());
		}
		
		return false;
	}
	
	@Override
	public int hashCode() {
		if (getId() == null) 
			return "".hashCode(); //$NON-NLS-1$
		return getId().hashCode();
	}
}