/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.ui.wizard;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;

import net.refractions.udig.catalog.internal.ui.Policy;
import net.refractions.udig.catalog.ui.CatalogUIPlugin;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

/**
 * David's magic superclass, seems to delegate to createAdvancedControl for anything cool.
 * 
 * @author dzwiers
 * @since 0.3
 */
public abstract class DataBaseRegistryWizardPage extends DataStoreWizardPage 
	implements ModifyListener, SelectionListener, FocusListener{
    
    private boolean focusing;
	public DataBaseRegistryWizardPage(String name){
        super(name);
    }
    public DataBaseRegistryWizardPage(){
        super(""); //$NON-NLS-1$
    }
    
    
    /**
     * TODO summary sentence for focusGained ...
     * 
     * @see org.eclipse.swt.events.FocusListener#focusGained(org.eclipse.swt.events.FocusEvent)
     * @param e
     */
    public void focusGained( FocusEvent e ) {
    	//set this flag to prevent further connections upon
    	// another focus
    	if (focusing)
    		return;
    	
    	focusing = true;
        if(e.widget!=null){
            if(e.widget.equals(database)){
                if(isDBCombo())
                    populateDB();
        	}
        
        	if(hasSchema() && e.widget.equals(schema)){
            	populateSchema();
        	}
        }
        getWizard().getContainer().updateButtons();
        focusing = false;
    }
    /**
     * TODO summary sentence for focusLost ...
     * 
     * @see org.eclipse.swt.events.FocusListener#focusLost(org.eclipse.swt.events.FocusEvent)
     * @param e
     */
    public void focusLost( FocusEvent e ) {
        // do nothing
        getWizard().getContainer().updateButtons();
    }
//    protected Text host = null;
    protected Control host = null;
    protected Text port = null;
    protected Text user = null;
    protected Text pass = null;
    protected Control database = null;
    protected CCombo schema = null;
    protected Button advancedKey = null;
    protected Group advanced = null;
	protected abstract boolean isDBCombo();
    protected abstract boolean isHostCombo();
    protected abstract boolean hasSchema();
    String hostStr = null;
    String portStr = null;
    String userStr = null;
    String passStr = null;
    String databaseStr = null;
    String schemaStr = null;

    /**
     * TODO summary sentence for createControl ...
     * 
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     * @param parent
     */
    public void createControl( Composite arg0 ) {
        Composite composite = new Group(arg0,SWT.NULL);
        composite.setLayout(new GridLayout(5, true));
        GridData data = new GridData(SWT.FILL, SWT.FILL, false,false);
        data.widthHint=arg0.getClientArea().width;
        composite.setLayoutData(data);
        
        // add host
        Label label = new Label( composite, SWT.NONE );
        label.setText(Policy.bind("DataBaseRegistryWizardPage.label.host.text") ); //$NON-NLS-1$
        label.setToolTipText( Policy.bind("DataBaseRegistryWizardPage.label.host.tooltip") ); //$NON-NLS-1$
        label.setLayoutData( new GridData(SWT.END, SWT.DEFAULT, false, false ) );
        
        if(isHostCombo()){
            CCombo host = new CCombo(composite,SWT.DROP_DOWN | SWT.READ_ONLY | SWT.BORDER);
            this.host = host;
            host.setLayoutData( new GridData(SWT.FILL, SWT.DEFAULT, true, false ,2,1) );
            host.addModifyListener(this);
            host.setToolTipText( Policy.bind("DataBaseRegistryWizardPage.label.host.tooltip") ); //$NON-NLS-1$
            host.setEditable(true);
        }else{
            Text host = new Text(composite,SWT.BORDER | SWT.SINGLE);
            this.host = host;
            host.setLayoutData( new GridData(SWT.FILL, SWT.DEFAULT, true, false ,2,1) );
            host.addModifyListener(this);
            host.setToolTipText( Policy.bind("DataBaseRegistryWizardPage.label.host.tooltip") ); //$NON-NLS-1$
        }
//        host = new Text( composite, SWT.BORDER | SWT.SINGLE );
//        host.setLayoutData( new GridData(GridData.FILL, SWT.DEFAULT, true, false ,2,1) );
//        host.addModifyListener(this);

        // add port
        label = new Label( composite, SWT.NONE );
        label.setText(Policy.bind("DataBaseRegistryWizardPage.label.port.text") ); //$NON-NLS-1$
        label.setToolTipText( Policy.bind("DataBaseRegistryWizardPage.label.port.tooltip") ); //$NON-NLS-1$
        label.setLayoutData( new GridData(SWT.END, SWT.DEFAULT, false, false ) );
        
        port = new Text( composite, SWT.BORDER | SWT.SINGLE );
        port.setLayoutData( new GridData(GridData.FILL, SWT.DEFAULT,true,false) );
        port.addModifyListener(this);
        port.setTextLimit(6);
        
        // add user
        label = new Label( composite, SWT.NONE );
        label.setText(Policy.bind("DataBaseRegistryWizardPage.label.username.text") ); //$NON-NLS-1$
        label.setToolTipText( Policy.bind("DataBaseRegistryWizardPage.label.username.tooltip") ); //$NON-NLS-1$
        label.setLayoutData( new GridData(SWT.END, SWT.DEFAULT, false, false ) );
        
        user = new Text( composite, SWT.BORDER | SWT.SINGLE );
        user.setLayoutData( new GridData(GridData.FILL, SWT.DEFAULT, true, false,2,1) );
        user.addModifyListener(this);
        
        //space
        label = new Label( composite, SWT.NONE );
        label.setLayoutData( new GridData(SWT.END, SWT.DEFAULT, false, false ,2,1) );

        // add pass
        label = new Label( composite, SWT.NONE );
        label.setText(Policy.bind("DataBaseRegistryWizardPage.label.password.text") ); //$NON-NLS-1$
        label.setToolTipText( Policy.bind("DataBaseRegistryWizardPage.label.password.tooltip") ); //$NON-NLS-1$
        label.setLayoutData( new GridData(SWT.END, SWT.DEFAULT, false, false ) );
        
        pass = new Text( composite, SWT.BORDER | SWT.SINGLE | SWT.PASSWORD);
        pass.setLayoutData( new GridData(GridData.FILL, SWT.DEFAULT, true, false,2,1) );
        pass.addModifyListener(this);
        
        //space
        label = new Label( composite, SWT.NONE );
        label.setLayoutData( new GridData(SWT.END, SWT.DEFAULT, false, false ,2,1) );
        
        // add spacer
        label = new Label( composite, SWT.SEPARATOR | SWT.HORIZONTAL );
        label.setLayoutData( new GridData(SWT.FILL, SWT.CENTER, true, false,5,3 ) );
        
        // database
        label = new Label( composite, SWT.NONE );
        label.setText(Policy.bind("DataBaseRegistryWizardPage.label.database.text") ); //$NON-NLS-1$
        label.setToolTipText( Policy.bind("DataBaseRegistryWizardPage.label.database.tooltip") ); //$NON-NLS-1$
        label.setLayoutData( new GridData(SWT.END, SWT.DEFAULT, false, false ) );
        
        if(isDBCombo()){
            CCombo db = new CCombo(composite,SWT.DROP_DOWN | SWT.READ_ONLY | SWT.BORDER);
        	this.database = db;
        	db.setLayoutData( new GridData(SWT.FILL, SWT.DEFAULT, true, false ,2,1) );
        	db.addModifyListener(this);
        	db.addFocusListener(this);
        	db.addSelectionListener(this);
        	db.setToolTipText( Policy.bind("DataBaseRegistryWizardPage.label.database.tooltip") ); //$NON-NLS-1$
        	db.setEditable(true);
        }else{
            Text db = new Text(composite,SWT.BORDER | SWT.SINGLE);
        	this.database = db;
        	db.setLayoutData( new GridData(SWT.FILL, SWT.DEFAULT, true, false ,2,1) );
        	db.addModifyListener(this);
        	db.addFocusListener(this);
        	db.setToolTipText( Policy.bind("DataBaseRegistryWizardPage.label.database.tooltip") ); //$NON-NLS-1$
        }
        
        label = new Label( composite, SWT.NONE );
        label.setLayoutData( new GridData(SWT.END, SWT.DEFAULT, false, false ,2,1) );

        if(hasSchema()){
            // schema
            label = new Label( composite, SWT.NONE );
            label.setText(Policy.bind("DataBaseRegistryWizardPage.label.schema.text") ); //$NON-NLS-1$
            label.setToolTipText( Policy.bind("DataBaseRegistryWizardPage.label.schema.tooltip") ); //$NON-NLS-1$
            label.setLayoutData( new GridData(SWT.END, SWT.DEFAULT, false, false ) );
            
            schema = new CCombo(composite,SWT.DROP_DOWN | SWT.READ_ONLY | SWT.BORDER);
            schema.setLayoutData( new GridData(SWT.FILL, SWT.DEFAULT, true, false ,2,1) );
            schema.addModifyListener(this);
            schema.addFocusListener(this);
            schema.setToolTipText( Policy.bind("DataBaseRegistryWizardPage.label.schema.tooltip") ); //$NON-NLS-1$
            schema.setEditable(true);
            
            label = new Label( composite, SWT.NONE );
            label.setLayoutData( new GridData(SWT.END, SWT.DEFAULT, false, false ,2,1) );
        }
        
        // add spacer
        label = new Label( composite, SWT.SEPARATOR | SWT.HORIZONTAL );
        label.setLayoutData( new GridData(SWT.FILL, SWT.CENTER, true, false,5,3 ) );
        
        // advanced 
        advancedKey = new Button( composite, SWT.CHECK );
        advancedKey.setLayoutData( new GridData(SWT.CENTER, SWT.DEFAULT, false, false ) );
        advancedKey.setSelection(false);
        advancedKey.addSelectionListener(this);
        advancedKey.setText(Policy.bind("DataBaseRegistryWizardPage.label.advanced.text")); //$NON-NLS-1$
        advancedKey.setToolTipText(Policy.bind("DataBaseRegistryWizardPage.label.advanced.tooltip")); //$NON-NLS-1$

        label = new Label( composite, SWT.NONE ); // advanced check row
        label.setLayoutData( new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false ,4,1) );
        label = new Label( composite, SWT.NONE ); // slot one of child row
        label.setLayoutData( new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false ) );
      
        advanced = createAdvancedControl(composite);
        if(advanced == null){
            advancedKey.setVisible(false);// turn off - we don't want events
            advancedKey.setEnabled(false);// turn off - we don't want events
        }else{
            data = new GridData(SWT.FILL, SWT.FILL, false, false );
            data.horizontalSpan=5;
            advanced.setLayoutData( data );
        	advanced.setVisible(false);
        }

        if( host.getData() == null && hostStr != null ) {
            if( isHostCombo() ) {
                ((CCombo)host).setText(hostStr);
                ((CCombo)host).add(hostStr);
            } else {
                ((Text)host).setText(hostStr);
            }
            host.setData(hostStr);
            port.setText(portStr);
            user.setText(userStr);
            pass.setText(passStr);
            if( isDBCombo() ) {
                ((CCombo)database).setText(databaseStr);
            } else {
                ((Text)database).setText(databaseStr);
            }
            if( hasSchema())
                schema.setText(schemaStr);
        }
        List<Control> tablist=new LinkedList<Control>();
        tablist.add(host);
        tablist.add(user);
        tablist.add(pass);
        tablist.add(database);
        if( schema!=null )
            tablist.add(schema);
        tablist.add(port);
        tablist.add(advancedKey);
        if( advanced!=null )
            tablist.add(advanced);


        //override tab order
        if (schema != null) {
            
            //Vitali Diatchkov, Arbonaut Oy LTD.

            if(advanced == null)
                composite.setTabList(new Control[]{host,user,pass,database,schema,port,advancedKey});
            else
                composite.setTabList(new Control[]{host,user,pass,database,schema,port, advancedKey,advanced});
        }
        else {
            if(advanced == null)
                composite.setTabList(new Control[]{host,user,pass,database,port,advancedKey});
            else
                composite.setTabList(new Control[]{host,user,pass,database,port,advancedKey,advanced});
        }
        
        
        setControl(composite);
        setPageComplete(true);
    }
    protected abstract Group createAdvancedControl( Composite arg0 );

    public boolean isPageComplete() {
        if( isHostCombo() ) {
            hostStr = ((CCombo)host).getText();
        } else {
            hostStr = ((Text)host).getText();
        }
        portStr = port.getText();
        userStr = user.getText();
        passStr = pass.getText();
        if( isDBCombo() ) {
            databaseStr = ((CCombo)database).getText();
        } else {
            databaseStr = ((Text)database).getText();
        }
        if( hasSchema() )
            schemaStr = schema.getText();
        return false;
    }
    
    /**
     * TODO summary sentence for modifyText ...
     * 
     * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
     * @param e
     */
    public void modifyText( ModifyEvent e ) {
        if(e.widget!=null){
            if(e.widget instanceof Text && !(e.widget instanceof CCombo)){
                ((Text)e.widget).setForeground(null);
                if(isDBCombo())
                    ((CCombo)database).removeAll();
            }
            if(hasSchema() && e.widget.equals(database)){
                schema.removeAll();
            }
        }
        
        getWizard().getContainer().updateButtons();
    }
    
    /**
     * TODO summary sentence for widgetSelected ...
     * 
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
     * @param e
     */
    public void widgetSelected( SelectionEvent e ) {
        if(e.widget!=null && e.widget.equals(advancedKey)){
            advanced.setVisible(advancedKey.getSelection());
        }
        
        widgetSelectedInternal(e);
        
        getWizard().getContainer().updateButtons();
    }
    
    protected void widgetSelectedInternal(SelectionEvent e) {
    	//do nothing
    }
    
    protected void populateDB(){
        // will need a connection ... 
        CCombo db = (CCombo)this.database;
        
        //save current state
        int selected = db.getSelectionIndex();
        String string = null;
        if (selected > -1) {
        	string = db.getItem(selected);
        }
        
        db.removeAll();
        db.setText(""); //$NON-NLS-1$
        
        Connection con = null;
        try {
        	con = getConnection();	
        }
        catch(Exception e) {
        	CatalogUIPlugin.log(e.getLocalizedMessage(), e);
        }
        
        if(con == null)return;
        
        //connection ok, reset any previous error messages
        setErrorMessage(null);
        
        ResultSet rs = null;
        if(con != null)
        try {
            rs = con.getMetaData().getCatalogs();
            while(rs.next()){
            	String dbName = rs.getString(1);
            	if (!excludeDB(dbName)) {
            		((CCombo)database).add(dbName);	
            	}
                
            }
            if (db.getItemCount() > 0)
            	db.select(0);
            
        } catch (SQLException e) {
            setErrorMessage(e.getLocalizedMessage());
            db.removeAll();
            db.setText(""); //$NON-NLS-1$
            return;
        }
        
        if (string != null) {
        	String[] items = db.getItems();
        	for (int i = 0; items != null && i < items.length; i++) {
        		if (string.equals(items[i])) {
        			db.select(i);
        			return;
        		}
        	}
        }
    }
    
    protected void populateSchema(){
        if (!hasSchema()) // error
            return;
        // will need a connection ... 
        
        //save some state
        int selected = schema.getSelectionIndex();
        String string = null;
        if (selected > -1) {
        	string = schema.getItem(selected);
        }
        
        schema.removeAll();
        schema.setText(""); //$NON-NLS-1$
        
        Connection con = null;
        try {
        	con = getConnection();	
        }
        catch(Exception e) {
            e.printStackTrace(System.out);
        	CatalogUIPlugin.log(e.getLocalizedMessage(), e);
        	setErrorMessage(e.getLocalizedMessage());
        }
        
        if(con == null)return;
        ResultSet rs = null;
        try {
            rs = con.getMetaData().getSchemas();
            while(rs.next()){
            	String schemaName = rs.getString(1);
            	if (!excludeSchema(schemaName))
                    schema.add(schemaName);
        	}
            if (schema.getItemCount() > 0)
            	schema.select(0);
            
        } catch (SQLException e) {
            setErrorMessage(e.getLocalizedMessage());
            e.printStackTrace(System.out);
            schema.removeAll();
            schema.setText(""); //$NON-NLS-1$
            return;
        }
        
        if (string != null) {
        	String[] items = schema.getItems();
        	for (int i = 0; items != null && i < items.length; i++) {
        		if (string.equals(items[i])) {
        			schema.select(i);
        			return;
        		}
        	}
        }
    }
    
    protected boolean excludeDB(String db) {
    	return false;
    }
    
    protected boolean excludeSchema(String schema) {
    	return false;
    }
    
    // to use for wizard polulation
    protected abstract Connection getConnection() throws Exception;
	
    /**
     * TODO summary sentence for widgetDefaultSelected ...
     * 
     * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
     * @param e
     */
    public void widgetDefaultSelected( SelectionEvent e ) {
        // do nothing
    }
    
    public class DataBaseConnInfo {
        private String host;
        private String port;
        private String user;
        private String pass;
        private String db;
        
        public DataBaseConnInfo (String host, String port, String user, String pass, String db) {
            this.host = host;
            this.port = port;
            this.user = user;
            this.pass = pass;
            this.db = db;
        }
        public DataBaseConnInfo (String dbEntry) {
            String[] temp = dbEntry.split("\t"); //$NON-NLS-1$
            if( temp.length < 5 ) {
                //try splitting on " "
                temp = dbEntry.split(" "); //$NON-NLS-1$
                if( temp.length < 5 ) return;
            }
            this.host = temp[0];
            this.port = temp[1];
            this.user = temp[2];
            this.pass = temp[3];
            this.db = temp[4];
        }
        public String getHost() {
            return host;
        }
        public String getPort() {
            return port;
        }
        public String getUser() {
            return user;
        }
        public String getPass() {
            return pass;
        }
        public String getDb() {
            return db;
        }
        public String toString() {
            return host + "\t" + port + "\t" + user + "\t" + pass + "\t" + db;  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
        }
    }
	
}
