package net.refractions.udig.core.internal;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.Plugin;
import org.osgi.framework.BundleContext;
import org.picocontainer.MutablePicoContainer;
import org.picocontainer.defaults.DefaultPicoContainer;
/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */

/**
 * PlugIn for net.refractions.udig.core, used by utility classes to access workbench log.
 * 
 * @author jones
 * @since 0.3
 */
public class CorePlugin extends Plugin {

    /** Plugin <code>ID</code> field */
    public static final String ID = "net.refractions.udig.core"; //$NON-NLS-1$
    private static CorePlugin plugin;

    /**
     * creates a plugin instance
     */
    public CorePlugin() {
        super();
        plugin = this;
    }

    /**
     * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
     */
    public void start(BundleContext context) throws Exception {
        super.start(context);     
    }

    /**
     * Returns the system created plugin object
     *
     * @return the plugin object
     */
    public static CorePlugin getDefault() {
        return plugin;
    }    

    private static MutablePicoContainer blackboard = null;
    
    /**
     * This is intended to return the top level Pico Container to use for blackBoarding.
     * <p>
     * For most applications, a sub container is required. 
     * I recommend the following code be inserted into your main plugin class.
     * <code>
     * private static MutablePicoContainer myContainer = null;
     * 
     * /**
     *  * Gets the container for my plugin.
     *  * 
     *  * Make it 'public' if you want to share ... protected otherwise.
     *  * /
     * public static MutablePicoContainer getMyContainer(){
     *   if(myContainer == null){
     *     // XXXPlugin is the name of the Plugin class
     *     synchronized(XXXPlugin.class){
     *       // check so see that you were not queued for double creation
     *       if(myContainer == null){
     *         // This line does it ... careful to only call it once!
     *         myContainer = CorePlugin.getBlackBoard().makeChildContainer();
     *       }
     *     }
     *   }
     *   return myContainer;
     * }
     * </code> 
     * </p>
     * <p>
     * NOTE:<br>
     * Please check to ensure the child you want is not already 
     * created (important for two plugins sharing one container).
     * </p>
     * 
     * @return
     */
    public static MutablePicoContainer getBlackBoard(){
        if(blackboard == null){
            synchronized(CorePlugin.class){
                if(blackboard == null){
                    blackboard = new DefaultPicoContainer();
                }
            }
        }
        return blackboard;
    }

    /**
     * Takes a string, and splits it on '\n' and calls stringsToURLs(String[])
     */
    public static List<URL> stringsToURLs(String string) {
        String[] strings = string.split("\n"); //$NON-NLS-1$
        
        return stringsToURLs(strings);
    }

    /**
     * Converts each element of an array from a String to a URL. If the String
     * is not a valid URL, it attempts to load it as a File and then convert
     * it. If that fails, it ignores it. It will not insert a null into the
     * returning List, so the size of the List may be smaller than the size
     * of <code>strings</code>
     * 
     * @param strings an array of strings, each to be converted to a URL
     * @return a List of URLs, in the same order as the array
     */
    public static List<URL> stringsToURLs(String[] strings) {
        List<URL> urls = new ArrayList<URL>();
        
        for ( String string : strings ) {
            try {
                urls.add(new URL(string));
            } catch (MalformedURLException e) {
                //not a URL, maybe it is a file
                try {
                    urls.add(new File(string).toURL());
                } catch (MalformedURLException e1) {
                    //Not a URL, not a File. nothing to do now.
                }
            }
        }
        return urls;
    }
}
