package net.refractions.udig.internal.ui;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IPropertyListener;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

public class UDIGEditorDropListener extends DropTargetAdapter {
	
	UDIGDropHandler handler;

	public UDIGEditorDropListener() {
		handler = new UDIGDropHandler(this);
		handler.setDestination(new EditorPlaceholder());
	}
	public void dragEnter( DropTargetEvent event ) {
        handler.dragEnter(event);
    }

    public void dragOperationChanged( DropTargetEvent event ) {
        handler.dragOperationChanged(event);
    }

    @Override
    public void dragLeave( DropTargetEvent event ) {
        handler.dragLeave(event);
    }
    @Override
    public void dragOver( DropTargetEvent event ) {
        handler.dragOver(event);
    }
    @Override
    public void dropAccept( DropTargetEvent event ) {
        handler.dropAccept(event);
    }
    public void drop(DropTargetEvent event ) {
		handler.drop(event);
		handler.performDrop(event.data);
	}

	class EditorPlaceholder implements IEditorPart {

		IEditorPart getEditorPart() {
			return PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getActivePage().getActiveEditor();
		}
		
		public IEditorInput getEditorInput() {
			return getEditorPart() != null ? getEditorPart().getEditorInput()
				: null;
		}

		public IEditorSite getEditorSite() {
			return getEditorPart() != null ? getEditorPart().getEditorSite()
				: null;
		}

		public void init(IEditorSite site, IEditorInput input) 
			throws PartInitException {
			
			if (getEditorPart() != null) 
					getEditorPart().init(site, input);
		}

		public void addPropertyListener(IPropertyListener listener) {
			if (getEditorPart() != null) 
				getEditorPart().addPropertyListener(listener);
		}

		public void createPartControl(Composite parent) {
			if (getEditorPart() != null)
				getEditorPart().createPartControl(parent);
		}

		public void dispose() {
			if (getEditorPart() != null)
				getEditorPart().dispose();
		}

		public IWorkbenchPartSite getSite() {
			return getEditorPart() != null ? getEditorPart().getSite() 
					: null;
		}

		public String getTitle() {
			return getEditorPart() != null ? getEditorPart().getTitle() 
					: null;
		}

		public Image getTitleImage() {
			return getEditorPart() != null ? getEditorPart().getTitleImage() 
					: null;
		}

		public String getTitleToolTip() {
			return getEditorPart() != null ? getEditorPart().getTitleToolTip() 
					: null;
		}

		public void removePropertyListener(IPropertyListener listener) {
			if (getEditorPart() != null)
				getEditorPart().removePropertyListener(listener);
		}

		public void setFocus() {
			if (getEditorPart() != null)
				getEditorPart().setFocus();
		}

		public Object getAdapter(Class adapter) {
			return getEditorPart() != null 
				? getEditorPart().getAdapter(adapter)
				: null;
		}

		public void doSave(IProgressMonitor monitor) {
			if (getEditorPart() != null)
				getEditorPart().doSave(monitor);
		}

		public void doSaveAs() {
			if (getEditorPart() != null)
				getEditorPart().doSaveAs();
		}

		public boolean isDirty() {
			return getEditorPart() != null ? getEditorPart().isDirty()
					: false;
		}

		public boolean isSaveAsAllowed() {
			return getEditorPart() != null ? getEditorPart().isSaveAsAllowed()
					: false;
		}

		public boolean isSaveOnCloseNeeded() {
			return getEditorPart() != null 
				? getEditorPart().isSaveOnCloseNeeded()
				: false;
		}
		
	}
	
}