package net.refractions.udig.mapgraphic.internal;

import java.io.IOException;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.IServiceInfo;
import net.refractions.udig.mapgraphic.MapGraphicFactory;
import net.refractions.udig.mapgraphic.Policy;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IProgressMonitor;

public class MapGraphicService extends IService {
    public Map<String, Serializable> params;
    public static URL SERVICE_URL = null;
    static {
       try {
           //SERVICE_URL = new URL("http://localhost/mapgraphic"); //$NON-NLS-1$
    	   SERVICE_URL = new URL("file:///localhost/mapgraphic");
       } 
       catch (MalformedURLException e) {
            e.printStackTrace();
       }
    }
    
    /** decorator resource children **/
    private List<MapGraphicResource> members;
    
    /** info object **/
    private MapGraphicServiceInfo info;
    
    /**
     * Construct <code>DecoratorService</code>.
     *
     */
    public MapGraphicService() {
        this.params = new HashMap<String, Serializable>();
        this.params.put(MapGraphicServiceExtension.KEY, SERVICE_URL);
    }
    
    public MapGraphicService(URL url, Map<String, Serializable> params) {
        this.params = params;
    }
    
    /*
     * @see net.refractions.udig.catalog.IService#resolve(java.lang.Class, org.eclipse.core.runtime.IProgressMonitor)
     */
    public <T> T resolve( Class<T> adaptee, IProgressMonitor monitor ) 
    	throws IOException {
    	
        if (adaptee == null) return null;
        
        if (adaptee.isAssignableFrom(IServiceInfo.class)) {
            if (info == null)
                info = new MapGraphicServiceInfo();
            
            return adaptee.cast(info);
        }
        if (adaptee.isAssignableFrom(List.class)) {
           return adaptee.cast(members); 
        }
        if (adaptee.isAssignableFrom(MapGraphicFactory.class)) {
            return adaptee.cast(MapGraphicFactory.getInstance());
        }
            
        return null;
    }

    /*
     * @see net.refractions.udig.catalog.IService#members(org.eclipse.core.runtime.IProgressMonitor)
     */
    public List< ? extends IGeoResource> members( IProgressMonitor monitor ) 
    	throws IOException {
    	
        if (members == null) {
        	synchronized (MapGraphicFactory.getInstance()) {
        		if (members == null) {
        			members = new ArrayList<MapGraphicResource>();
        			List<IConfigurationElement> graphics = MapGraphicFactory
        				.getInstance().getMapGraphics();
        			
        			if (graphics != null) {
        				for (IConfigurationElement graphic : graphics) {
        					members.add(new MapGraphicResource(this,graphic));
        				}
        			}
        		}
			}
        }
        
        return members;
    }

    /*
     * @see net.refractions.udig.catalog.IService#getConnectionParams()
     */
    public Map<String, Serializable> getConnectionParams() {
        return this.params;
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#canResolve(java.lang.Class)
     */
    public <T> boolean canResolve( Class<T> adaptee ) {
        return adaptee != null && (
            adaptee.isAssignableFrom(IServiceInfo.class) ||  
            adaptee.isAssignableFrom(List.class)  
            //adaptee.isAssignableFrom(MapGraphicFactory.class)

        ); 
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#getStatus()
     */
    public Status getStatus() {
        return Status.CONNECTED;
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#getMessage()
     */
    public Throwable getMessage() {
        return null;
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#getIdentifier()
     */
    public URL getIdentifier() {
        return SERVICE_URL;
    }
    
    /**
     * 
     * Adds a new resource to the map graphic service.
     * 
     */
//    public void addDecoratorResource(IConfigurationElement element) {
//        MapGraphicResource resource = new MapGraphicResource(this, element); 
//        members.add(resource);
//    }
    
//    MapGraphicFactory getFactory() {
//        if (factory == null) {
//            synchronized (this) {
//                if (factory == null) {
//                    factory = new MapGraphicFactory();
//                }
//            }
//        }
//        
//        return factory;
//    }
    
    class MapGraphicServiceInfo extends IServiceInfo {
        /*
         * @see net.refractions.udig.catalog.IServiceInfo#getTitle()
         */
        public String getTitle() {
            return Policy.bind("MapGraphicService.title"); //$NON-NLS-1$
        }
        
        @Override
        public String getDescription() {
            return getTitle();
        }
    }
}