/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.printing.ui;

import java.util.Iterator;

import net.refractions.udig.printing.model.Page;
import net.refractions.udig.project.internal.Map;

/**
 * A Template describes each element to be realized onto a Page. It is used to 
 * provide users with a performatted Page that can be used multiple times to 
 * easily print maps without having to worry about repositioning and resizing
 * each Box every time they create a new page.
 * 
 * @author Richard Gould
 */

public interface Template extends Cloneable {

	/**
	 * This method initializes the template to the
	 * page according to its own interests. Some interests may include TODO fill me in ...
	 * 
	 * @param page the Page to initialize
	 * @param map the Map that this page is centered around
	 * @param width the desired width for the page, in pixels
	 * @param height the desired height for the page, in pixels
	 */
	public void init(Page page, Map map, int width, int height);

	/**
	 * Returns an iterator that iterates over each Box contained in the
	 * template.
     * 
     * This is used by the framework to access a Template's Boxes.
	 * 
	 * @return an iterator where each element is of type Box
	 * @see Box
	 */
	public Iterator iterator();
	
	/**
	 * @return A human-readable String that identifies this template
	 */
	public String getName();
    
    /**
     * Templates <b>must</b> clone themselves and their contents properly.
     * 
     * @return a copy of this template
     * @see Cloneable
     * @see Object#clone
     */
    public Template clone();
	
}
