/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.printing.ui.actions;

import java.awt.print.PageFormat;
import java.awt.print.PrinterJob;
import java.text.MessageFormat;
import java.util.Iterator;
import java.util.Locale;

import net.refractions.udig.printing.model.Box;
import net.refractions.udig.printing.model.ModelFactory;
import net.refractions.udig.printing.model.Page;
import net.refractions.udig.printing.ui.Policy;
import net.refractions.udig.printing.ui.Template;
import net.refractions.udig.printing.ui.TemplateFactory;
import net.refractions.udig.printing.ui.internal.PrintingPlugin;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.Project;
import net.refractions.udig.project.ui.internal.MapEditorInput;
import net.refractions.udig.project.ui.internal.ProjectExplorer;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ListDialog;

/**
 * Creates a Page using the current map
 * 
 * @author Richard Gould
 */
public class CreatePageAction implements IEditorActionDelegate {

    public void run( IAction action ) {
        IEditorInput input = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
                .getActiveEditor().getEditorInput();

        if (!(input instanceof MapEditorInput)) {
            MessageDialog.openError(Display.getDefault().getActiveShell(), Policy.bind("CreatePageAction.printError.title"), //$NON-NLS-1$
                    Policy.bind("CreatePageAction.printError.text")); //$NON-NLS-1$
        }

        final java.util.Map<String, TemplateFactory> templateFactories = PrintingPlugin.getDefault().getTemplateFactories();

        //TODO move to a preference initializer
        PrintingPlugin.getDefault().getPluginPreferences().setValue(
                PrintingPlugin.PREF_DEFAULT_TEMPLATE,
                "net.refractions.udig.printing.ui.internal.BasicTemplate"); //$NON-NLS-1$

        String defaultTemplate = PrintingPlugin.getDefault().getPluginPreferences().getString(
                PrintingPlugin.PREF_DEFAULT_TEMPLATE);

        TemplateFactory templateFactory;

        ListDialog dialog = new ListDialog(Display.getDefault().getActiveShell());
        dialog.setTitle(Policy.bind("CreatePageAction.dialog.title")); //$NON-NLS-1$
        dialog.setMessage(Policy.bind("CreatePageAction.dialog.message")); //$NON-NLS-1$
        dialog.setInput(templateFactories.values());
        ArrayContentProvider provider = new ArrayContentProvider();
        dialog.setContentProvider(provider);

        ILabelProvider labelProvider = new LabelProvider(){
            public String getText( Object element ) {
                return ((TemplateFactory) element).getName();
            }
        };
        dialog.setLabelProvider(labelProvider);

        templateFactory = (TemplateFactory) PrintingPlugin.getDefault().getTemplateFactories().get(defaultTemplate);

        dialog.setInitialSelections(new Object[]{templateFactory});
        int result = dialog.open();
        if (result == Window.CANCEL || dialog.getResult().length == 0) {
            return;
        }

        Template template = null;
        
        templateFactory = ((TemplateFactory) dialog.getResult()[0]);

        if (templateFactory == null) {
            PrintingPlugin.getDefault().log(Policy.bind("CreatePageAction.error.cannotFindDefaultTemplate"),null); //$NON-NLS-1$

            TemplateFactory firstAvailable = (TemplateFactory) templateFactories.values().iterator().next();
            if (firstAvailable == null) {
            	PrintingPlugin.getDefault().log("Unable to locate any templates, aborting page creation.", null);
            	//TODO should probably present the user with this error
            	return;
            }
            template = firstAvailable.createTemplate();
        } else {
        	template = templateFactory.createTemplate();
        }

        Map map = null;
        Project project = null;

        Map oldMap = (Map) ((MapEditorInput) input).getProjectElement();
        project = oldMap.getProjectInternal();

        
        map = (Map) EcoreUtil.copy(oldMap);

        MessageFormat formatter = new MessageFormat(Policy.bind("CreatePageAction.newPageName"), Locale.getDefault()); //$NON-NLS-1$
        map.setName(formatter.format(new Object[] { map.getName() }));

        PageFormat pageFormat = PrinterJob.getPrinterJob().defaultPage();
        int width = new Double(pageFormat.getImageableWidth()).intValue();
        int height = new Double(pageFormat.getImageableHeight()).intValue();

        Page page = ModelFactory.eINSTANCE.createPage();

        page.setSize(new Dimension(width, height));
        if (page.getName() == null || page.getName().length() == 0) {
            page.setName("Untitled");
        }
        template.init(page, map, width, height);
        Iterator<Box> iter = template.iterator();
        while (iter.hasNext()) {
            page.getBoxes().add(iter.next());
        }

//        project.getElements().add(page);
        page.setProjectInternal(project);
        
        ProjectExplorer.getProjectExplorer().open(page);
        ProjectExplorer.getProjectExplorer().selectReveal(new StructuredSelection(page));

    }

    /**
     * TODO summary sentence for setActiveEditor ...
     * 
     * @see org.eclipse.ui.IEditorActionDelegate#setActiveEditor(org.eclipse.jface.action.IAction,
     *      org.eclipse.ui.IEditorPart)
     * @param action
     * @param targetEditor
     */
    public void setActiveEditor( IAction action, IEditorPart targetEditor ) {
    }

    /**
     * TODO summary sentence for selectionChanged ...
     * 
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
     *      org.eclipse.jface.viewers.ISelection)
     * @param action
     * @param selection
     */
    public void selectionChanged( IAction action, ISelection selection ) {
    }
}
