/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.printing.ui.internal;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.util.Iterator;

import net.refractions.udig.printing.model.Box;
import net.refractions.udig.printing.model.Page;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;


/**
 * The PrintingEngine takes a page and processes it, taking each Box and 
 * realizing its contents and sending the result to the printer.
 * 
 * @author Richard Gould
 */
public class PrintingEngine implements Printable{

//	private Page page;
	
	Page diagram;

    /**
	 * Constructs a PrintingEngine using the given Page
	 * @param page the Page to be printed
	 */
	public PrintingEngine(Page diagram) {
//		this.page = page;
	    this.diagram = diagram;
	}

	/**
	 * Iterates through the Page's Boxes, drawing to the provided Graphics 
	 * object
	 * 
	 * @see java.awt.print.Printable#print(java.awt.Graphics, java.awt.print.PageFormat, int)
	 */
	public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException {

		if (pageIndex >= 1) {
			return Printable.NO_SUCH_PAGE;
		}

		final Graphics2D graphics2d = (Graphics2D) graphics;
	
		graphics2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());

        Job job=new Job("Printing"){

            protected IStatus run( IProgressMonitor monitor ) {

                Iterator iter = diagram.getBoxes().iterator();
                while(iter.hasNext()) {
                    Box box = (Box) iter.next();
                                
                    AffineTransform at = graphics2d.getTransform();
                    double dpi = at.getScaleX() * 72;
                    
                    graphics2d.translate(box.getLocation().x, box.getLocation().y);
                    
                    Shape clip = graphics2d.getClip();
                    graphics2d.setClip(0, 0, box.getSize().width, box.getSize().height);
                    
                    box.draw(graphics2d, monitor);
                    
                    graphics2d.setClip(clip);
                    graphics2d.setTransform(at);
                }
                return Status.OK_STATUS;
            }
            
        };
        job.schedule();
        try {
            job.join();
        } catch (InterruptedException e) {
            // don't worry about it.
        }
		return Printable.PAGE_EXISTS;
	}
}
