/*
 * Created on 29-Sep-2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package net.refractions.udig.printing.ui.internal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.refractions.udig.printing.model.Box;
import net.refractions.udig.printing.ui.BoxFactory;
import net.refractions.udig.printing.ui.Policy;
import net.refractions.udig.printing.ui.Template;
import net.refractions.udig.printing.ui.TemplateFactory;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * @author Richard Gould
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class PrintingPlugin extends AbstractUIPlugin {
	private static PrintingPlugin plugin;

	private Map<String, TemplateFactory> templateFactories;

	private Map<BoxFactory, Boolean> boxes;

	private static final String templateFactoriesID = "net.refractions.udig.printing.ui.templateFactories"; //$NON-NLS-1$
	private static final String boxFactoriesID = "net.refractions.udig.printing.ui.boxFactories"; //$NON-NLS-1$

    public static final String PREF_DEFAULT_TEMPLATE = "udig.preferences.defaultTemplate"; //$NON-NLS-1$

    public static final String PLUGIN_ID = "net.refractions.udig.printing.ui"; //$NON-NLS-1$

	public PrintingPlugin() {
		super();
		plugin = this;
	}

	public static PrintingPlugin getDefault() {
		return plugin;
	}

	public Map<String, TemplateFactory> getTemplateFactories() {
		if (templateFactories == null) {
			templateFactories = gatherTemplateFactories();
		}
		return templateFactories;
	}

	private Map<String, TemplateFactory> gatherTemplateFactories() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();

		IExtensionPoint extensionPoint = registry.getExtensionPoint(templateFactoriesID);

		IExtension[] extensions = extensionPoint.getExtensions();

		HashMap<String, TemplateFactory> results = new HashMap<String, TemplateFactory>();

		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement[] elements = extensions[i].getConfigurationElements();
			
			for(int j = 0; j < elements.length; j++) {
				try {
					Object templateFactory = elements[j].createExecutableExtension("class"); //$NON-NLS-1$
					if (templateFactory instanceof TemplateFactory) {
					    String id = elements[j].getAttribute("id"); //$NON-NLS-1$
						results.put(id, (TemplateFactory) templateFactory);
					} else {
						log("Bad extension! Declared class is not of type TemplateFactory!", null);
					}
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
		}

		return results;
	}

    /**
     * Provides easy access to the Eclipse plugin logger     
     * @param msg a message to be logged
     * @param e an exception to be logged, can be null
     */
    public void log(String msg, Exception e) {
        getLog().log(new Status(IStatus.INFO, PLUGIN_ID, IStatus.OK, msg, e));
    }
    /*
     * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
     */
    public void start( BundleContext context ) throws Exception {
        super.start(context);
        Policy.localize("net.refractions.udig.printing.ui.messages"); //$NON-NLS-1$
    }
    
    
    /*
     * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
     */
    public void stop( BundleContext context ) throws Exception {
        Policy.cleanup();
        super.stop(context);
    }

	public List<BoxFactory> getVisibleBoxes() {
		if (boxes == null) {
			boxes = gatherBoxes();
		}
		ArrayList<BoxFactory> visibleBoxes = new ArrayList<BoxFactory>();

		for (Map.Entry<BoxFactory, Boolean> entry : boxes.entrySet()) {
			if (entry.getValue() == Boolean.TRUE) {
				visibleBoxes.add(entry.getKey());
			}
		}
		
		return visibleBoxes;
	}

	private Map<BoxFactory, Boolean> gatherBoxes() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint extensionPoint = registry.getExtensionPoint(boxFactoriesID);
		IExtension[] extensions = extensionPoint.getExtensions();

		HashMap<BoxFactory, Boolean> results = new HashMap<BoxFactory, Boolean>();

		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement[] elements = extensions[i].getConfigurationElements();
			
			for(int j = 0; j < elements.length; j++) {
				try {
					Object boxFactory = elements[j].createExecutableExtension("class"); //$NON-NLS-1$
					if (boxFactory instanceof BoxFactory) {
					    String visibleAttribute = elements[j].getAttribute("visible"); //$NON-NLS-1$
					    Boolean visible = Boolean.TRUE;
					    if ("false".equalsIgnoreCase(visibleAttribute)) {
					    	visible = Boolean.FALSE;
					    }
						results.put((BoxFactory) boxFactory, visible);
					} else {
						log("Bad extension! Declared class is not of type BoxFactory!", null);
					}
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
		}

		return results;
	}
}
