/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.command;

import java.util.LinkedList;
import java.util.List;

import net.refractions.udig.project.internal.Map;

import org.eclipse.core.runtime.IProgressMonitor;

/**
 * TODO Purpose of net.refractions.udig.project.command <p> </p>
 * @author   Jesse
 * @since   1.0.0
 */
public class CompositeCommand implements MapCommand {

	protected List<? extends MapCommand> commands = new LinkedList<MapCommand>();

	private Map map;

	/**
	 * Creates a new instance of CompositeCommand
	 * 
	 * @param commands A list of commands to execute.
	 */
	public CompositeCommand(List<? extends MapCommand> commands) {
		this.commands = commands;
	}

	/**
	 * @see net.refractions.udig.project.internal.command.MapCommand#run()
	 */
	public void run(IProgressMonitor monitor) throws Exception {
		for (MapCommand command : commands) {
			command.run(monitor);
		}

	}

	/**
	 * @see net.refractions.udig.project.internal.command.MapCommand#copy()
	 */
	public MapCommand copy() {
		return new CompositeCommand(new LinkedList<MapCommand>(commands));
	}

	/**
	 * @see net.refractions.udig.project.command.MapCommand#getName()
	 */
	public String getName() {
		StringBuffer buffer = new StringBuffer("["); //$NON-NLS-1$

		for (MapCommand command : commands)
			buffer.append(command.getName() + ","); //$NON-NLS-1$
		buffer.replace(buffer.length() - 1, buffer.length(), "]"); //$NON-NLS-1$
		return buffer.toString();
	}

	/**
     * @see net.refractions.udig.project.command.MapCommand#setMap(net.refractions.udig.project.Map)
     * @uml.property   name="map"
     */
	public void setMap(Map map) {
		this.map = map;
	}

	/**
     * @see net.refractions.udig.project.command.MapCommand#getMap()
     * @uml.property   name="map"
     */
	public Map getMap() {
		return map;
	}

}
