/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.ICatalog;
import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.IServiceFactory;
import net.refractions.udig.project.internal.impl.LayerImpl;

import org.eclipse.emf.common.util.URI;

/**
 * TODO Purpose of net.refractions.udig.project.internal <p> </p>
 * @author   Jesse
 * @since   1.0.0
 */
public class CatalogRef {
	protected Layer layer;

	protected static final String ID_SEPERATOR = "-ID_SEP-"; //$NON-NLS-1$

	protected static final String CONNECTION_PARAM = "-ENTRY_SEP-"; //$NON-NLS-1$

	protected static final String KEY_VALUE_SEPERATOR = "-KEY_VALUE_SEP-"; //$NON-NLS-1$

	protected static final String ENCODING = ProjectPlugin.Implementation.ENCODING;

	protected static final String SERVICE_TAG = "-SERVICE_TAG-:"; //$NON-NLS-1$

	/**
     * @uml.property   name="connectionParams"
     * @uml.associationEnd   qualifier="key:java.lang.Object java.util.Map<String,Serializable>"
     */
	protected Map<URL, Map<String, Serializable>> connectionParams = new HashMap<URL,Map<String, Serializable>>();

    private boolean loaded=false;

	/**
	 * Construct <code>LayerRef</code>.
	 * 
	 * @param layer
	 */
	public CatalogRef(Layer layer) {
		this.layer = layer;
	}

	/**
	 * Construct <code>LayerRef</code>.
	 */
	public CatalogRef() {
		// do nothing
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer stringBuffer = new StringBuffer();
		List<IGeoResource> resources = layer.getGeoResources();
		if (resources != LayerImpl.NULL) {

			connectionParams.clear();
			for (IGeoResource resource : resources) {
				try {
					IService service = resource.service(null);
					connectionParams.put(service.getIdentifier(),service.getConnectionParams());
				} catch (IOException e) {
					ProjectPlugin.log("Error saving project", e); //$NON-NLS-1$
				}

			}
		}
		Iterator<Map<String, Serializable>> paramIter = connectionParams
				.values().iterator();
		Iterator<URL> idIter = connectionParams.keySet().iterator();
		while (paramIter.hasNext()) {
			Map<String, Serializable> params = paramIter.next();
			URL serviceID = idIter.next();
			boolean resourceInit = false;
			String id = encodeServiceID(serviceID);

			stringBuffer.append(id);
			if (resourceInit)
				stringBuffer.append(SERVICE_TAG);
			else
				resourceInit = true;

			for (Iterator e = params.entrySet().iterator(); e.hasNext();) {
				stringBuffer.append(encodeParameter((Entry) e.next()));
				if (e.hasNext())
                    stringBuffer.append(CONNECTION_PARAM);
			}
		}
		return stringBuffer.toString();
	}

	private String encodeParameter(Map.Entry entry) {
		StringBuffer stringBuffer = new StringBuffer();
		String key = entry.getKey().toString();
		String value = entry.getValue() == null ? null : entry.getValue()
				.toString();

		if (key.equalsIgnoreCase("URL")) //$NON-NLS-1$
			try {
				value = encodeURL(value);
			} catch (UnsupportedEncodingException e) {
				// do nothing
			}
		if (value != null)
			stringBuffer.append(key + KEY_VALUE_SEPERATOR + value);
		return stringBuffer.toString();
	}

	/**
	 * @param service
	 *            the service to encode as a string reference.
	 * @return the id of the service encoded as a UTF-16 string.
	 */
	private String encodeServiceID(URL serviceID) {
		String id = ""; //$NON-NLS-1$
		try {
			id = encodeURL(serviceID.toString());
		} catch (UnsupportedEncodingException e1) {
			// TODO Catch e1
		}

		return id + ID_SEPERATOR;

	}

	/**
	 * TODO summary sentence for encodeURL ...
	 * 
	 * @param identifier
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	private String encodeURL(String url) throws UnsupportedEncodingException {
		String encoding = URLEncoder.encode(url, ENCODING);
        if( url.startsWith("file:") )//$NON-NLS-1$ 
        {
            URI uri = getLayer().getContextModel().getMap().getProjectInternal()
            .eResource().getURI();
    		if (!(uri.isFile()) )
                return getRelativePath(encoding, uri);
        }

        return encoding;


	}

	/**
	 * @param url
	 * @param uri
	 * @return
	 */
	private String getRelativePath(String url, URI uri) {
		String[] layerParts = getFile(url).split(File.pathSeparator);
		int i = 0;
		LOCATE_RELATIVE_PATH: for (String part : layerParts) {
			if (i >= uri.segmentCount())
				break;
			if (part.equals(uri.segment(i)))
				continue LOCATE_RELATIVE_PATH;

			i++;
		}

//		String newEncoding;
//		if (i >= uri.segmentCount())
//			newEncoding = getRelativePath(layerParts, i);
//		else
//			newEncoding = 0 + url;

		return url;
	}

//	private String getRelativePath(String[] layerParts, int i) {
//		// TODO Auto-generated method stub
//		return null;
//	}

	private String getFile(String encoding2) {
		String[] parts = encoding2.split(":"); //$NON-NLS-1$
		if (parts.length == 1)
			return parts[0];
		return parts[1];
	}

	/**
	 * @param string
	 */
	public void addServiceToCatalog(String string) {


		String[] resources = string.split(SERVICE_TAG);
		DECODING: for (int i = 0; i < resources.length; i++) {
			decodeConnectionParams(resources[i]);
		}

	}

	/**
	 * @return A list of services identified by the string encodedService
	 */
	protected void decodeConnectionParams(String encodedService) {
		String[] tmp = encodedService.split(ID_SEPERATOR);
		String id = tmp[0];

		String[] entries = tmp[1].split(CONNECTION_PARAM);
		Map<String, Serializable> map = decodeConnectionParams(entries);
        URL url = decodeServiceURL(id, map);
		connectionParams.put(url,map);
	}

	/**
	 * @return a map of the connection parameters obtained from parsing the
	 *         encodedParams field
	 */
	private Map<String, Serializable> decodeConnectionParams(
			String[] encodedParams) {
		Map<String, Serializable> map = new HashMap<String, Serializable>();
		for (int j = 0; j < encodedParams.length; j++) {
			String[] entry = encodedParams[j].split(KEY_VALUE_SEPERATOR);
			if (entry[0].equalsIgnoreCase("URL")) //$NON-NLS-1$
				try {
					entry[1] = decodeURL(entry[1]);
				} catch (UnsupportedEncodingException e) {
					// do nothing
				}
                if( entry.length==2 )
                    map.put(entry[0], entry[1]);
		}
		return map;
	}

	private URL decodeServiceURL(String id, Map<String, Serializable> map) {
		URL url = null;
		try {
			url = new URL(decodeURL(id));
		} catch (Exception e) {
			try {
				url = new URL(decodeURL((String) map.get("url"))); //$NON-NLS-1$
			} catch (Exception e2) {
				e.printStackTrace();
				return null;
			}
		}
		return url;
	}

	/**
	 * @return a url from the string url
	 * @throws UnsupportedEncodingException
	 */
	private String decodeURL(String url) throws UnsupportedEncodingException {
		return URLDecoder.decode(url, ENCODING);
	}

	/**
     * @return   Returns the layer.
     * @uml.property   name="layer"
     */
	public Layer getLayer() {
		return layer;
	}

	/**
     * @param layer   The layer to set.
     * @uml.property   name="layer"
     */
	public void setLayer(Layer layer) {
		this.layer = layer;
	}

    /**
     * Adds the required services into the catalog.
     */
    public void load(){
        ICatalog catalog = CatalogPlugin.getDefault().getLocalCatalog();
        IServiceFactory serviceFactory = CatalogPlugin.getDefault().getServiceFactory();
        for( Map.Entry<URL,Map<String,Serializable>> entry : connectionParams.entrySet() ) {
            for( IService service : serviceFactory.aquire(entry.getKey(), entry.getValue()) ) {
                if (catalog.findService(service.getIdentifier()).isEmpty())
                    catalog.add(service);
            }
        }

        loaded=true;
    }
    
    /**
     * Returns true if the required GeoResources have been added to the catalog.
     * @return
     * @uml.property   name="loaded"
     */
    public boolean isLoaded() {
        return loaded;
    }
}
