/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal;

import java.util.List;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.ecore.EObject;
import org.geotools.filter.Filter;

import com.vividsolutions.jts.geom.Envelope;

/**
 * TODO Purpose of net.refractions.udig.project.internal <p> </p>
 * @author   Jesse
 * @since   1.0.0
 * @model
 */
public interface ContextModel extends EObject {
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	String copyright = "uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details."; //$NON-NLS-1$

	/**
	 * Select all selectable layers, with a BoundingBox Geometry filter. Boundingbox must be in the
	 * same CRS as the ViewportModel
	 * <p>
	 * Any Tool that wishes to be undoable would will need to remember the previous state.
	 * </p>
	 * 
	 * @param boundingBox the bounding box in Viewportmodel CRS to create a filter with
	 */
	public void select(Envelope boundingBox);

	/**
	 * Select all selectable layers, with a BoundingBox Geometry filter. Boundingbox must be in the
	 * same CRS as the ViewportModel
	 * <p>
	 * Any Tool that wishes to be undoable would will need to remember the previous state.
	 * </p>
	 * 
	 * @param boundingBox the bounding box in Viewportmodel CRS to create a filter with
	 * @param and true adds (or with current filter)to the current selection, false removes from the
	 *        current selection(and with current filter).
	 */
	public void select(Envelope boundingBox, boolean and);

	/**
	 * Select all selectable layers, with the provided filter.
	 * <p>
	 * Any Tool that wishes to be undoable would will need to remember the previous state.
	 * </p>
	 * 
	 * @param filter new selection filter
	 */
	public void select(Filter filter);

	/**
	 * Combines selection, from the selectable layers, with the provided filter.
	 * <p>
	 * Any Tool that wishes to be undoable would will need to remember the previous state.
	 * </p>
	 * 
	 * @param filter the new filter
	 * @param and true adds (or with current filter)to the current selection, false removes from the
	 *        current selection(and with current filter).
	 */
	public void select(Filter filter, boolean and);

	/**
	 * Gets the complete set of Layer objects. The order of the objects represents the the z-order
	 * in which they are renderered
	 * 
	 * @return A List of all the Layers in the object
	 * @model type="Layer" containment="true" opposite="contextModel"
	 */
	public List<Layer> getLayers();

	/**
	 * Gets the complete set of Layer objects. The order of the objects represents the the z-order
	 * in which they are renderered
	 * 
	 * @return A List of all the Layers in the object
	 * @see getLayers for the EMF data model
	 */
	public List<Layer> layers();

	/**
	 * Gets the owning map object.
	 * 
	 * @return the owning map object.
	 * @model opposite="contextModel" many="false"
	 */
	public Map getMap();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.ContextModel#getMap <em>Map</em>}' container reference.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>Map</em>' container reference.
	 * @see #getMap()
	 * @generated
	 */
	void setMap(Map value);

	/**
	 * Returns the value of the '<em><b>Deep Adapters</b></em>' attribute. <!-- begin-user-doc
	 * -->
	 * <p>
	 * If the meaning of the '<em>Deep Adapters</em>' attribute list isn't clear, there really
	 * should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * 
	 * @return the value of the '<em>Deep Adapters</em>' attribute.
	 * @see net.refractions.udig.project.internal.ProjectPackage#getContextModel_DeepAdapters()
	 * @model dataType="org.eclipse.emf.common.notify.Adapter" transient="true" many="true"
	 */
	List<Adapter> getDeepAdapters();

	/**
	 * Increases the ZOrder of the layer so it is rendered earlier with incomparison to the other
	 * layers. If the Layer is at the bottom of the render list(first to be drawn) it is not
	 * affected.
	 * 
	 * @param layer The layer whose rendering order will be modified
	 * @model
	 */
	public void lowerLayer(Layer layer);

	/**
	 * Decreases the ZOrder of the layer so it is rendered later with incomparison to the other
	 * layers. If the Layer is at the top of the render list(last to be drawn) it is not affected.
	 * 
	 * @param layer The layer whose rendering order will be modified
	 * @model
	 */
	public void raiseLayer(Layer layer);

}
