/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal.commands.edit;

import net.refractions.udig.core.internal.FeatureUtils;
import net.refractions.udig.project.command.Command;
import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.internal.Policy;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.data.FeatureReader;
import org.geotools.data.FeatureResults;
import org.geotools.data.FeatureStore;
import org.geotools.feature.Feature;
import org.geotools.feature.FeatureType;
import org.geotools.filter.Filter;
import org.geotools.filter.FilterFactory;

import com.vividsolutions.jts.geom.Envelope;

/**
 * Modifies an existing feature so it is the same as the provided feature.  The feature must be from the 
 * current edit layer otherwise nothing will happen.
 * 
 * @author Jesse
 * @since 1.0.0
 */
public class ModifyEditFeatureCommand extends AbstractEditCommand implements MapCommand {

    private Feature editedFeature;

    /**
     * Create a new instance.
     * 
     * @param editedFeature feature to modify.
     */
    public ModifyEditFeatureCommand( Feature editedFeature ) {
        this.editedFeature=editedFeature;
    }

    public void run( IProgressMonitor monitor ) throws Exception {
        
        if( FeatureUtils.same( editFeature, editedFeature)==0 )
            return;
        
        editLayer = getMap().getEditManagerInternal().getEditLayerInternal();
        if ( editedFeature == null || editLayer == null ) return;
        FeatureType featureType = editedFeature.getFeatureType();
        FilterFactory factory = FilterFactory.createFilterFactory();
        FeatureStore store = editLayer.getResource(FeatureStore.class, null);
        Filter filter = factory.createFidFilter(editedFeature.getID());
        FeatureResults results = store.getFeatures(filter);

        FeatureReader reader = results.reader();
        Envelope dirtyArea = new Envelope(editedFeature.getBounds());
        try {
            if (reader.hasNext()) {
                try {
                    Feature oldFeature = reader.next();
                    if( FeatureUtils.same( oldFeature, editedFeature)==0 )
                        return;
                    dirtyArea.expandToInclude(oldFeature.getBounds());
                    store.modifyFeatures(featureType.getAttributeTypes(), editedFeature
                            .getAttributes(new Object[featureType.getAttributeCount()]), filter);
                    editLayer.refresh(dirtyArea);
                } catch (Exception e) {
                    getMap().getEditManagerInternal().addFeature(editedFeature, editLayer);
                }
                
            }
        } finally {
            if( reader!=null )
                reader.close();
        }
    }

    public Command copy() {
        return new ModifyEditFeatureCommand(editedFeature);
    }

    public String getName() {
        return Policy.bind("ModifyEditFeatureCommand.0"); //$NON-NLS-1$
    }

}
