/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal.commands.edit;

import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.internal.Policy;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.data.FeatureReader;
import org.geotools.data.FeatureResults;
import org.geotools.data.FeatureStore;
import org.geotools.feature.Feature;
import org.geotools.feature.FeatureType;
import org.geotools.filter.Filter;
import org.geotools.filter.FilterFactory;

import com.vividsolutions.jts.geom.Envelope;

/**
 * Compares the editFeature with the corresponding feature in the data store (The feature with the
 * same fid) and replaces the old feature with the edit feature. If the feature is a new feature
 * then the feature is added to the datastore.
 * 
 * @author jeichar
 * @since 0.3
 */
public class WriteEditFeatureCommand extends AbstractEditCommand {
    /**
     * @see net.refractions.udig.project.internal.command.MapCommand#run()
     */
    public void run( IProgressMonitor monitor ) throws Exception {
        editFeature = getMap().getEditManagerInternal().getEditFeature();
        editLayer = getMap().getEditManagerInternal().getEditLayerInternal();
        if ( editFeature == null || editLayer == null ) return;
        FeatureType featureType = editFeature.getFeatureType();
        FilterFactory factory = FilterFactory.createFilterFactory();
        FeatureStore store = editLayer.getResource(FeatureStore.class, null);
        Filter filter = factory.createFidFilter(editFeature.getID());
        FeatureResults results = store.getFeatures(filter);
        
        FeatureReader reader = results.reader();
        Envelope dirtyArea = new Envelope(editFeature.getBounds());
        try {
            if (reader.hasNext()) {
                try {
                    Feature oldFeature = reader.next();
                    dirtyArea.expandToInclude(oldFeature.getBounds());
                    store.modifyFeatures(featureType.getAttributeTypes(), editFeature
                            .getAttributes(new Object[featureType.getAttributeCount()]), filter);
//                    getMap().getEditManagerInternal().setEditFeature(null, editLayer);
//                    editLayer.refresh(dirtyArea);
                } catch (Exception e) {
                    getMap().getEditManagerInternal().addFeature(editFeature, editLayer);
                }
                
            } else {
                getMap().getEditManagerInternal().addFeature(editFeature, editLayer);
            }
        } finally {
            if( reader!=null )
                reader.close();
        }
    }

    /**
     * @see net.refractions.udig.project.internal.command.MapCommand#copy()
     */
    public MapCommand copy() {
        return new WriteEditFeatureCommand();
    }

    /**
     * @see net.refractions.udig.project.command.MapCommand#getName()
     */
    public String getName() {
        return Policy.bind("SetEditFeatureCommand.setCurrentEditFeature"); //$NON-NLS-1$
    }

}
