package net.refractions.udig.project.internal.impl;

import java.io.IOException;
import java.util.Set;

import net.refractions.udig.project.internal.Policy;

import org.geotools.data.DataStore;
import org.geotools.data.FeatureListener;
import org.geotools.data.FeatureReader;
import org.geotools.data.FeatureStore;
import org.geotools.data.Query;
import org.geotools.data.Transaction;
import org.geotools.feature.AttributeType;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureType;
import org.geotools.filter.Filter;

import com.vividsolutions.jts.geom.Envelope;

/**
 * A FeatureStore decorator that does not allow the transaction to be set more than once.
 * (Only if the current transaction is "AUTO_COMMIT" can the transaction be set)
 * 
 * @author jones
 * @since 1.0.0
 */
public class UDIGFeatureStore implements FeatureStore {

    FeatureStore wrapped;
    
    /**
     * Create a new FeatureStore decorator that does not allow the transaction to be set more than once.
     * (Only if the current transaction is "AUTO_COMMIT" can the transaction be set)
     * 
     * @param store the feature store that will be decorated
     */
    public UDIGFeatureStore( FeatureStore store ) {
        wrapped=store;
    }

    public Set addFeatures( FeatureReader arg0 ) throws IOException {
        return wrapped.addFeatures(arg0);
    }

    public void removeFeatures( Filter arg0 ) throws IOException {
        wrapped.removeFeatures(arg0);
    }

    public void modifyFeatures( AttributeType[] arg0, Object[] arg1, Filter arg2 )
            throws IOException {
        wrapped.modifyFeatures(arg0, arg1, arg2);
    }

    public void modifyFeatures( AttributeType arg0, Object arg1, Filter arg2 )
            throws IOException {
        wrapped.modifyFeatures(arg0, arg1,arg2);
    }

    public void setFeatures( FeatureReader arg0 ) throws IOException {
        wrapped.setFeatures(arg0);
    }

    public void setTransaction( Transaction arg0 ) {
        if (wrapped.getTransaction()!=Transaction.AUTO_COMMIT || !(arg0 instanceof UDIGTransaction) )
            throw new IllegalArgumentException( Policy.bind("UDIGFeatureStore.0") + //$NON-NLS-1$
                    Policy.bind("UDIGFeatureStore.1") ); //$NON-NLS-1$
        wrapped.setTransaction(arg0);
    }

    public Transaction getTransaction() {
        return wrapped.getTransaction();
    }

    public DataStore getDataStore() {
        return wrapped.getDataStore();
    }

    public void addFeatureListener( FeatureListener arg0 ) {
        wrapped.addFeatureListener(arg0);
    }

    public void removeFeatureListener( FeatureListener arg0 ) {
        wrapped.removeFeatureListener(arg0);
    }

    public FeatureCollection getFeatures( Query arg0 ) throws IOException {
        return wrapped.getFeatures(arg0);
    }

    public FeatureCollection getFeatures( Filter arg0 ) throws IOException {
        return wrapped.getFeatures(arg0);
    }

    public FeatureCollection getFeatures() throws IOException {
        return wrapped.getFeatures();
    }

    public FeatureType getSchema() {
        return wrapped.getSchema();
    }

    public Envelope getBounds() throws IOException {
        return wrapped.getBounds();
    }

    public Envelope getBounds( Query arg0 ) throws IOException {
        return wrapped.getBounds(arg0);
    }

    public int getCount( Query arg0 ) throws IOException {
        return wrapped.getCount(arg0);
    }

}