/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.project.internal.provider;

import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.PropertyPage;
import org.geotools.referencing.FactoryFinder;
import org.opengis.metadata.Identifier;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.crs.CRSAuthorityFactory;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

/**
 * Creates a Control for choosing a Coordinate Reference System.
 * 
 * @author jeichar
 * @since 0.6.0
 */
public class CRSChooser {
    
    ListViewer codesList;
    Text searchText;
    Text wktText;
    CoordinateReferenceSystem selectedCRS;
    Matcher matcher;
    boolean painted;
	private TabFolder folder;
    private PropertyPage parentPage;
    
    public CRSChooser(PropertyPage parentPage){
        matcher = Pattern.compile(".*?\\(([^(]*)\\)$").matcher(""); //$NON-NLS-1$ //$NON-NLS-2$
        this.parentPage = parentPage;
    }
    
    private Control createCustomCRSControl(Composite parent){
        Composite composite = new Composite(parent, SWT.NONE);
        
        GridLayout layout = new GridLayout();
        composite.setLayout(layout);
        
        GridData gridData = new GridData();
        Label editorLabel = new Label(composite, SWT.NONE);
        editorLabel.setText(Policy.bind("CRSChooser.label.crsWKT")); //$NON-NLS-1$
        editorLabel.setLayoutData(gridData);
        
        gridData = new GridData(SWT.FILL,SWT.FILL,true,true);
        wktText = new Text(composite, SWT.MULTI|SWT.BORDER|SWT.V_SCROLL|SWT.H_SCROLL);
        if(selectedCRS!=null)
            wktText.setText(selectedCRS.toWKT());
        wktText.setLayoutData(gridData);
        return composite;
    }
    
    private Control createStandardCRSControl(Composite parent){
        Composite composite = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        composite.setLayout(layout);
        
        GridData gridData = new GridData();
        Label codesLabel = new Label(composite, SWT.NONE);
        codesLabel.setText(Policy.bind("CRSChooser.label.crs")); //$NON-NLS-1$
        codesLabel.setLayoutData(gridData);
        
        gridData = new GridData(SWT.FILL,SWT.FILL,false,false);
        searchText = new Text(composite, SWT.SINGLE | SWT.BORDER);
        searchText.setLayoutData(gridData);
        searchText.addModifyListener(new ModifyListener(){
            public void modifyText( ModifyEvent e ) {
                fillCodesList();
            }
        });

//        gridData = new GridData(SWT.FILL,400,true, true);
        gridData = new GridData(400, 300);
        codesList = new ListViewer(composite);
        codesList.setContentProvider(new ArrayContentProvider());
        codesList.setLabelProvider(new LabelProvider());
        codesList.addSelectionChangedListener(new ISelectionChangedListener(){

            public void selectionChanged( SelectionChangedEvent event ) {
                selectedCRS = null;
                String crsCode = (String) ((IStructuredSelection) codesList.getSelection())
                        .getFirstElement();
                if( crsCode == null ) return;
                matcher.reset(crsCode);
                if (matcher.matches()) {
                    selectedCRS = createCRS(matcher.group(1));
                    if (selectedCRS != null && wktText != null)
                        wktText.setText(selectedCRS.toWKT());
                }

            }

        });
        
        codesList.addDoubleClickListener(new IDoubleClickListener(){

            public void doubleClick( DoubleClickEvent event ) {
                parentPage.performOk();
                parentPage.getControl().getShell().close();
            }
            
        });
        		
        		
//        		new SelectionAdapter(){
//            public void widgetDefaultSelected( SelectionEvent e ) {
//                widgetSelected(e);
//            }
//
//            public void widgetSelected( SelectionEvent e ) {
//                selectedCRS = null;
//                String crsCode = codesList.getItem(codesList.getSelectionIndex());
//                matcher.reset(crsCode);
//                if(matcher.matches()){
//                    selectedCRS = createCRS(matcher.group(1));
//                    if(selectedCRS!=null)
//                        wktText.setText(selectedCRS.toWKT());
//                }
//            }
//        });
        codesList.getControl().setLayoutData(gridData);
        /*
         * fillCodesList() by itself resizes the Preferences Page
         * but in the paintlistener it flickers the window
         */
      fillCodesList();
//        painted=false;
//        codesList.addPaintListener(new PaintListener(){
//            public void paintControl( PaintEvent e ) {
//                if(!painted){
//                    painted=true;
//                    fillCodesList();
//                }
//            }
//        });
        
        return composite;
    }
    
    /**
     * 
     * Creates the CRS PreferencePage root control with a CRS already selected
     * 
     * @param parent PreferencePage for this chooser
     * @param crs current CRS for the associated map
     * @return control for the PreferencePage
     */
    public Control createControl(Composite parent, CoordinateReferenceSystem crs){
        Control control = createControl(parent);
        selectedCRS = crs;
        gotoCRS(selectedCRS);
        return control;
    }
    
    public void clearSearch() {
        searchText.setText("");
    }
    
    /**
     * Takes in a CRS, finds it in the list and highlights it
     * 
     * @param crs
     */
    public void gotoCRS(CoordinateReferenceSystem crs) {
        if( crs != null ){
            List list = codesList.getList();
            Set<Identifier> identifiers = crs.getIdentifiers();
            
            for( int i = 0; i < list.getItemCount(); i++ ) {
                for( Identifier identifier : identifiers ) {
                    if (list.getItem(i).contains(identifier.toString())) {// selectedCRS.getIdentifiers()[0].toString()))
                                                                            // {
                        codesList.setSelection(new StructuredSelection(list.getItem(i)));
                        break;
                    }
                }
            }
        }
    }
    
    /**
     * 
     * Creates the CRS PreferencePage root control with no CRS selected
     * 
     * @param parent PreferencePage for this chooser
     * @return control for the PreferencePage
     */
    public Control createControl(Composite parent){
        GridData gridData = null;

        gridData = new GridData(SWT.FILL,SWT.FILL,true,true);
        folder = new TabFolder(parent,SWT.NONE);
        folder.setLayoutData(gridData);
        
        TabItem standard = new TabItem(folder,SWT.NONE);
        standard.setText(Policy.bind("CRSChooser.tab.standardCRS")); //$NON-NLS-1$
        Control stdCRS = createStandardCRSControl(folder);
        standard.setControl(stdCRS);
        
        TabItem custom = new TabItem(folder,SWT.NONE);
        custom.setText(Policy.bind("CRSChooser.tab.customCRS")); //$NON-NLS-1$
        Control cstCRS = createCustomCRSControl(folder);
        custom.setControl(cstCRS);
        
        return folder;
    }
    
    /**
     * 
     * checks if all keywords in filter array are in input
     * 
     * @param input test string 
     * @param filter array of keywords
     * @return true, if all keywords in filter are in the input, false otherwise
     */
    protected boolean matchesFilter(String input, String[] filter){
        for(String match: filter){
            if(!input.contains(match))
               return false; 
        }
        return true;
    }
    
    /**
     * 
     * filters all CRS Names from all available CRS authorities
     * 
     * @param filter array of keywords
     * @return Set of CRS Names which contain all the filter keywords
     */
    protected Set<String> filterCRSNames(String[] filter){
        Set<String> descriptions = new TreeSet<String>();
        for (Object object : FactoryFinder.getCRSAuthorityFactories()) {
            CRSAuthorityFactory factory = (CRSAuthorityFactory) object;
        
            try {
                Set<String> codes = factory.getAuthorityCodes(CoordinateReferenceSystem.class);
                for( Object codeObj: codes){
                    String code = (String)codeObj;
                    String description;
                    try{
                        description = factory.getDescriptionText(code).toString();
                    }catch(Exception e1){
                        description = Policy.bind("CRSChooser.unnamed"); //$NON-NLS-1$
                    }
                    description += " ("+code+")"; //$NON-NLS-1$ //$NON-NLS-2$
                    if(matchesFilter(description.toUpperCase(),filter))
                        descriptions.add(description);
                }
            } catch (FactoryException e) {
                //list = null;
            }
        }
        return descriptions;
    }
    
    /**
     * 
     * populates the codes list with a filtered list of CRS names
     *
     */
    protected void fillCodesList(){
        String[] searchParms = searchText.getText().toUpperCase().split(" "); //$NON-NLS-1$
        Set<String> descriptions = filterCRSNames(searchParms);
        String list[]= descriptions.toArray(new String[descriptions.size()]);
        codesList.setInput(list);
        if( list.length==1)
        	codesList.setSelection(new StructuredSelection(list[0]));
    }
    
    /**
     * 
     * creates a CRS from a code when the appropriate CRSAuthorityFactory is unknown
     * 
     * @param code CRS code
     * @return CRS object from appropriate authority, or null if the appropriate factory cannot be determined
     */
    protected CoordinateReferenceSystem createCRS(String code){
        if(code==null)
            return null;
        for (Object object : FactoryFinder.getCRSAuthorityFactories()) {
            CRSAuthorityFactory factory = (CRSAuthorityFactory) object;
            try {
                return (CoordinateReferenceSystem) factory.createObject(code);
            } catch (FactoryException e2) {
                //then we have the wrong factory
                //is there a better way to do this?
            }
        }
        return null; //should throw an exception?
    }

    /**
     * 
     * returns the selected CRS
     * 
     * @return selected CRS
     */
    public CoordinateReferenceSystem getCRS(){
        if( folder==null )
            return selectedCRS;
    	if(folder.getSelectionIndex()==1){
    		try {
				return FactoryFinder.getCRSFactory(null).createFromWKT(wktText.getText());
			} catch (Exception e) {
				ProjectEditPlugin.log( null, e);
			} 
    	}
    	if( selectedCRS==null ){
    		return createCRS(searchText.getText());
    	}
        return selectedCRS;
    }
    
}