package net.refractions.udig.project.internal.render;

import java.awt.Color;
import java.io.IOException;
import java.net.URL;

import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.project.StyleContent;
import net.refractions.udig.project.internal.Layer;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.IMemento;
import org.geotools.styling.FeatureTypeStyle;
import org.geotools.styling.Graphic;
import org.geotools.styling.Mark;
import org.geotools.styling.PointSymbolizer;
import org.geotools.styling.Stroke;
import org.geotools.styling.Style;
import org.geotools.styling.StyleBuilder;
import org.geotools.styling.Symbolizer;

import com.vividsolutions.jts.geom.Point;

/**
 * The style used for Selection
 * @author jeichar
 * @since 0.6.0
 */
public class SelectionStyleContent extends StyleContent {

	/** <code>ID</code> field */
	public static final String ID = "net.refractions.udig.project.selectionStyle"; //$NON-NLS-1$

	private static final StyleBuilder builder = new StyleBuilder();

	private static final String fTypeName = "fTypeName"; //$NON-NLS-1$

	/**
	 * Construct <code>SelectionStyleContent</code>.
	 */
	public SelectionStyleContent() {
		super(ID);
	}

	/**
	 * Returns a geotools style object.
	 */
	public Class getStyleClass() {
		return Style.class;
	}

	/**
	 * Does nothing.
	 */
	public void save(IMemento momento, Object value) {
		Style style = (Style) value;

		// store the feature type name
		momento.putString(fTypeName, style.getFeatureTypeStyles()[0]
				.getFeatureTypeName());
	}

	public Object load(IMemento momento) {
		Style style = createDefaultStyle(null);
		style.getFeatureTypeStyles()[0].setFeatureTypeName(momento
				.getString(fTypeName));

		return style;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.refractions.udig.project.StyleContent#createDefaultStyle(net.refractions.udig.catalog.IGeoResource)
	 */
    public Object createDefaultStyle(IGeoResource resource, Color colour,
           IProgressMonitor monitor) {
//	public Object createDefaultStyle(IGeoResource resource, ColourScheme scheme,
//			int layerIndex, IProgressMonitor monitor) {
		return null;
	}

	public static Style createDefaultStyle(Layer layer) {
		Style style = builder.createStyle();
        
		Stroke stroke = builder.createStroke(Color.ORANGE, 2);
		stroke.setDashArray(new float[] { 4, 4 });
		Symbolizer sym = builder.createLineSymbolizer(stroke);
        
        FeatureTypeStyle fs = builder.createFeatureTypeStyle(sym);
        style.addFeatureTypeStyle(fs);
        
        if( layer != null ) {
            if( layer.getSchema().getDefaultGeometry().getType() == Point.class ) {
                Mark[] mark = new Mark[1];
                mark[0] = builder.createMark("square"); //$NON-NLS-1$
                mark[0].setFill(builder.createFill(Color.PINK));
                //mark[0].getFill().setOpacity(builder.literalExpression(0.4));
                mark[0].setStroke(builder.createStroke(Color.ORANGE, 2, new float[]{ 1, 1 }));
                //mark[0].getStroke().setOpacity(builder.literalExpression(0.4));
                PointSymbolizer pointSymbolizer = builder.createPointSymbolizer();
                Graphic g = pointSymbolizer.getGraphic();
                g.setMarks(mark);
                
                fs = builder.createFeatureTypeStyle(pointSymbolizer);
                style.addFeatureTypeStyle(fs);
            }
        }
        

//        Fill fill = builder.createFill();
//        fill.setColor(builder.colorExpression(Color.ORANGE));
//        fill.setOpacity(builder.literalExpression(0.5));
//        Stroke polyStroke = builder.createStroke(Color.ORANGE, 1);
//        polyStroke.setDashArray(new float[] { 4, 4 });
//        PolygonSymbolizer polygonSymbolizer = builder.createPolygonSymbolizer();
//        polygonSymbolizer.setStroke(polyStroke);
//        polygonSymbolizer.setFill(fill);

//        fs = builder.createFeatureTypeStyle(polygonSymbolizer);
//        style.addFeatureTypeStyle(fs);

		return style;
	}

	/*
	 * @see net.refractions.udig.project.StyleContent#load(java.net.URL)
	 */
	public Object load(URL url, IProgressMonitor monitor) throws IOException {
		return null;
	}

}