/**
 * <copyright></copyright> $Id: RenderExecutorImpl.java 16014 2005-10-04 17:53:49Z jeichar $
 */
package net.refractions.udig.project.internal.render.impl;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Policy;
import net.refractions.udig.project.internal.ProjectPackage;
import net.refractions.udig.project.internal.ProjectPlugin;
import net.refractions.udig.project.internal.render.ExecutorVisitor;
import net.refractions.udig.project.internal.render.RenderContext;
import net.refractions.udig.project.internal.render.RenderExecutor;
import net.refractions.udig.project.internal.render.RenderPackage;
import net.refractions.udig.project.internal.render.Renderer;
import net.refractions.udig.project.internal.render.SelectionLayer;
import net.refractions.udig.project.render.IRenderContext;
import net.refractions.udig.project.render.IRenderer;
import net.refractions.udig.project.render.RenderException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.ENotificationImpl;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;

/**
 * Runs a renderer in its own thread. Is responsible for stopping the thread.
 * Does not do composite renderer or multilayer renderers.
 * 
 * @author Jesse
 * @since 1.0.0
 * @generated
 */
public class RenderExecutorImpl extends RendererImpl implements RenderExecutor {
	/**
	 * Listens to a layer for visibility events and styling events.
	 * 
	 * <b>Public ONLY for testing purposes</b>
	 * 
	 * @author Jesse
	 * @since 1.0.0
	 */
	public static class LayerListener extends AdapterImpl {
		protected RenderExecutorImpl executor;

		LayerListener(RenderExecutorImpl executor) {
			this.executor = executor;
		}

		/**
		 * @see org.eclipse.emf.common.notify.impl.AdapterImpl#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
		public void notifyChanged(Notification msg) {
			Layer layer = (Layer) msg.getNotifier();
			switch (msg.getFeatureID(Layer.class)) {
			case ProjectPackage.LAYER__STYLE_BLACKBOARD:
				if (executor.getContext().getLayer() instanceof SelectionLayer)
					return;

				styleBlackboardChanged(msg);
				break;
			case ProjectPackage.LAYER__VISIBLE:
				if (executor.getContext().getLayer() instanceof SelectionLayer)
					return;
				if (msg.getNewBooleanValue())
					layerVisible(layer, msg);
				else
					layerNotVisible(layer, msg);
				break;

			default:
				break;
			}
		}

		protected void layerNotVisible(Layer layer, Notification msg) {
			if (executor.getState() == RENDERING) {
				executor.stopRendering();
				executor.getContext().setStatus(ILayer.DONE);
				executor.getContext().setStatusMessage(""); //$NON-NLS-1$
				executor.setState(NEVER);
			} else
				executor.setState(DONE);
		}

		protected void layerVisible(Layer layer, Notification msg) {
			if (executor.getState() == Renderer.RENDERING)
				return;
			if (executor.getState() != Renderer.DONE || executor.dirty) {
				executor.getRenderer().setState(RENDER_REQUEST);
			} else
				executor.setState(Renderer.DONE);

		}

		protected void styleBlackboardChanged(Notification msg) {
			executor.getContext().getRenderManager().refresh(
					(ILayer) msg.getNotifier(), null);
		}
	}

	/**
	 * TODO Purpose of net.refractions.udig.project.internal.render.impl
	 * <p>
	 * </p>
	 * 
	 * @author Jesse
	 * @since 1.0.0
	 */
	protected static class RendererListener extends AdapterImpl {

		protected RenderExecutor executor;

		RendererListener(RenderExecutor executor) {
			this.executor = executor;
		}

		/**
		 * @see org.eclipse.emf.common.notify.impl.AdapterImpl#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
		public void notifyChanged(Notification msg) {
			if (msg.getNotifier() instanceof Renderer) {
				if (msg.getFeatureID(Renderer.class) == RenderPackage.RENDERER__STATE)
					stateChanged(msg);
			}

		}

		protected void stateChanged(Notification msg) {
			if (msg.getNewIntValue() == Renderer.RENDER_REQUEST)
				try {
					executor.render((Envelope) null);
				} catch (RenderException e) {
					// won't happen
					ProjectPlugin.log(null, e);
				}

			executor.setState(msg.getNewIntValue());
		}
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public static final String copyright = "uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details."; //$NON-NLS-1$

	/**
	 * TODO Purpose of net.refractions.udig.project.internal.render.impl
	 * <p>
	 * </p>
	 * 
	 * @author Jesse
	 * @since 1.0.0
	 */
	protected static class RenderJob extends Job {
		private IProgressMonitor monitor;

		protected RenderExecutor executor;

		Envelope bounds;

		/**
		 * Creates an new instance of AbstractRenderer.RenderThread
		 */
		public RenderJob(RenderExecutor executor) {
			super(Policy.bind("RenderExecutorImpl.title")); //$NON-NLS-1$
			setPriority(Job.LONG);
			this.executor = executor;
		}

		/**
		 * @return Returns the executor.
		 * @uml.property name="executor"
		 */
		public RenderExecutor getExecutor() {
			return executor;
		}

		/**
		 * Returns the progress monitor or null if not rendering.
		 * 
		 * @return the progress monitor or null if not rendering.
		 * @uml.property name="monitor"
		 */
		public IProgressMonitor getMonitor() {
			return monitor;
		}

		protected void startRendering(Envelope bounds, IProgressMonitor monitor)
				throws Throwable {
			executor.getContext().setStatus(Layer.WAIT);
			executor.getContext().setStatusMessage(
					Policy.bind("RenderExecutorImpl.0")); //$NON-NLS-1$
			executor.getRenderer().setState(Renderer.RENDERING);

			if (bounds != null
					&& !bounds.contains(executor.getContext()
							.getViewportModel().getBounds())) {
				Point min = executor.getContext().worldToPixel(
						new Coordinate(bounds.getMinX(), bounds.getMinY()));
				Point max = executor.getContext().worldToPixel(
						new Coordinate(bounds.getMaxX(), bounds.getMaxY()));
				int width = Math.abs(max.x - min.x);
				int height = Math.abs(max.y - min.y);
				Rectangle paintArea = new Rectangle(Math.min(min.x, max.x),
						Math.min(min.y, max.y), width, height);
				executor.getContext().clearImage(paintArea);
				executor.getRenderer().render(bounds, monitor);
			} else {
				executor.getContext().clearImage();
				executor.getRenderer().render((Envelope) null, monitor);
			}
		}

		protected void handleException(Throwable renderError) {
			try {
				ILayer layer = executor.getContext().getLayer();
				ProjectPlugin.getPlugin().log(renderError);
				if (layer != null) {
					executor.getContext().setStatus(Layer.ERROR); // indicate
																	// to
					// user
					executor
							.getContext()
							.setStatusMessage(
									Policy.bind("RenderExecutorImpl.1") + renderError.getLocalizedMessage()); //$NON-NLS-1$
				}
			} catch (Exception e) {
				ProjectPlugin.log(renderError.getLocalizedMessage(),
						renderError);
				ProjectPlugin.trace("Error handling Exception", e); //$NON-NLS-1$
			}
		}

		protected void postRendering() {
			if (monitor.isCanceled()) {
				executor.setState(RENDERING);
				executor.setState(NEVER);
				return;
			}
			if (executor.getRenderer().getState() != IRenderer.DONE)
				executor.getRenderer().setState(IRenderer.DONE);
			else
				executor.setState(Renderer.DONE);

			// resyncState(getRenderer());
		}

		/**
		 * @see org.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.IProgressMonitor)
		 */
		protected IStatus run(IProgressMonitor monitor) {
			this.monitor = monitor;

			try {
				startRendering(bounds, monitor);
			} catch (Throwable renderError) {

				handleException(renderError);
				getExecutor().getRenderer().setState(Renderer.DONE);
				return Status.OK_STATUS;
			}

			postRendering();
			return Status.OK_STATUS;

		}

		@Override
		public boolean belongsTo(Object family) {
			return family == executor.getContext().getRenderManager();
		}

		/**
		 * @param object
		 * @uml.property name="bounds"
		 */
		public void setBounds(Envelope bounds) {
			this.bounds = bounds;
		}
	}

	/**
	 * The cached value of the '{@link #getRenderer() <em>Renderer</em>}'
	 * reference. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getRenderer()
	 * @generated
	 * @ordered
	 */
	protected Renderer renderer = null;

	protected Adapter renderListener = getRendererListener();

	/**
	 * A listener that triggers an update when the style of its layer changes or
	 * the visibility changes.
	 */
	protected Adapter layerListener = getLayerListener();

	protected RenderJob renderJob;

	boolean dirty;

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public RenderExecutorImpl() {
		super();
		renderJob = new RenderJob(this);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected EClass eStaticClass() {
		return RenderPackage.eINSTANCE.getRenderExecutor();
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public void dispose() {
		while (getRenderer().eAdapters().remove(renderListener))
			;
		eAdapters().clear();
		removeLayerListener(getContext());
		stopRendering();
		getRenderer().dispose();
		if (getRenderer().getState() != DISPOSED)
			getRenderer().setState(DISPOSED);
		getRenderer().setContext(null);
	}

	/**
	 * @see net.refractions.udig.project.internal.render.impl.RendererImpl#setState(int)
	 */
	public void setState(int newState) {
		super.setState(newState);
		resyncState(getRenderer());
	}

	protected void resyncState(Renderer renderer) {
		IRenderContext context = renderer.getContext();
		if (context == null)
			return;

		setLayerState(context, renderer.getState());
	}

	protected void setLayerState(IRenderContext context, int renderState) {
		ILayer layer = context.getLayer();
		if (layer == null)
			return;

		switch (renderState) {
		case NEVER:
		case RENDER_REQUEST:
			if (layer.getStatus() != Layer.WAIT) {
				context.setStatus(Layer.WAIT); // show paused clock
			}
			break;

		case RENDERING:
			context.setStatus(Layer.WORKING); // this will update the clock
			break;

		case DONE:
		case DISPOSED:
			// layer can get set with error, warn or something else
			// renderer pipeline is not so explicit
			//
			if (layer.getStatus() == Layer.WORKING) {
				// but if the layer thinks it is working, we can it at least
				// tell it is done
				context.setStatus(Layer.DONE);
				context.setStatusMessage(Policy.bind("RenderExecutorImpl.2")); //$NON-NLS-1$
			}
			break;
		default:
			// layer can get set with error, warn or something else
			// renderer pipeline is not so explicit
			//
			if (layer.getStatus() == Layer.WORKING) {
				// but if the layer thinks it is working, we can it at least
				// tell it is done
				context.setStatus(Layer.DONE);
			}
			break;
		}
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public void stopRendering() {
		if (renderJob.getMonitor() != null)
			renderJob.getMonitor().setCanceled(true);
		Thread.yield();
		Thread.yield();
		renderJob.cancel();
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @throws RenderException
	 * 
	 * @generated NOT
	 */
	public void render(Graphics2D destination, IProgressMonitor monitor)
			throws RenderException {

		if (getState() == DISPOSED)
			return;
		getRenderer().render(destination, validateMonitor(monitor));
	}

	private IProgressMonitor validateMonitor(IProgressMonitor monitor) {
		if (monitor != null)
			return monitor;

		return new NullProgressMonitor();
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public void render() {
		render((Envelope) null);
	}

	/**
	 * @see net.refractions.udig.project.internal.render.Renderer#getContext()
	 */
	public RenderContext getContext() {
		if (getRenderer() == null)
			return null;
		return (RenderContext) getRenderer().getContext();
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Renderer getRenderer() {
		return renderer;
	}

	void runRenderJob() {
		dirty = false;
		if (getContext().getMapDisplay() == null
				|| getContext().getMapDisplay().getWidth() < 1
				|| getContext().getMapDisplay().getHeight() < 1)
			return;
		if (RenderExecutorImpl.this.getState() == Renderer.DISPOSED)
			throw new RuntimeException("attempted to run a disposed renderer"); //$NON-NLS-1$

		if (RenderExecutorImpl.this.getState() == Renderer.RENDERING) {
			stopRendering();
		}
		if (renderJob.getMonitor() != null)
			renderJob.getMonitor().setCanceled(false);
		renderJob.setName(getRenderJobName());
		renderJob.schedule();
	}

	protected String getRenderJobName() {
		return MessageFormat
				.format(
						Policy.bind("RenderExecutorImpl.message"), new Object[] { getRenderer().getName() }); //$NON-NLS-1$
	}

	/**
	 * @see net.refractions.udig.project.internal.render.RenderExecutor#setRenderer(net.refractions.udig.project.render.Renderer)
	 * @uml.property name="renderer"
	 */
	@SuppressWarnings("unchecked")//$NON-NLS-1$
	public void setRenderer(Renderer newRenderer) {
		if (getRenderer() != null) {
			getRenderer().eAdapters().remove(renderListener);
			removeLayerListener((RenderContext) getRenderer().getContext());
		}
		setRendererGen(newRenderer);
		// registerFeatureListener();
		if (newRenderer != null) {
			newRenderer.eAdapters().add(renderListener);
			addLayerListener((RenderContext) newRenderer.getContext());
		}
	}

	@SuppressWarnings("unchecked") //$NON-NLS-1$
	protected void removeLayerListener(RenderContext context) {
		if (context.getLayerInternal() != null) {
			List<Adapter> adapters = context.getLayerInternal().eAdapters();
			List<Adapter> toRemove = new ArrayList<Adapter>();
			for (Adapter adapter : adapters) {
				if (adapter instanceof RenderExecutorImpl.LayerListener)
					toRemove.add(adapter);
			}
			adapters.removeAll(toRemove);
		}
	}

	@SuppressWarnings("unchecked")//$NON-NLS-1$
	protected void addLayerListener(RenderContext context) {

		if (context.getLayer() != null && !(context.getLayer() instanceof SelectionLayer) ) {
			removeLayerListener(context);
			context.getLayerInternal().eAdapters().add(layerListener);
		}
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setRendererGen(Renderer newRenderer) {
		Renderer oldRenderer = renderer;
		renderer = newRenderer;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET,
					RenderPackage.RENDER_EXECUTOR__RENDERER, oldRenderer,
					renderer));
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Object eGet(EStructuralFeature eFeature, boolean resolve) {
		switch (eDerivedStructuralFeatureID(eFeature)) {
		case RenderPackage.RENDER_EXECUTOR__STATE:
			return new Integer(getState());
		case RenderPackage.RENDER_EXECUTOR__NAME:
			return getName();
		case RenderPackage.RENDER_EXECUTOR__CONTEXT:
			return getContext();
		case RenderPackage.RENDER_EXECUTOR__RENDERER:
			return getRenderer();
		}
		return eDynamicGet(eFeature, resolve);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void eSet(EStructuralFeature eFeature, Object newValue) {
		switch (eDerivedStructuralFeatureID(eFeature)) {
		case RenderPackage.RENDER_EXECUTOR__STATE:
			setState(((Integer) newValue).intValue());
			return;
		case RenderPackage.RENDER_EXECUTOR__NAME:
			setName((String) newValue);
			return;
		case RenderPackage.RENDER_EXECUTOR__CONTEXT:
			setContext((IRenderContext) newValue);
			return;
		case RenderPackage.RENDER_EXECUTOR__RENDERER:
			setRenderer((Renderer) newValue);
			return;
		}
		eDynamicSet(eFeature, newValue);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void eUnset(EStructuralFeature eFeature) {
		switch (eDerivedStructuralFeatureID(eFeature)) {
		case RenderPackage.RENDER_EXECUTOR__STATE:
			setState(STATE_EDEFAULT);
			return;
		case RenderPackage.RENDER_EXECUTOR__NAME:
			setName(NAME_EDEFAULT);
			return;
		case RenderPackage.RENDER_EXECUTOR__CONTEXT:
			setContext((IRenderContext) null);
			return;
		case RenderPackage.RENDER_EXECUTOR__RENDERER:
			setRenderer((Renderer) null);
			return;
		}
		eDynamicUnset(eFeature);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public boolean eIsSet(EStructuralFeature eFeature) {
		switch (eDerivedStructuralFeatureID(eFeature)) {
		case RenderPackage.RENDER_EXECUTOR__STATE:
			return state != STATE_EDEFAULT;
		case RenderPackage.RENDER_EXECUTOR__NAME:
			return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT
					.equals(name);
		case RenderPackage.RENDER_EXECUTOR__CONTEXT:
			return context != null;
		case RenderPackage.RENDER_EXECUTOR__RENDERER:
			return renderer != null;
		}
		return eDynamicIsSet(eFeature);
	}

	/**
	 * @see net.refractions.udig.project.internal.render.impl.RendererImpl#getInfo(Point,
	 *      Layer)
	 * 
	 * public InfoList getInfo(Point screenLocation) throws IOException { if
	 * (!(getContext().getLayer() instanceof SelectionLayer)) return
	 * renderer.getInfo(screenLocation); return new InfoList(screenLocation.x,
	 * screenLocation.y, null); }
	 */

	/**
	 * @see net.refractions.udig.project.internal.render.RenderExecutor#visit(net.refractions.udig.project.render.ExecutorVisitor)
	 */
	public void visit(ExecutorVisitor visitor) {
		visitor.visit(this);
	}

	/**
	 * @return Returns the layerListener.
	 * @uml.property name="layerListener"
	 */
	protected LayerListener getLayerListener() {
		return new LayerListener(this);
	}

	protected RendererListener getRendererListener() {
		return new RendererListener(this);
	}

	/**
	 * @see RenderExecutor#render(Rectangle)
	 */
	public void render(Rectangle screenArea) {
		Coordinate min = getContext().pixelToWorld(screenArea.x, screenArea.y);
		Coordinate max = getContext().pixelToWorld(
				screenArea.width + screenArea.x,
				screenArea.height + screenArea.y);
		Envelope env = new Envelope(min, max);
		render(env);
	}

	/**
	 * @see net.refractions.udig.project.internal.render.Renderer#render(com.vividsolutions.jts.geom.Envelope)
	 */
	public void render(final Envelope bounds) {
		// Check the visibility of all other layers...if none are visible then
		// the
		// last visible one was just turned off...fire off a render
		List<IRenderer> renderers = getContext().getRenderManager()
				.getRenderers();
		boolean visible = false;
		for (IRenderer renderer : renderers) {
			if (renderer.getContext().isVisible()) {
				visible = true;
				break;
			}
		}
		if (getState() == DISPOSED
				|| (!getRenderer().getContext().isVisible() && visible)) {
			dirty = true;
			return;
		}
		if (getState() == RENDERING)
			stopRendering();
		redraw.bounds = bounds;
		redraw.schedule();
		return;

	}

	RedrawJob redraw = new RedrawJob();

	class RedrawJob extends Job {
		RedrawJob() {
			super("Redraw"); //$NON-NLS-1$
			setSystem(true);
		}

		@Override
		public boolean belongsTo(Object family) {
			return renderJob.belongsTo(family);
		}

		Envelope bounds;

		@Override
		protected IStatus run(IProgressMonitor monitor) {
			int i = 0;
			try {
				Thread.sleep(50);
			} catch (InterruptedException e1) {
				ProjectPlugin.log("unexpected error", e1); //$NON-NLS-1$
			}
			while (getState() == IRenderer.RENDERING && i < 20) {
				i++;
				synchronized (getRenderer()) {
					try {
						getRenderer().wait(50);
					} catch (InterruptedException e) {
						return Status.OK_STATUS;
					}
				}
			}
			renderJob.setBounds(bounds);
			runRenderJob();
			return Status.OK_STATUS;
		}

	};

	/**
	 * @see net.refractions.udig.project.internal.render.impl.RendererImpl#render(com.vividsolutions.jts.geom.Envelope,
	 *      org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void render(Envelope bounds, IProgressMonitor monitor) {
		render(bounds);
	}

	@Override
	public String toString() {
		String selection = ""; //$NON-NLS-1$
		if (getContext().getLayer() instanceof SelectionLayer)
			selection = "Selection "; //$NON-NLS-1$
		return getContext().getMap().getName()
				+ ":" + selection + (getRenderer() != null ? getRenderer().getName() : "null"); //$NON-NLS-1$ //$NON-NLS-2$
	}

} // RenderExecutorImpl
