/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal;

import net.refractions.udig.project.ui.IUDIGDialogPage;
import net.refractions.udig.project.ui.IUDIGView;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.geotools.feature.Feature;

class FeatureEditorLoader {
    /** FeatureEditorLoader processor field */
    final FeatureEditorExtensionProcessor processor;

    private String dialogPage;

    String viewId;

    private String name;

    String id;

    ImageDescriptor icon;

    IAction contribution;

    private IConfigurationElement definition;

    private FeatureTypeMatch featureTypeMatcher;

    FeatureEditorLoader( FeatureEditorExtensionProcessor processor, IConfigurationElement definition ) {
        this.processor = processor;
        String iconID = definition.getAttribute("icon"); //$NON-NLS-1$
        if (iconID != null) {
            icon = AbstractUIPlugin.imageDescriptorFromPlugin(definition.getNamespace(), iconID);
        }

        id = definition.getAttribute("id"); //$NON-NLS-1$
        name = definition.getAttribute("name"); //$NON-NLS-1$
        viewId = definition.getAttribute("viewId"); //$NON-NLS-1$
        dialogPage = definition.getAttribute("dialogPage"); //$NON-NLS-1$
        this.definition = definition;
        IConfigurationElement[] featureTypeDef = definition.getChildren("featureType");//$NON-NLS-1$
        if (featureTypeDef.length == 1)
            this.featureTypeMatcher = new FeatureTypeMatch(featureTypeDef[0]);
        else
            this.featureTypeMatcher = FeatureTypeMatch.ALL;
    }

    IAction getAction( final ISelection selection ) {
        if (!(selection instanceof IStructuredSelection))
            return null;

        IStructuredSelection structuredSelection = (IStructuredSelection) selection;
        if (!processor.sameFeatureTypes(structuredSelection))
            return null; //$NON-NLS-1$

        final Feature feature = (Feature) structuredSelection.getFirstElement();
        if (featureTypeMatcher.matches(feature) == -1)
            return null;
        contribution = new Action(name, IAction.AS_RADIO_BUTTON){
            public void runWithEvent( org.eclipse.swt.widgets.Event event ) {
                
                //note: when the gui is used the contribution is checked by the framework.
                // so the gui must be used in order to test this.
                if(contribution.isChecked()){
                    processor.createEditAction(FeatureEditorLoader.this, selection, feature);
                    open(event.display, selection);
                }
            }
        };
        contribution.setId(id);
        contribution.setImageDescriptor(icon);
        IContributionItem selected = this.processor.selectedEditors.get(feature.getFeatureType());
        if ( selected==null ){
            selected=processor.createEditAction(processor.getClosestMatch(selection), selection, feature);
        }
        if (selected!=null && selected.getId().equals(id))
            contribution.setChecked(true);
        else
            contribution.setChecked(false);

        return contribution;
    }

    void open( Display display, ISelection selection ) {
        Feature feature = (Feature) ((IStructuredSelection) selection).getFirstElement();

        if (viewId != null) {
            try {
                IUDIGView view = (IUDIGView) PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                        .getActivePage().showView(viewId, null, IWorkbenchPage.VIEW_VISIBLE);
                view.editFeatureChanged(feature);
            } catch (PartInitException e) {
                ProjectUIPlugin.log(null, e);
            }
        } else if (dialogPage != null) {
            try {
                IUDIGDialogPage page = (IUDIGDialogPage) definition
                        .createExecutableExtension("dialogPage"); //$NON-NLS-1$
                page.setContext(this.processor.partListener.currentContext);
                Dialog dialog = this.processor.new EditorDialog(new Shell(display.getActiveShell(),
                        SWT.RESIZE | SWT.PRIMARY_MODAL), page);
                dialog.setBlockOnOpen(false);
                page.setFeature(feature);
                dialog.open();

                PlatformUI.getPreferenceStore().putValue(
                        FeatureEditorExtensionProcessor.CURRENT_LOADER_ID, id);

            } catch (CoreException e) {
                ProjectUIPlugin.log(null, e);
            }

        }
    }

    /**
     * Returns an integer indicating the accuracy of the match between a featureType and the one
     * declared in the FeatureEditor extension declaration.
     * <p>
     * -1 indicates no match, 0 a perfect match and each number higher is a match but a poorer
     * match.
     * </p>
     * 
     * @param selection
     * @return
     */
    public int match( ISelection selection ) {
        IStructuredSelection structuredSelection = (IStructuredSelection) selection;
        if (!processor.sameFeatureTypes(structuredSelection))
            return -1; //$NON-NLS-1$

        final Feature feature = (Feature) structuredSelection.getFirstElement();
        return featureTypeMatcher.matches(feature);
    }
}