/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This
 * library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal;


import net.refractions.udig.project.internal.ProjectPlugin;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

/**
 * Create a NewProject.
 * 
 * @author jgarnett
 * @since 0.3
 */
public class NewProjectWizard extends Wizard implements INewWizard {

    /** ProjectNewPage used to collect file */
    protected ProjectNewPage page;

    /**
     * Override to make title reflect current page.
     * 
     * @param page The page to set.
     */
    public void setPage( ProjectNewPage page ) {
        this.page = page;
        setWindowTitle(page.getTitle());
    }
    /**
     * The <code>Wizard</code> implementation of this <code>IWizard</code> method does nothing. Subclasses should
     * extend if extra pages need to be added before the wizard opens. New pages should be added by calling
     * <code>addPage</code>.
     */
    public void addPages() {
        page = new ProjectNewPage();
        addPage(page);
        setWindowTitle(page.getTitle());
        setHelpAvailable(true);
    }

    /**
     * Create a project based on page.getFile() and complete the wizard.
     * <p>
     * If page.getFile() returns <code>null</code> nothing will happen.
     * </p>
     * 
     * @see org.eclipse.jface.wizard.IWizard#performFinish()
     * @return <code>true</code> when project successfully created
     */
    public boolean performFinish() {
        final String path = page.getFile();
        if (path == null)
            return false;
        ProjectPlugin.getPlugin().getProjectRegistry().getProject(path);
        return true;
    }
    /**
     * We can finish if the user has entered a file.
     * 
     * @return true if we can finish
     */
    public boolean canFinish() {
        return page.isPageComplete();
    }
    /**
     * Hook us up to the world .. I mean workbench.
     * <p>
     * This is where all the magic is supposed to happen to remember the previous directory and so on.
     * </p>
     * <p>
     * </p>
     * 
     * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
     *      org.eclipse.jface.viewers.IStructuredSelection)
     * @param workbench
     * @param selection
     */
    public void init( IWorkbench workbench, IStructuredSelection selection ) {
        // TODO: Magic to remember previous directory
    }

}
