/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.ui.palette.ColourPalette;
import net.refractions.udig.ui.palette.ColourPaletteMasterList;
import net.refractions.udig.ui.palette.ColourScheme;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ScrollBar;



/**
 *<p>
 * <b>Purpose:</b>
 * </p>
 * <p>
 * Panel for Property page for <em><b>Map</b></em> objects with widgets
 * that allow users to change default colours for the map.
 * </p>
 *  @author ptozer
 */
public class PaletteDefaultChooserPanel {
    Composite composite;
    ScrolledComposite scrolledcomposite;
    ColourScheme mapScheme = null;
    ColourPalette mapPalette = null;
    Combo schemeCombo = null;
    ArrayList<PaletteCombo> allLayerControls = new ArrayList<PaletteCombo>();
    ScrollBar vScroller = null;
    Map map = null;

    int index = 0;
    int numberoflayers = 0;
    
    /**
     * Constructor
     *
     */
    public PaletteDefaultChooserPanel() {
        /*
         *
         * colourLetterCombo lists to choose colours per
         * Each layer- show colour and allow colour choice in drop-down
         * Background colour neutrals plus blue
         * Polygon borders neutrals
         * Polygon Fill palette 
         * Line palette plus black and white
         * Point palette plus black and white
         * Text palette plus black and white
         * 
         */
    }
    /**
     * <p>Creates a Control with:<br>
     * A colourLetterCombo box to allow the user to change the default colour scheme
     * used to colour all subsequent layers after closing this panel.<br>
     * <br>
     * A list of all current Map Layers and the colour assigned to the layer.
     * The colour is combined with a drop-down of all available colours to choose from.
     * Currently, choosing from this list does nothing expect change the colour beside it.
     * No changes are affected to the layer.
     * </p>
     * @param parent
     * @param map
     * @return
     */
    public Control createPaletteDefaultChooserPanel(Composite parent, Map thisMap){
        /*
         * uses ColorBrewer.com as a guide- all palettes and colour schemes
         * taken from this.
         * The concept of lettered colours also comes from ColorBrewer. 
        * 
        * TODO: scroller does not work
        *
        * 
        */
        
        this.map = thisMap;
        numberoflayers = this.map.getContextModel().getLayers().size();
        mapScheme = map.getColourScheme();
        mapPalette = map.getColorPalette();
        scrolledcomposite = new ScrolledComposite(parent, SWT.V_SCROLL | SWT.H_SCROLL);
        composite = new Composite(scrolledcomposite, SWT.NONE);
        
        scrolledcomposite.setContent(composite);

        GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 7;
        composite.setLayout( gridLayout );

        //title at top of panel
        Label titleLabel = new Label(composite, SWT.NONE);
        titleLabel.setText( "Map Colour Defaults" ); //$NON-NLS-1$
    
        GridData data = new GridData();
        data.horizontalSpan = 7;
        titleLabel.setLayoutData(data);

        //palette label
        Label paletteLabel = new Label(composite, SWT.NONE);
        paletteLabel.setText( "Change the Map's Default Colour Scheme:" ); //$NON-NLS-1$
    
        data = new GridData();
        data.horizontalSpan = 1;
        paletteLabel.setLayoutData(data);
        
        //palette colourLetterCombo box
        schemeCombo = new Combo(composite,SWT.DROP_DOWN | SWT.READ_ONLY);

        List<String>  names = mapPalette.getAllSchemeNames();
        String [] schemeNames = new String[names.size()];
        
        names.toArray(schemeNames);
        schemeCombo.setItems(schemeNames);
        
        //find out index number for selection
        index = 0;
        for(int i = 0; i < schemeNames.length; i++) {
            if((schemeNames[i]).equalsIgnoreCase(mapScheme.getSchemeName())){
                index = i;
            }
        }
        
        schemeCombo.addSelectionListener(new SelectionListener() {            
            public void widgetSelected(SelectionEvent e) {
                /*
                 * When a new palette is selected:
                 * get the palette String and 
                 * then we want to repaint all paletteCombos to have new colours
                 */
                index = schemeCombo.getSelectionIndex();
                String name =  schemeCombo.getItem(index);
                mapScheme = mapPalette.getColourSchemeByName(name);
                map.setColourScheme(mapScheme);
                //TODO: Might want to update layer data displays with new colours- 
                             
            }
            public void widgetDefaultSelected(SelectionEvent e) {
                // TODO Auto-generated method stub               
            }
        });
        
        //TODO set selection to default or current colour palette
        schemeCombo.select(index);//default

        data = new GridData();
        data.horizontalSpan = 6;
        data.widthHint = 50;
        
        schemeCombo.setLayoutData(data);
        
/*      //TODO  what we want eventually here is to allow the
         * user to change to map background, polygon outline colours.
         * Each layer has its own colourscheme
         * These should be defaulted to greysclae (plus cyan for background)
         */
		
		Label checkBoxLabel = new Label(composite, SWT.NONE);
		checkBoxLabel.setText("Check the layers that you want to apply this new Colour Scheme to:");
		data = new GridData();
        data.horizontalSpan = 7;
		checkBoxLabel.setLayoutData(data);

        updateLayerDisplay();
       // composite.layout(true);
        Point pt = composite.computeSize(SWT.DEFAULT, SWT.DEFAULT);
        composite.setSize(pt);
        
        return scrolledcomposite;
        
    }
  
    void updateLayerDisplay() {
        //clear all existing colour controls
        if(allLayerControls != null && allLayerControls.size() > 0) {
            Iterator iter = allLayerControls.iterator();
            while(iter.hasNext()) {
                 ((PaletteCombo)iter.next()).dispose();
            }
            allLayerControls = new ArrayList<PaletteCombo>();
        }
        //this array of letter used for display colourLetterCombo box
        String [] colourletters = ColourPaletteMasterList.instance().getColourLetters(map.getColorPalette().getPaletteType(), numberoflayers);
//      NOTE: colour scheme may provide colours in weird orders, that is why we care about these letters
       
        //get the layers from the map
        List<ILayer> layers = map.getMapLayers();
        Iterator<ILayer> layerIterator = layers.iterator();
        while(layerIterator.hasNext()) {
            //for each layer display its current colour
            Layer layer = (Layer) layerIterator.next();
           // System.out.println(layer.getName());

            PaletteCombo layerColourCombo = new PaletteCombo(composite);
            //ad the colour indicator for the layer to the dialog panel
            Control layerCombo = layerColourCombo.getPaletteCombo(mapScheme, layer,colourletters);

            GridData data = new GridData();
            data.horizontalSpan = 6;
            layerCombo.setLayoutData(data);

            composite.layout(true);
            scrolledcomposite.layout(true);
            allLayerControls.add(layerColourCombo);
        }
    }
    
   
   
    /**
     * TODO: rewrite to dispose off all class vars.
     */
    public void dispose() {
        Control [] controls = composite.getChildren();
        for (int i = 0; i < controls.length; i ++) {
            
            controls[i].dispose();
        }
        composite.dispose();
		scrolledcomposite.dispose();
    }
    /**
     * @return Returns the allLayerControls.
     */
    public ArrayList<PaletteCombo> getAllLayerControls() {
        return allLayerControls;
    }
}
