/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This
 * library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.internal.ui.UiPlugin;
import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.IProject;
import net.refractions.udig.project.IProjectElement;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.Project;
import net.refractions.udig.project.internal.ProjectPlugin;
import net.refractions.udig.project.internal.ProjectRegistry;
import net.refractions.udig.project.ui.AdapterFactoryLabelProviderDecorator;
import net.refractions.udig.project.ui.internal.actions.OpenProject;
import net.refractions.udig.project.ui.internal.tool.display.ToolManager;
import net.refractions.udig.ui.UDIGDragDropUtilities;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.preference.IPreferenceNode;
import org.eclipse.jface.preference.PreferenceDialog;
import org.eclipse.jface.preference.PreferenceManager;
import org.eclipse.jface.preference.PreferenceNode;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.dialogs.PropertyPage;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.ISetSelectionTarget;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheetPage;

/**
 * Displays the Projects and their contents
 * 
 * @author jeichar
 * @since 0.3
 */
public class ProjectExplorer extends ViewPart implements IMenuListener,
		ISetSelectionTarget {

	/** The Extension ID of ProjectExplorer */
	public static final String ID = "net.refractions.udig.project.ui.projectExplorer"; //$NON-NLS-1$

	ProjectRegistry projectRegistry;

	private Composite container;

	TreeViewer treeViewer;

	private PropertySheetPage propertySheetPage;

	private static ProjectExplorer explorer = new ProjectExplorer();

	private IAction propertiesAction;

	private Action openAction;

	private Action openProjectAction;

    private AdapterFactoryContentProvider contentProvider;

	/**
	 * Construct <code>ProjectExplorer</code>.
	 */
	public ProjectExplorer() {
		super();
		CatalogPlugin.getDefault();
		explorer = this;

		projectRegistry = ProjectPlugin.getPlugin().getProjectRegistry();

	}

	/**
	 * returns the list of all the selected objects of the given class.
	 * 
	 * @param clazz
	 *            The object types to add to the list.
	 * @return the list of all the selected objects of the given class
	 */
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	public <T> List<T> getSelected(Class<T> clazz) {
		StructuredSelection selection = (StructuredSelection) treeViewer
				.getSelection();
		List<T> selected = new ArrayList<T>();
		for (Iterator<T> iter = selection.iterator(); iter.hasNext();) {
			T obj = iter.next();
			if (clazz != null && clazz.isAssignableFrom(obj.getClass()))
				selected.add(obj);
		}
		return selected;
	}

	/**
	 * TODO summary sentence for init ...
	 * 
	 * @see org.eclipse.ui.part.ViewPart#init(org.eclipse.ui.IViewSite)
	 * @param site
	 * @throws org.eclipse.ui.PartInitException
	 */
	public void init(IViewSite site) throws PartInitException {
		super.init(site);
		// projectViewActionBarContributor = new
		// ProjectViewActionBarContributor();
		// projectViewActionBarContributor.init(site.getActionBars(),
		// getEditingDomain());

	}

	/**
	 * This creates a context menu for the viewer and adds a listener as well
	 * registering the menu for extension. <!--
	 * 
	 * @param viewer
	 *            The viewer to create a contect menu for.
	 */
	protected void createContextMenuFor(StructuredViewer viewer) {
		MenuManager contextMenu = new MenuManager("#PopUp"); //$NON-NLS-1$
		contextMenu.setRemoveAllWhenShown(true);
		contextMenu.addMenuListener(this);
		Menu menu = contextMenu.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(contextMenu, viewer);
	}

	/**
	 * Returns the composite object that is the UI of the ProjectExplorer.
	 * 
	 * @return the composite object that is the UI of the ProjectExplorer.
	 */
	public Composite getContainer() {
		return container;
	}

	/**
	 * TODO summary sentence for createPartControl ...
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 * @param parent
	 */
	public void createPartControl(Composite parent) {

        ToolManager.addToolScope(getSite());

        container = parent;

		Tree tree = new Tree(getContainer(), SWT.MULTI);

		treeViewer = new TreeViewer(tree);

        UDIGDragDropUtilities.addDragDropSupport(treeViewer,this);
        
		treeViewer.setAutoExpandLevel(1);
        contentProvider = new UDIGAdapterFactoryContentProvider(
                getAdapterFactory());
		treeViewer.setContentProvider(contentProvider);

		treeViewer.setLabelProvider(new AdapterFactoryLabelProviderDecorator(
				getAdapterFactory(), treeViewer));
		treeViewer.setInput(projectRegistry);
		treeViewer.setSorter(new ViewerSorter() {
			/**
			 * @see org.eclipse.jface.viewers.ViewerSorter#compare(org.eclipse.jface.viewers.Viewer,
			 *      java.lang.Object, java.lang.Object)
			 */
			public int compare(Viewer viewer, Object e1, Object e2) {
				if (e1 instanceof Layer && e2 instanceof Layer) {
					return ((Layer) e1).compareTo((Layer) e2);
				}
				return super.compare(viewer, e1, e2);
			}
		});
		createContextMenuFor(treeViewer);
		addMenuActions();
		addToobarActions();
		treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {

			public void selectionChanged(SelectionChangedEvent event) {
				if (event.getSelection().isEmpty())
					return;
				final Object obj = ((IStructuredSelection) treeViewer
						.getSelection()).getFirstElement();
				if (obj instanceof IProject)
					projectRegistry.setCurrentProject((Project) obj);
				if (obj instanceof ILayer)
					getViewSite().getActionBars().getStatusLineManager()
							.setMessage(((ILayer) obj).getStatusMessage());
			}

		});
		treeViewer.addDoubleClickListener(new IDoubleClickListener() {

			public void doubleClick(DoubleClickEvent event) {
				if( PlatformUI.getWorkbench().isClosing() )
					return;

				final Object obj = ((IStructuredSelection) treeViewer
						.getSelection()).getFirstElement();
				if (obj instanceof IProjectElement ) {
					open(obj);
				}else if( obj instanceof ILayer ){
					open( ((ILayer)obj).getMap());	
				}else
				Display.getDefault().asyncExec(new Runnable() {

					public void run() {
						treeViewer.setExpandedState(obj, !treeViewer
								.getExpandedState(obj));
					}
				});

			}

		});

	}

	/**
	 * @return a ComposeableAdapterFactory for all the Udig's EMF objects.
	 */
	public AdapterFactory getAdapterFactory() {
		return ProjectUIPlugin.getDefault().getAdapterFactory();
	}

	private void addToobarActions() {
		getActionBars().getMenuManager().add(
				new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void addMenuActions() {
		getActionBars().getMenuManager().add(getOpenProjectAction());
		getActionBars().getMenuManager().add(
				new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));

	}

	/**
	 * TODO summary sentence for setFocus ...
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
		getSite().setSelectionProvider(treeViewer);
	}

	/**
	 * This is how the framework determines which interfaces we implement. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @param key
	 *            The desired class
	 * @return An object of type key or null;
	 */
	public Object getAdapter(Class key) {
		if (key.equals(IPropertySheetPage.class))
			return getPropertySheetPage();
		return super.getAdapter(key);
	}

	/**
	 * This accesses a cached version of the property sheet. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @return An IProperty page for the selected object
	 */
	public IPropertySheetPage getPropertySheetPage() {
		if (propertySheetPage == null) {
			propertySheetPage = new PropertySheetPage() {
				public void makeContributions(IMenuManager menuManager,
						IToolBarManager toolBarManager,
						IStatusLineManager statusLineManager) {
					super.makeContributions(menuManager, toolBarManager,
							statusLineManager);
				}

				public void setActionBars(IActionBars actionBars) {
					super.setActionBars(actionBars);
				}
			};
			propertySheetPage
					.setPropertySourceProvider(new AdapterFactoryContentProvider(
							getAdapterFactory()));
		}

		return propertySheetPage;
	}

	/**
	 * Gets the site's action bars
	 * 
	 * @return the site's action bars
	 */
	public IActionBars getActionBars() {
		return getViewSite().getActionBars();
	}

	/**
	 * @see org.eclipse.jface.action.IMenuListener#menuAboutToShow(org.eclipse.jface.action.IMenuManager)
	 */
	public void menuAboutToShow(IMenuManager manager) {
		boolean addOpenAction = false;
		for (Iterator iter = ((IStructuredSelection) treeViewer.getSelection())
				.iterator(); iter.hasNext();) {
			Object obj = iter.next();
			if (obj instanceof IProjectElement) {
				addOpenAction = true;
				break;
			}
		}

		if (addOpenAction) {
			manager.add(getOpenAction());
		}
		manager.add(getOpenProjectAction());
		manager.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		LayerApplicabilityMenuCreator creator=getApplicabilityMenu();
		if( creator!=null )
        	manager.add(creator.getMenuManager());

		manager.add(UiPlugin.getDefault().getOperationMenuFactory()
				.getContextMenu());

		if (!treeViewer.getSelection().isEmpty()
				&& ((IStructuredSelection) treeViewer.getSelection())
						.getFirstElement() instanceof Map) {
			manager.add(new Separator());
			manager.add(getPropertiesAction());
		}
   
	}
    
    private LayerApplicabilityMenuCreator applicabilityCreator;
    private LayerApplicabilityMenuCreator getApplicabilityMenu() {
        if (applicabilityCreator == null) {
            applicabilityCreator = new LayerApplicabilityMenuCreator();
        }

		StructuredSelection selection=(StructuredSelection) treeViewer.getSelection();
		for (Iterator iter = selection.iterator(); iter.hasNext();) {
			Object element = iter.next();
			if ( !(element instanceof Layer) )
				return null;
		}
		
        return applicabilityCreator;
    }
	/**
	 * TODO summary sentence for getOpenProjectAction ...
	 * 
	 * @param manager
	 * @return
	 */
	private IAction getOpenProjectAction() {
		if (openProjectAction == null) {
			openProjectAction = new Action() {
				OpenProject delegate = new OpenProject();

				/**
				 * @see org.eclipse.jface.action.Action#run()
				 */
				public void run() {
					if( PlatformUI.getWorkbench().isClosing() )
						return;

					delegate.run(this);
				}
			};
			openProjectAction.setText(Policy
					.bind("ProjectExplorer.openProject.text")); //$NON-NLS-1$
			openProjectAction.setToolTipText(Policy
					.bind("ProjectExplorer.openProject.tooltip")); //$NON-NLS-1$
		}
		return openProjectAction;
	}

	private IAction getPropertiesAction() {
		if (propertiesAction == null) {
			// propertiesAction=ActionFactory.PROPERTIES.create(getSite().getWorkbenchWindow());
			// propertiesAction.setEnabled(true);
			propertiesAction = new Action() {
				/**
				 * @see org.eclipse.jface.action.Action#run()
				 */
				public void run() {
                    Map map=(Map) ((IStructuredSelection)treeViewer.getSelection()).getFirstElement();
					PropertyPage page = new CRSPropertyPage(map);
					PreferenceManager mgr = new PreferenceManager();
					IPreferenceNode node = new PreferenceNode("1", page); //$NON-NLS-1$
					mgr.addToRoot(node);

					PropertyPage palettePage = new PalettePropertyPage(map);
					IPreferenceNode node2 = new PreferenceNode("2", palettePage); //$NON-NLS-1$
					mgr.addToRoot(node2);

                    PreferenceDialog dialog = new PreferenceDialog(getSite().getShell(), mgr);
					dialog.create();
					dialog.open();
				}
			};
			propertiesAction.setText(Policy
					.bind("ProjectExplorer.properties.text")); //$NON-NLS-1$
			IAction tmp = ActionFactory.PROPERTIES.create(getSite()
					.getWorkbenchWindow());
			propertiesAction.setAccelerator(tmp.getAccelerator());
			propertiesAction.setToolTipText(tmp.getToolTipText());
			propertiesAction.setText(tmp.getText());
			propertiesAction.setActionDefinitionId(tmp.getActionDefinitionId());
		}
		getActionBars().setGlobalActionHandler(
				ActionFactory.PROPERTIES.getId(), propertiesAction);
		return propertiesAction;
	}

	private Action getOpenAction() {
		if (openAction == null)
			openAction = new Action() {
				/**
				 * @see org.eclipse.jface.action.Action#run()
				 */
				public void run() {
					for (Iterator iter = ((IStructuredSelection) treeViewer
							.getSelection()).iterator(); iter.hasNext();) {
						open(iter.next());
					}
				}
			};
		openAction.setText(Policy.bind("ProjectExplorer.open.text")); //$NON-NLS-1$
		return openAction;
	}

	class OpenWithActions extends Action {
		UDIGEditorInputDescriptor input;

		/**
		 * Construct <code>ProjectExplorer.OpenWithActions</code>.
		 * 
		 * @param input
		 *            the editor input that
		 */
		public OpenWithActions(UDIGEditorInputDescriptor input) {
			this.input = input;
		}

		/**
		 * @see org.eclipse.jface.action.Action#run()
		 */
		public void run() {
			for (Iterator iter = ((IStructuredSelection) treeViewer
					.getSelection()).iterator(); iter.hasNext();) {
				defaultEditorMap.put(input.getType(), this);
				open(iter.next());
			}
		}
	}

	List getOpenWithActions(Class type) {
		List actions = (List) editorInputsMap.get(type);
		if (actions == null) {
			actions = new ArrayList();
			List<UDIGEditorInputDescriptor> inputs = ProjectUIPlugin
					.getDefault().getEditorInputs(type);
			for (Iterator iter = inputs.iterator(); iter.hasNext();) {
				UDIGEditorInputDescriptor desc = (UDIGEditorInputDescriptor) iter
						.next();
				Action openWithAction = new OpenWithActions(desc);
				openWithAction.setText(desc.getName());
				actions.add(openWithAction);
			}
			if (actions != null)
				editorInputsMap.put(type, actions);
		}
		return actions;
	}

	/**
	 * Maps between a class and a list of associated UDIGEditorInput Objects
	 */
	java.util.Map editorInputsMap = new HashMap();

	/**
	 * Maps between a class and the id of the editor input to use (from the list
	 * in editorInputsMap)
	 */
	java.util.Map defaultEditorMap = new HashMap();

	/**
	 * Opens a map or page in the editor or opens the treeviewer 1 level if the
	 * object is a project
	 * 
	 * @param obj
	 *            the object to open
	 */
	public void open(final Object obj) {
		open(obj, false);
	}

	/**
	 * Opens a map or page in the editor or opens the treeviewer 1 level if the
	 * object is a project
	 * 
	 * @param obj
	 *            the object to open
	 * @param dirtyEditor
	 *            whether to mark the newly opened editor as dirty
	 */
	public void open(final Object obj, final boolean dirtyEditor) {
		if( PlatformUI.getWorkbench().isClosing() )
			return;

		List inputs = getOpenWithActions(obj.getClass());
		String defaultEditor = (String) defaultEditorMap.get(obj.getClass());

		OpenWithActions action = null;
		if (defaultEditor == null) {
			action = (OpenWithActions) inputs.get(0);
		} else {
			for (Iterator iter = inputs.iterator(); iter.hasNext();) {
				OpenWithActions current = (OpenWithActions) iter.next();
				if (current.input.getEditorID().equals(defaultEditor)) {
					action = current;
					break;
				}
			}
		}

		final OpenWithActions finalAction = action;

        final Runnable open=new Runnable() {

            public void run() {

                try {
                    EditorPart editPart = (EditorPart) PlatformUI
                            .getWorkbench()
                            .getActiveWorkbenchWindow()
                            .getActivePage()
                            .openEditor(
                                    finalAction.input
                                            .createInput((IProjectElement) obj),
                                    finalAction.input.getEditorID(), true);
                } catch (CoreException e) {
                    ProjectUIPlugin.getDefault().getLog().log(
                            new Status(IStatus.ERROR,
                                    "net.refractions.udig.project.ui", 0, //$NON-NLS-1$
                                    "Error Initializing Page Editor", e)); //$NON-NLS-1$
                }
            }
        };
        
        
        if( Display.getCurrent()!=null )
        	BusyIndicator.showWhile(Display.getCurrent(), open);
        else
        	Display.getDefault().asyncExec(new Runnable() {
				public void run() {
		        	BusyIndicator.showWhile(Display.getDefault(), open);
				}
			});
	}

	/*
	 * @see org.eclipse.ui.part.ISetSelectionTarget#selectReveal(org.eclipse.jface.viewers.ISelection)
	 */
	public void selectReveal(ISelection selection) {
		treeViewer.setSelection(selection);
	}

	/**
	 * Returns the ProjectExplorer view if it has been created, otherwise an
	 * object is created and returned.
	 * 
	 * @return the ProjectExplorer view if it has been created, otherwise an
	 *         object is created and returned.
	 */
	public static ProjectExplorer getProjectExplorer() {
		return explorer;
	}

	/**
	 * Returns the layers viewer
	 */
	public TreeViewer getViewer() {
		return treeViewer;
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	@Override
	public void dispose() {
        if( PlatformUI.getWorkbench().isClosing()){
            ProjectPlugin.getPlugin().turnOffEvents();
        }
		treeViewer.getLabelProvider().dispose();
        contentProvider.dispose();
		super.dispose();
	}
}
