/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import net.refractions.udig.project.ui.internal.tool.display.ToolManager;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.osgi.framework.BundleContext;

/**
 * The Plugin class for the net.refractions.udig.project plugin. Provides access to plugin
 * resources.
 * 
 * @author Jesse Eichar
 * @version $Revision: 1.9 $
 */
public class ProjectUIPlugin extends AbstractUIPlugin {
    /**
     * Keep track of the singleton.
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * @generated
     */
	public static final ProjectUIPlugin INSTANCE = new ProjectUIPlugin();

	private static final String ICONS_PATH = "icons/"; //$NON-NLS-1$

	/** The plugin ID */
	public static final String ID = "net.refractions.udig.project.ui"; //$NON-NLS-1$

	/** Preference store for the last directory open by the file selection dialog */
	public static final String PREF_OPEN_DIALOG_DIRECTORY = "udig.preferences.openDialog.lastDirectory"; //$NON-NLS-1$

	/** The maximum number of resources per service that can be added to a map without
	 * asking the user for permission. */
	public static final int MAX_RESOURCES_IN_SERVICE = 1;
	
	private static ProjectUIPlugin plugin;

	Images images = new Images();

	List<AdapterFactory> adapterFactories;

	private static final String ADAPTER_FACTORIES_ID = "net.refractions.udig.project.ui.itemProviderAdapterFactories"; //$NON-NLS-1$

	private java.util.Map<Class, List<UDIGEditorInputDescriptor>> editorInputDescriptors = new HashMap<Class, List<UDIGEditorInputDescriptor>>();

	private ToolManager toolManager;

	private PropertySheetPage propertySheetPage;

	/**
	 * creates a plugin instance
	 */
	public ProjectUIPlugin() {
		super();
		plugin = this;
	}

	FeatureEditorExtensionProcessor featureEditProcessor=new FeatureEditorExtensionProcessor();
	
	/**
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		Policy.localize("net.refractions.udig.project.ui.internal.messages"); //$NON-NLS-1$

		URL iconsUrl = context.getBundle().getEntry(ICONS_PATH);
		
		images.initializeImages(iconsUrl, getImageRegistry());
	}


	/**
	 * Returns the system created plugin object
	 * 
	 * @return the plugin object
	 */
	public static ProjectUIPlugin getDefault() {
		return plugin;
	}

	/**
	 * @return Returns the images.
	 */
	public Images getImages() {
		return images;
	}

	/**
	 * Returns the ToolManager singleton.
	 * 
	 * @return the ToolManager singleton.
	 */
	public ToolManager getToolManager() {
		if (toolManager == null) {
			synchronized (this) {
				if (toolManager == null)
					toolManager = new ToolManager();
			}
		}
		return toolManager;
	}

	/**
	 * @return Returns the adapterFactories.
	 */
	public List<AdapterFactory> getAdapterFactories() {
		if (adapterFactories == null) {
			adapterFactories = new ArrayList<AdapterFactory>();

			IExtensionRegistry registry = Platform.getExtensionRegistry();
			IExtensionPoint extensionPoint = registry
					.getExtensionPoint(ADAPTER_FACTORIES_ID);
			IExtension[] extensions = extensionPoint.getExtensions();

			for (int i = 0; i < extensions.length; i++) {
				IConfigurationElement[] elements = extensions[i]
						.getConfigurationElements();

				for (int j = 0; j < elements.length; j++) {
					try {
						Object adapterFactory = elements[j]
								.createExecutableExtension("class"); //$NON-NLS-1$
						if (adapterFactory instanceof AdapterFactory) {
							adapterFactories
									.add((AdapterFactory) adapterFactory);
						}
					} catch (CoreException e) {
						e.printStackTrace();
					}
				}
			}

		}
		return Collections.unmodifiableList(adapterFactories);
	}

	/**
	 * Returns an editor input for the type passed in. Processes the editorInputs extension point.
	 * 
	 * @param type
	 * @return an editor input for the type passed in.
	 */
	public List<UDIGEditorInputDescriptor> getEditorInputs(final Class type) {
		if (!editorInputDescriptors.containsKey(type)) {
			synchronized (editorInputDescriptors) {
				if (editorInputDescriptors.containsKey(type))
					return editorInputDescriptors.get(type);

				final List<UDIGEditorInputDescriptor> newInputs = new ArrayList<UDIGEditorInputDescriptor>();

				IExtensionRegistry registry = Platform.getExtensionRegistry();
				IExtensionPoint extensionPoint = registry
						.getExtensionPoint("net.refractions.udig.project.ui.editorInputs"); //$NON-NLS-1$
				IExtension[] extensions = extensionPoint.getExtensions();

				for (int i = 0; i < extensions.length; i++) {
					IConfigurationElement[] elements = extensions[i]
							.getConfigurationElements();

					for (int j = 0; j < elements.length; j++) {
						try {
							Object object = elements[j]
									.createExecutableExtension("projectElement"); //$NON-NLS-1$
							if (object.getClass().isAssignableFrom(type)) {
								UDIGEditorInputDescriptor input = new UDIGEditorInputDescriptor();
								input.setEditorID(elements[j]
										.getAttribute("editorPartID")); //$NON-NLS-1$
								input.setName(elements[j].getAttribute("name")); //$NON-NLS-1$
								input.setExtensionElement(elements[j]);
								input.setType(type);
								newInputs.add(input);
							}
						} catch (CoreException e) {
							e.printStackTrace();
						}
					}
				}

				editorInputDescriptors.put(type, newInputs);
			}
		}

		return editorInputDescriptors.get(type);
	}
    
	Object mutex = new Object();

	/**
	 * This accesses a cached version of the property sheet. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @return An IProperty page for the selected object
	 */
	public IPropertySheetPage getPropertySheetPage() {
		if (propertySheetPage == null) {
			synchronized (mutex) {

				propertySheetPage = new PropertySheetPage() {
                    
					public void makeContributions(IMenuManager menuManager,
							IToolBarManager toolBarManager,
							IStatusLineManager statusLineManager) {
						super.makeContributions(menuManager, toolBarManager,
								statusLineManager);
					}

					public void setActionBars(IActionBars actionBars) {
						super.setActionBars(actionBars);
					}
				};
				propertySheetPage
						.setPropertySourceProvider(new AdapterFactoryContentProvider(
								getAdapterFactory()));
			}
		}

		return propertySheetPage;
	}

	private AdapterFactory adapterFactory;

	/**
	 * Returns the adapterfactory instance.
	 * 
	 * @return the adapterfactory instance.
	 */
	public AdapterFactory getAdapterFactory() {
		if (adapterFactory == null) {
			synchronized (mutex) {
				List<AdapterFactory> factories = new ArrayList<AdapterFactory>();

				factories.addAll(ProjectUIPlugin.getDefault()
						.getAdapterFactories());

				// This one should be added last
				factories.add(new ReflectiveItemProviderAdapterFactory());

				adapterFactory = new ComposedAdapterFactory(factories);
			}
		}
		return adapterFactory;
	}

	/**
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	
    public void stop(BundleContext context) throws Exception {
		Policy.cleanup();
		featureEditProcessor.stopPartListener();
        if(LayerGeneratedGlyphDecorator.getInstance() != null)
            LayerGeneratedGlyphDecorator.getInstance().dispose();
		super.stop(context);
	}
	
	/**
	 * Gets the FeatureEditorProcessor instance.
	 * 
	 * @return
	 */
	public FeatureEditorExtensionProcessor getFeatureEditProcessor() {
		if( !featureEditProcessor.isRunning() )
			featureEditProcessor.startPartListener();

		return featureEditProcessor;
	}
    
    /**
     * Writes an info log in the plugin's log.
     * <p>
     * This should be used for user level messages.
     * </p>
     */
    public static void log( String message, Throwable e) {
        if( message == null ) message = Policy.bind("ProjectUIPlugin.error")+e; //$NON-NLS-1$
        getDefault().getLog().log(new Status(IStatus.INFO, ID, IStatus.OK, message, e));
    }
    /**
     * Messages that only engage if getDefault().isDebugging()
     * <p>
     * It is much prefered to do this:<pre><code>
     * private static final String RENDERING = "net.refractions.udig.project/render/trace";
     * if( ProjectUIPlugin.getDefault().isDebugging() && "true".equalsIgnoreCase( RENDERING ) ){
     *      System.out.println( "your message here" );
     * }
     */
    public static void trace( String message, Throwable e) {
        if( getDefault().isDebugging() ) {
            if( message != null ) System.out.println( message );
            if( e != null ) e.printStackTrace();
        }
    }
    /**
     * Performs the Platform.getDebugOption true check on the provided trace
     * <p>
     * Note: ProjectUIPlugin.getDefault().isDebugging() must also be on.
     * <ul>
     * <li>Trace.RENDER - trace rendering progress
     * </ul>
     * </p> 
     * @param trace currently only RENDER is defined
     */
    public static boolean isDebugging( final String trace ){
        return getDefault().isDebugging() &&
            "true".equalsIgnoreCase(Platform.getDebugOption(trace)); //$NON-NLS-1$    
    }
}   
