/**
 * <copyright></copyright> $Id: RenderManagerDynamic.java 17136 2005-12-02 22:06:38Z jeichar $
 */
package net.refractions.udig.project.ui.internal;

import static net.refractions.udig.project.ui.internal.Trace.RENDER;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.internal.ContextModelListenerAdapter;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.ProjectPackage;
import net.refractions.udig.project.internal.render.CompositeRenderContext;
import net.refractions.udig.project.internal.render.ExecutorVisitor;
import net.refractions.udig.project.internal.render.MultiLayerRenderer;
import net.refractions.udig.project.internal.render.RenderContext;
import net.refractions.udig.project.internal.render.RenderExecutor;
import net.refractions.udig.project.internal.render.RenderFactory;
import net.refractions.udig.project.internal.render.RenderPackage;
import net.refractions.udig.project.internal.render.SelectionLayer;
import net.refractions.udig.project.internal.render.impl.CompositeRenderContextImpl;
import net.refractions.udig.project.internal.render.impl.CompositeRendererExecutorImpl;
import net.refractions.udig.project.internal.render.impl.RenderManagerImpl;
import net.refractions.udig.project.internal.render.impl.RendererExecutorMultiLayer;
import net.refractions.udig.project.render.IRenderer;
import net.refractions.udig.project.render.RenderException;
import net.refractions.udig.project.render.displayAdapter.IMapDisplay;
import net.refractions.udig.project.ui.render.displayAdapter.ViewportPane;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;

import com.vividsolutions.jts.geom.Envelope;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Manager</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link net.refractions.udig.project.render.impl.RenderManagerImpl#getMap <em>Map</em>}</li>
 * <li>{@link net.refractions.udig.project.render.impl.RenderManagerImpl#getViewport <em>Viewport</em>}</li>
 * <li>{@link net.refractions.udig.project.render.impl.RenderManagerImpl#getRendererCreator <em>Renderer Creator</em>}</li>
 * <li>{@link net.refractions.udig.project.render.impl.RenderManagerImpl#getViewportModel <em>Viewport Model</em>}</li>
 * <li>{@link net.refractions.udig.project.render.impl.RenderManagerImpl#getRenderExecutor <em>Render Executor</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class RenderManagerDynamic extends RenderManagerImpl {

	ContextModelListenerAdapter contextModelAdapter = RenderManagerAdapters
			.createContextModelListener(this);

	Adapter viewportListener = RenderManagerAdapters
			.createViewportListener(this);

    private Adapter viewportModelChangeListener=RenderManagerAdapters.createViewportModelChangeListener(this,
            viewportListener, contextModelAdapter);

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	@SuppressWarnings("unchecked")//$NON-NLS-1$
	public RenderManagerDynamic() {
		super();
		eAdapters().add(
				viewportModelChangeListener);
	}

	/**
	 * @see net.refractions.udig.project.render.impl.RenderManagerImpl#refresh(net.refractions.udig.project.Layer)
	 */
	public void refresh(final ILayer layer, final Envelope bounds) {
        checkState();
		if (getMapDisplay() == null || getRenderExecutor() == null)
			return;

		final SelectionLayer selectionLayer = getRendererCreator()
				.findSelectionLayer(layer);

		getRenderExecutor().visit(new ExecutorVisitor() {
			public void visit(RenderExecutor executor) {
				if (executor.getContext().getLayer() == layer
						|| selectionLayer == executor.getContext().getLayer())
					executor.getRenderer().setState(IRenderer.RENDER_REQUEST);
			}

			public void visit(RendererExecutorMultiLayer executor) {
				if (executor.getContext().getLayers().contains(layer)
						|| executor.getContext().getLayers().contains(
								selectionLayer))
					executor.getRenderer().setState(IRenderer.RENDER_REQUEST);
			}

			public void visit(CompositeRendererExecutorImpl executor) {
				List<RenderExecutor> executors=new ArrayList<RenderExecutor>(executor.getRenderer()
				.getRenderExecutors());
				for (RenderExecutor child : executors)
					child.visit(this);
			}
		});
	}

	/**
	 * @see net.refractions.udig.project.render.impl.RenderManagerImpl#refreshSelection(com.vividsolutions.jts.geom.Envelope)
	 */
	public void refreshSelection(ILayer layer, final Envelope bounds) {
        checkState();
        if (getMapDisplay() == null || getRenderExecutor() == null)
			return;
		final Layer selectionLayer = getRendererCreator().findSelectionLayer(
				layer);

		if (selectionLayer == null)
			return;
		getRenderExecutor().visit(new ExecutorVisitor() {
			public void visit(RenderExecutor executor) {
				if (selectionLayer == executor.getContext().getLayer())
					executor.getRenderer().setState(IRenderer.RENDER_REQUEST);
			}

			public void visit(RendererExecutorMultiLayer executor) {
				CompositeRenderContext contexts = executor.getContext();
				for (RenderContext context : contexts.getContextsInternal()) {
					if (context.getLayer() == selectionLayer) {
						executor.getRenderer().setState(
								IRenderer.RENDER_REQUEST);
						return;
					}
				}
			}

			public void visit(CompositeRendererExecutorImpl executor) {
				for (RenderExecutor child : executor.getRenderer()
						.getRenderExecutors())
					child.visit(this);
			}
		});
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	@SuppressWarnings("unchecked")//$NON-NLS-1$
	public void refresh(Envelope bounds) {
        checkState();
        getViewportModelInternal().setInitialized(true);
		if (getMapDisplay() == null) {
			return;
		}
		if (getMapDisplay().getWidth() < 1 || getMapDisplay().getHeight() < 1)
			return;

		validateRendererConfiguration();
		
		if (!getMapInternal().getContextModel().eAdapters().contains(
				contextModelAdapter))
			getMapInternal().getContextModel().eAdapters().add(
					contextModelAdapter);

		try {
			getRenderExecutor().render(bounds);
		} catch (RenderException e) {
			// Won't happen
		}
	}

	void initRenderCreator(RenderContext context) {
        checkState();
        synchronized (this) {
            getRendererCreator().setContext(context);

    		List<Layer> layers = getMapInternal().getContextModel().getLayers();

    		synchronized (getRendererCreator()) {
    			ENotificationImpl notification = new ENotificationImpl(this,
    					Notification.ADD_MANY,
    					ProjectPackage.CONTEXT_MODEL__LAYERS, null, layers);
    			getRendererCreator().changed(notification);
    		}
		}
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	@SuppressWarnings("unchecked") //$NON-NLS-1$
    public void dispose() {
		configuration = null;
        Set<EObject> set=new HashSet<EObject>();
        set.add(getMapInternal());
		Iterator<EObject> iter=getMapInternal().eAllContents();
        while (iter.hasNext()){
            EObject obj=iter.next();
            removeAdapters(obj);
        }
        removeAdapters(getRenderExecutor());
        super.dispose();
	}

    /**
     *
     * @param obj
     */
    private void removeAdapters( EObject obj ) {
        obj.eAdapters().remove(this.viewportListener);
        obj.eAdapters().remove(this.viewportModelChangeListener);
        obj.eAdapters().remove(this.contextModelAdapter);
        obj.eAdapters().remove(this.renderExecutorListener);
        obj.eAdapters().remove(this.selectionListener);
    }

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setDisplayGen(IMapDisplay newDisplay) {
		IMapDisplay oldDisplay = mapDisplay;
		mapDisplay = newDisplay;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET,
					RenderPackage.RENDER_MANAGER__MAP_DISPLAY, oldDisplay,
					mapDisplay));
	}

	/**
	 * TODO summary sentence for setViewport ...
	 * 
	 * @see net.refractions.udig.project.render.RenderManager#setDisplay(net.refractions.udig.project.render.displayAdapter.IMapDisplay)
	 * @param value
	 */
	public void setDisplay(IMapDisplay value) {
        checkState();
        ((ViewportPane) value).setRenderManager(this);
		setDisplayGen(value);
	}

	Adapter renderExecutorListener = RenderManagerAdapters
			.getRenderExecutorListener(this);

	/**
	 * @see net.refractions.udig.project.render.impl.RenderManagerImpl#setRenderExecutor(net.refractions.udig.project.render.RenderExecutor)
	 */
	@SuppressWarnings("unchecked")//$NON-NLS-1$
	public void setRenderExecutor(RenderExecutor newRenderExecutor) {
        checkState();
        if (renderExecutor != null) {
			renderExecutor.eAdapters().remove(renderExecutorListener);
		}
		if (newRenderExecutor != null
				&& !newRenderExecutor.eAdapters().contains(
						renderExecutorListener)) {
			newRenderExecutor.eAdapters().add(renderExecutorListener);
		}
		super.setRenderExecutor(newRenderExecutor);
	}

	Adapter selectionListener = RenderManagerAdapters
			.createLayerListener(this);

	/**
	 * @see net.refractions.udig.project.render.impl.RenderManagerImpl#basicSetMap(net.refractions.udig.project.Map,
	 *      org.eclipse.emf.common.notify.NotificationChain)
	 */
	@SuppressWarnings("unchecked")//$NON-NLS-1$
	public NotificationChain basicSetMapInternal(Map newMap,
			NotificationChain msgs) {
		if (getMapInternal() != null) {
			getMapInternal().eAdapters().remove(selectionListener);
			if (getMapInternal().getContextModel() != null)
				getMapInternal().getContextModel().getDeepAdapters().remove(
						selectionListener);
		}
		if (newMap != null) {
			newMap.eAdapters().add(selectionListener);
			if (newMap.getContextModel() != null)
				newMap.getContextModel().getDeepAdapters().add(
						selectionListener);
		}
		return super.basicSetMapInternal(newMap, msgs);
	}

	Collection<RenderContext> configuration = null;

	/**
	 * Ensures that the current configuration of renderer is a valid choice. For
	 * example the each of the layers in the map has a renderer that can render
	 * it.
	 */
	public void validateRendererConfiguration() {
        checkState();
        synchronized (this) {
	        if (configuration != null) {
					CompositeRenderContext compositecontext = (CompositeRenderContext) getRenderExecutor()
							.getContext();
					List<RenderContext> removeList = new ArrayList<RenderContext>();
					List<RenderContext> addList = new ArrayList<RenderContext>();
					for (RenderContext context : compositecontext
							.getContextsInternal()) {
						if (getRendererCreator().getConfiguration().contains(
								context))
							continue;
	
						removeList.add(context);
					}
					for (RenderContext context : getRendererCreator()
							.getConfiguration()) {
						if (compositecontext.getContextsInternal()
								.contains(context))
							continue;
	
						addList.add(context);
					}
					compositecontext.getContextsInternal().removeAll(removeList);
					compositecontext.getContextsInternal().addAll(addList);
	
			} else {
				initRenderExecutor();
			}
			configuration = getRendererCreator().getConfiguration();
			logRendererTypes();
        }
	}

	/**
	 * 
	 */
	private void initRenderExecutor() {
        checkState();
        MultiLayerRenderer renderExecutor = RenderFactory.eINSTANCE
				.createCompositeRenderer();
		CompositeRenderContext context = new CompositeRenderContextImpl(){
			@Override
			public synchronized BufferedImage getImage(int width, int height) {
				if ( image==null || image.getWidth()<width || image.getHeight()< height){
					image=new BufferedImage(width, height, BufferedImage.TYPE_3BYTE_BGR );
					Graphics2D g = image.createGraphics();
					g.setBackground(Color.WHITE);
					g.clearRect(0,0,width, height);
					g.dispose();
				}
				return image;
			}
			@Override
			public synchronized void clearImage(Rectangle paintArea) {
				if( image==null )
					return;
				Graphics2D g = image.createGraphics();
				g.setBackground(Color.WHITE);
				g.clearRect(paintArea.x, paintArea.y, paintArea.width, paintArea.height);
				g.dispose();
			}
		
		};
		context.setMapInternal(getMapInternal());
		context.setRenderManagerInternal(this);
		renderExecutor.setContext(context);
		

		initRenderCreator(context);
		
		renderExecutor.setName(Policy
				.bind("RenderManagerDynamic.allLayers")); //$NON-NLS-1$
		context.getContextsInternal().addAll(
				getRendererCreator().getConfiguration());
		setRenderExecutor(RenderFactory.eINSTANCE
				.createRenderExecutor(renderExecutor));
	}

	private void logRendererTypes() {
        if (ProjectUIPlugin.isDebugging(RENDER)) {

			final StringBuffer log = new StringBuffer("Current Renderers:"); //$NON-NLS-1$
			getRenderExecutor().visit(new ExecutorVisitor() {
				public void visit(RenderExecutor executor) {
					log
							.append("\n\t" + executor.getClass().getSimpleName() + ":" + executor.getRenderer().getClass().getSimpleName() + "-" + executor.getContext().getLayer().getName()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				}

				public void visit(CompositeRendererExecutorImpl executor) {
					log
							.append("\n\t" + executor.getRenderer().getClass().getSimpleName()); //$NON-NLS-1$
					for (RenderExecutor child : executor.getRenderer()
							.getRenderExecutors())
						child.visit(this);
				}

				public void visit(RendererExecutorMultiLayer executor) {
					log
							.append("\n\t" + executor.getRenderer().getClass().getSimpleName()); //$NON-NLS-1$
				}
			});
			System.out.println(log);
		}
	}

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * @generated
     */
	public RenderExecutor getRenderExecutor() {
        checkState();
        if( renderExecutor==null ){
			initRenderExecutor();
        }
		
        return renderExecutor;
    }	
} // RenderManagerImpl
