package net.refractions.udig.project.ui.internal.actions;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.IResolve;
import net.refractions.udig.catalog.internal.ui.CatalogImportWizard;
import net.refractions.udig.catalog.internal.ui.HeadlessWizardDialog;
import net.refractions.udig.catalog.internal.ui.Policy;
import net.refractions.udig.catalog.ui.DataSourceSelectionPage;
import net.refractions.udig.catalog.ui.UDIGImportPage;
import net.refractions.udig.core.internal.ExtensionPointProcessor;
import net.refractions.udig.core.internal.ExtensionPointUtil;
import net.refractions.udig.mapgraphic.MapGraphicFactory;
import net.refractions.udig.project.IMap;
import net.refractions.udig.project.ui.IDataDropWizard;
import net.refractions.udig.project.ui.PlatformGIS;
import net.refractions.udig.project.ui.internal.ProjectUIPlugin;
import net.refractions.udig.project.ui.internal.ResourceSelectionPage;
import net.refractions.udig.ui.IDropAction;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;

public class WizardDropAction extends IDropAction {

	PageProcessor p;
	
	public WizardDropAction() {
		super();
	}

	@Override
	public boolean accept(Object source, Object destination) {
        //check for real data
        if (source instanceof IResolve) {
            IResolve resolve = (IResolve)source;
            if (resolve.canResolve(MapGraphicFactory.class)) 
                return false;
        }
        
        if (source instanceof String) {
            URL url = extractURL((String) source);
            if (url != null) {
                source = url;
            }
        }
        
        //process the wizard page extension point to determine if anyone
		// can process the object
        p = new PageProcessor(source);
		ExtensionPointUtil.process(ProjectUIPlugin.getDefault(), UDIGImportPage.XPID, p);
		
		return true;
	}

	@Override
	public void perform(
        Object source, Object destination, IProgressMonitor monitor
    ) {

        if (source instanceof String) {
            URL url = extractURL((String) source);
            if (url != null) {
                source = url;
            }
        }
        
        WizardProcessor wizardProcessor=new WizardProcessor(destination);
        ExtensionPointUtil.process(ProjectUIPlugin.getDefault(), "net.refractions.udig.project.ui.dataDropWizard", wizardProcessor); //$NON-NLS-1$
        
       if (wizardProcessor.wizards.size()==0)
			return;
		
       // need to change now only the first of the extensions is being used.
       
       final IWizard fWizard = wizardProcessor.wizards.get(0);
       final Object fSource = source;
        
		Display.getDefault().syncExec(
			new Runnable() {

				public void run() {
					Shell shell = new Shell(Display.getDefault());
			        HeadlessWizardDialog dialog = new HeadlessWizardDialog(shell,fWizard);
			        
			        //the source object could be from an external source (another program), 
                    // set the selection explicitly
                    dialog.setSelection(new StructuredSelection(fSource));
                    
					dialog.setBlockOnOpen(true);
					dialog.open();
					dialog.runHeadless();
					
					if (!shell.isDisposed())
						shell.dispose();
					
				}
				
			}
		);
	
		//TODO: fix
		//this is a huge hack but for now its all we got
		if (!(destination instanceof IEditorPart))
			return;
			
		//one good hack deserves another if there is no map, wait until it has been
		// created to continue
		IMap map = PlatformGIS.getActiveMap();
		if (map == null) {
			try {
				final ArrayList<Boolean> wait = new ArrayList<Boolean>();
				wait.add(true);
				int tries=80;
				while(wait.get(0)&& tries<80 ) {
					tries++;
					Display.getDefault().syncExec(
						new Runnable() {
							public void run() {
								IMap map = PlatformGIS.getActiveMap();
								if (map != null) {
									wait.set(0, false);
								}
								
							}
						}
					);
					Thread.sleep(500);
				}
			} 
			catch (InterruptedException e) {
				// don't even bother, this is hack anyways
			}
		}
	}
    
    /**
     * Searches a String looking for URLs and returns the first one it can find.
     *
     * @param data
     * @return
     */
    private URL extractURL(String data) {
        String lines[] = data.split("\n"); //$NON-NLS-1$
        for (String line : lines) {
            
            //try direct conversion
            
            try {
                return new URL(line);
            } catch (MalformedURLException e) {
            }
            
            //search for anchor tags
            Pattern anchorPattern = Pattern.compile("<[Aa].*[Hh][Rr][Ee][Ff]=\"([Hh][Tt][Tt][Pp].*)\""); //$NON-NLS-1$
            Matcher anchorMatcher = anchorPattern.matcher(line);
            
            if (anchorMatcher.find()) {
                try {
                	String str=anchorMatcher.group(1);
                	if( str.contains("\""))
                		str=str.substring(0,str.indexOf("\""));
                    return new URL(str);
                } catch (MalformedURLException e) {
                    e.printStackTrace();
                }
            }
            
            //search for URL
            
            Pattern urlPattern = Pattern.compile("([Hh][Tt][Tt][Pp]\\S*)"); //$NON-NLS-1$
            Matcher urlMatcher = urlPattern.matcher(line);
            
            if (urlMatcher.find()) {
                try {
                    return new URL(urlMatcher.group(1));
                } catch (MalformedURLException e) {
                    e.printStackTrace();
                }
            }
        }
        
        return null;
    }

    class WizardProcessor implements ExtensionPointProcessor {
    	
    	private Object destination;
    	List<IDataDropWizard> wizards=new LinkedList<IDataDropWizard>();
		WizardProcessor(Object destination){
    		this.destination=destination;
    	}
    	
    	@SuppressWarnings("unchecked") //$NON-NLS-1$
		public void process(IExtension extension, IConfigurationElement element) throws Exception {
    		Class<Object> c=(Class<Object>) Class.forName(element.getAttribute("target")); //$NON-NLS-1$
    			if( c.isAssignableFrom(destination.getClass())){
    				IDataDropWizard current = (IDataDropWizard) element.createExecutableExtension("class"); //$NON-NLS-1$
    				current.setIDs(p.ids);
    				wizards.add(current);
    			}
    	}
    }
    
	class PageProcessor implements ExtensionPointProcessor {

		Object data;
		List<String> ids = new ArrayList<String>();
		
		PageProcessor(Object data) {
			this.data = data;
		}
		
		public void process(IExtension extension, IConfigurationElement element) throws Exception {
			UDIGImportPage page = null;
			try {
				page 
				 = (UDIGImportPage) element.createExecutableExtension("class"); //$NON-NLS-1$
				if (page.canProcess(data)) {
					ids.add(page.getId());
				}
			}
			catch(Throwable t) {
				String msg = net.refractions.udig.project.ui.internal.Policy.bind("WizardDropAction.error"); //$NON-NLS-1$
				CatalogPlugin.log(msg, t);
			}
		}
		
	}
	
	public static class CatalogWizard extends CatalogImportWizard implements IDataDropWizard{
        
		List<String> ids;
		
		public void setIDs(List<String> dataIDs){
			this.ids = dataIDs;	
		}
		
        @Override
        protected WizardPage[] getPrimaryPages() {
            
            DataSourceSelectionPage page = new DataSourceSelectionPage();
            page.select(ids);
            
            return new WizardPage[] { page };

        }
        
    }
	
	public static class MapWizard extends AddLayersWizard implements IDataDropWizard{
        
		List<String> ids;
		
		public void setIDs(List<String> dataIDs){
			this.ids = dataIDs;	
		}
	    
        @Override
        protected WizardPage[] getPrimaryPages() {
           
            DataSourceSelectionPage page = new DataSourceSelectionPage();
            page.select(ids);
            
			ResourceSelectionPage rpage = new ResourceSelectionPage(
				Policy.bind("AddLayersWizard.layerSelection") //$NON-NLS-1$
			);
			
            return new WizardPage[] { page, rpage };

        }
        
    }
}
