/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.project.ui.internal.commands.draw;

import java.awt.Color;
import java.io.IOException;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.ui.internal.Policy;
import net.refractions.udig.ui.Drawing;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.feature.Feature;
import org.geotools.referencing.CRS;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.geotools.styling.Symbolizer;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.MathTransform;

/**
 * Draws a feature on the screen.
 * 
 * @author jeichar
 * @since 0.9
 */
public class DrawFeatureCommand extends AbstractDrawCommand {
    
    private Feature feature;

    private Drawing drawing=Drawing.create();
	private CoordinateReferenceSystem featureCRS;
	
	Symbolizer[] syms=new Symbolizer[1];
    /**
     * @param feature
     * @param layer
     * @throws IOException 
     */
    public DrawFeatureCommand(Feature feature, ILayer layer) throws IOException {
    	this( feature, layer.getCRS(null));
    }

    /**
     * @param feature
     * @param crs
     */
    public DrawFeatureCommand(Feature feature, CoordinateReferenceSystem crs) {
        this.feature=feature;
        if( crs==null )
        	this.featureCRS=DefaultGeographicCRS.WGS84;
        else
        	this.featureCRS=crs;
        syms=Drawing.getSymbolizers(feature.getDefaultGeometry().getClass(), Color.ORANGE);
    }

    /**
     * @param feature
     * @param crs
     */
    public DrawFeatureCommand(Feature feature) {
    	this( feature, feature.getFeatureType().getDefaultGeometry().getCoordinateSystem() );
    }

    /**
     * @see net.refractions.udig.project.command.MapCommand#run()
     */
    public void run(IProgressMonitor monitor) throws Exception {
        MathTransform mt=null;
        try{
        mt=CRS.transform(featureCRS, 
                getMap().getViewportModel().getCRS(), true);
        }catch (Exception e) {
            mt=null;
        }
        drawing.drawFeature(graphics, feature, getMap().getViewportModel().worldToScreenTransform(), false, syms, mt);
    }

    /**
     * @see net.refractions.udig.project.command.MapCommand#copy()
     */
    public MapCommand copy() {
        return new DrawFeatureCommand(feature,featureCRS);
    }

    /**
     * @see net.refractions.udig.project.command.MapCommand#getName()
     */
    public String getName() {
        return Policy.bind("DrawFeatureCommand.drawFeature")+feature.getFeatureType().getTypeName(); //$NON-NLS-1$
    }

}
