/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.project.ui.internal.commands.draw;

import java.awt.Color;
import java.awt.Paint;
import java.awt.Shape;

import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.ui.internal.Policy;
import net.refractions.udig.ui.graphics.ViewportGraphics;

import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Draws the outline of a shape on the Acetate layer.
 * 
 * @author jeichar
 * @since 0.3
 */
public class DrawShapeCommand extends AbstractDrawCommand {

  Shape shape=null;
  Color paint=null;
  private int style=-1;
  private int width;
 
    /**
     * Creates a new instance of DrawShapeCommand
     */
    public DrawShapeCommand() {
      this( null, Color.BLACK, ViewportGraphics.LINE_SOLID, 1 );
    }

  /**
   * Creates a new instance of DrawShapeCommand
   * @param shape The shape to draw
   * @param color The paint to draw the shape with.
   * @param lineStyle the line style to use for the shape outline 
   * @param lineWidth the line width in pixels. 
   */
  public DrawShapeCommand(Shape shape, Color color, int lineStyle, int lineWidth) {
    this.shape=shape;
    this.paint=color;
    this.style=lineStyle;
    this.width=lineWidth;

  }

  /**
   * @see net.refractions.udig.project.internal.command.MapCommand#copy()
   */
  public MapCommand copy() {
    return new DrawShapeCommand(shape, paint, style, width);
  }
  

  /**
   * @see MapCommand#run()
   */
  public void run(IProgressMonitor monitor) {
      if(paint!=null)
          graphics.setColor(paint);
      if(style>-1)
          graphics.setStroke(style,width);
    graphics.draw(shape);
  }
  
  /**
   * @return Returns the paint.
   */
  public Paint getPaint() {
    return paint;
  }
  /**
   * @param paint The paint to set.
   */
  public void setPaint(Color paint) {
    this.paint = paint;
  }
  /**
   * @return Returns the shape.
   */
  public Shape getShape() {
    return shape;
  }
  /**
   * @param shape The shape to set.
   */
  public void setShape(Shape shape) {
    this.shape = shape;
  }
  /**
   * @return Returns the line style.
   */
  public int getLineStyle() {
    return style;
  }  /**
   * @return Returns the line width.
   */
  public int getLineWidth() {
    return width;
  }

  /**
 * Sets the line style
 * @param lineStyle the style of the line
 * @param lineWidth the width of the line
 */
public void setStroke(int lineStyle, int lineWidth) {
    this.style=lineStyle;
    this.width=lineWidth;
  }

/**
 * @see net.refractions.udig.project.command.MapCommand#getName()
 */
public String getName() {
    return Policy.bind("DrawShapeCommand.setStroke"); //$NON-NLS-1$
}
}
