/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal.properties;

import net.refractions.udig.project.ui.internal.Policy;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.geotools.feature.AttributeType;
import org.opengis.util.CodeList;

/**
 * A PropertyDescriptor for attributes of a Feature.
 * 
 * @author jeichar
 * @since 0.3
 */
public class AttributePropertyDescriptor extends PropertyDescriptor {
    protected AttributeType type;
    AttributeValidator validator;
    private String[] comboBoxList;

    /**
     * Creates a new instance of AttributePropertyDescriptor
     * 
     * @param id
     *            The object used to identify the value
     * @param displayName
     *            The Property display name
     * @param type
     *            The Attribute type that describes the attribute
     */
    public AttributePropertyDescriptor(Object id, String displayName, AttributeType type) {
        super(id, displayName);
        this.type = type;
        validator=new AttributeValidator(type);
        comboBoxList=createComboList();
    }
    
    /**
     * @see org.eclipse.ui.views.properties.PropertyDescriptor#createPropertyEditor(org.eclipse.swt.widgets.Composite)
     */
    public CellEditor createPropertyEditor(Composite parent) {
        if (Boolean.class.isAssignableFrom(type.getType()) || boolean.class.isAssignableFrom(type.getType()) )
            return new ComboBoxCellEditor(parent, new String[] { Policy.bind("AttributePropertyDescriptor.true"), Policy.bind("AttributePropertyDescriptor.false") }); //$NON-NLS-1$ //$NON-NLS-2$
        if (String.class.isAssignableFrom(type.getType()))
            return new TextCellEditor(parent);
        if (Integer.class.isAssignableFrom(type.getType()))
            return new TextCellEditor(parent);
        if (Double.class.isAssignableFrom(type.getType()))
            return new TextCellEditor(parent);
        if (Float.class.isAssignableFrom(type.getType()))
            return new TextCellEditor(parent);
        if (CodeList.class.isAssignableFrom(type.getType())) {
            return new ComboBoxCellEditor(parent, comboBoxList);
        }
        return super.createPropertyEditor(parent);
    }
    
    String[] createComboList(){
        if( !(CodeList.class.isAssignableFrom(type.getType())) )
            return null;
        CodeList list = (CodeList) type.createDefaultValue();
        CodeList[] family = list.family();
        String[] names = new String[family.length];
        for (int i = 0; i < names.length; i++) {
            names[i] = family[i].name();
        }
        
        return names;
    }
    
    /**
     * @see org.eclipse.ui.views.properties.PropertyDescriptor#getValidator()
     */
    protected ICellEditorValidator getValidator() {
        return validator;
    }
    
    /**
	 * @see org.eclipse.ui.views.properties.PropertyDescriptor#getLabelProvider()
	 */
	public ILabelProvider getLabelProvider() {
		return new LabelProvider(){
		    /**
             * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
             */
            public String getText( Object element ) {
                if( Boolean.class.isAssignableFrom(type.getType()) ){
                    return ((Integer)element).intValue()==1?"true":"false";  //$NON-NLS-1$ //$NON-NLS-2$
                }
                 return element.toString();   
            }
		};
	}	
    /**
     * @return Returns the comboBoxList.
     */
    public String[] getComboBoxList() {
        return comboBoxList;
    }
}