/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This
 * library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal.properties;

import net.refractions.udig.core.internal.GeometryBuilder;
import net.refractions.udig.project.ui.internal.Policy;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource2;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryCollection;
import com.vividsolutions.jts.geom.LinearRing;
import com.vividsolutions.jts.geom.Polygon;

/**
 * TODO provide type description
 * 
 * @author jeichar
 * @since TODO provide version
 */
public class GeomPropertySource implements IPropertySource2 {
    private final String TYPE = "TYPE"; //$NON-NLS-1$
    private final String AREA = "AREA"; //$NON-NLS-1$
    private static final String COORDS = "COORDS"; //$NON-NLS-1$

    private Geometry geom;
    GeometryCollection collection;
    IPropertyDescriptor[] propertyDescriptors;
    GeometryBuilder builder = GeometryBuilder.create();
    private static final String SHELL = "SHELL"; //$NON-NLS-1$
    private static final String HOLES = "HOLES"; //$NON-NLS-1$
    private static final String COLLECTION = "COLLECTION"; //$NON-NLS-1$
    /**
     * Creates a new instance of GeomPropertySource
     * 
     * @param geometry the geometry that is the source for this PropertySource
     */
    public GeomPropertySource( Geometry geometry ) {
        this.geom = geometry;
        if (geometry instanceof GeometryCollection) {
            collection = (GeometryCollection) geometry;
            createGeometryCollectionDescriptors();
        } else if (geometry instanceof Polygon) {
            createPolygonDescriptors();
        } else {
            createGeometryDescriptors();
        }
    }

    /**
     * TODO summary sentence for createPolygonDescriptors ...
     */
    private void createPolygonDescriptors() {
        Polygon poly = (Polygon) geom;
        propertyDescriptors = new IPropertyDescriptor[poly.getNumInteriorRing() + 2];
        propertyDescriptors[0] = new PropertyDescriptor(new ID(AREA), Policy.bind("GeomPropertySource.area")); //$NON-NLS-1$
        propertyDescriptors[1] = new GeometryPropertyDescriptor(new ID(SHELL), Policy.bind("GeomPropertySource.exteriorShell")); //$NON-NLS-1$
        for( int i = 0; i < propertyDescriptors.length - 2; i++ ) {
            propertyDescriptors[i + 2] = new GeometryPropertyDescriptor(new ID(HOLES, i), Policy.bind("GeomPropertySource.polygonHole")); //$NON-NLS-1$
        }
    }

    private void createGeometryDescriptors() {
        IPropertyDescriptor coords=new PropertyDescriptor(new ID(COORDS), Policy.bind("GeomPropertySource.coordinates")){ //$NON-NLS-1$
            /**
             * @see org.eclipse.ui.views.properties.PropertyDescriptor#getLabelProvider()
             */
            public ILabelProvider getLabelProvider() {
                return new LabelProvider(){
                    /**
                     * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
                     */
                    public String getText( Object element ) {
                        return ""; //$NON-NLS-1$
                    }
                };
            }
        };
        propertyDescriptors = new IPropertyDescriptor[]{
                new PropertyDescriptor(new ID(AREA), Policy.bind("GeomPropertySource.area")), //$NON-NLS-1$
                coords
                };
    }

    void createGeometryCollectionDescriptors() {
        propertyDescriptors = new IPropertyDescriptor[collection.getNumGeometries() + 1];
        propertyDescriptors[0] = new PropertyDescriptor(new ID(AREA), Policy.bind("GeomPropertySource.area")); //$NON-NLS-1$
        for( int i = 0; i < propertyDescriptors.length - 1; i++ ) {
            Geometry geometry = collection.getGeometryN(i);
            propertyDescriptors[i + 1] = new GeometryPropertyDescriptor(new ID(COLLECTION, i, collection.getGeometryN(i)),
                    geometry.getGeometryType());
        }
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
     */
    public Object getEditableValue() {
        return geom;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
    public IPropertyDescriptor[] getPropertyDescriptors() {
        return propertyDescriptors;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
     */
    public Object getPropertyValue( Object idObject ) {
        ID id=(ID) idObject;
        if (id.id==COLLECTION) {
            return new GeomPropertySource(collection.getGeometryN(id.index));
        }
        if (id.id == TYPE)
            return geom.getGeometryType();
        if (id.id == AREA)
            return String.valueOf(geom.getArea());
        if (id.id == COORDS){
            return new CoordinateSetPropertySource(geom.getCoordinates(), geom);
        }
        if (id.id == HOLES) {
            Polygon poly = (Polygon) geom;
            return new GeomPropertySource(poly.getInteriorRingN(id.index));
        }
        if (id.id == SHELL) {
            Polygon poly = (Polygon) geom;
            return new GeomPropertySource(poly.getExteriorRing());
        }
        return null;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
     */
    public boolean isPropertySet( Object id ) {
        // TODO Auto-generated method stub
        return false;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
     */
    public void resetPropertyValue( Object id ) {
        // TODO Auto-generated method stub
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
     */
    public void setPropertyValue( Object idObject, Object value ) {
        ID id=(ID) idObject;
        if (id.id == COORDS) {
            geom = builder.createGeometry(geom.getClass(), (Coordinate[]) value);
        }
        if (id.id == SHELL) {
            Polygon polygon = (Polygon) geom;
            LinearRing[] rings = new LinearRing[polygon.getNumInteriorRing()];
            for( int i = 0; i < rings.length; i++ ) {
                rings[i] = (LinearRing) polygon.getInteriorRingN(i);
            }
            geom = builder.factory.createPolygon((LinearRing) value, rings);
        }
        if (id.id == HOLES) {
            Polygon polygon = (Polygon) geom;
            LinearRing[] rings = new LinearRing[polygon.getNumInteriorRing()];
            for( int i = 0; i < rings.length; i++ ) {
                rings[i] = (LinearRing) polygon.getInteriorRingN(i);
            }
            rings[id.index] = (LinearRing) value;
            geom = builder.factory.createPolygon((LinearRing) polygon.getExteriorRing(), rings);
        }
        if( id.id == COLLECTION ){
            Geometry[] geoms=new Geometry[collection.getNumGeometries()];
            for( int i = 0; i < geoms.length; i++ ) {
                geoms[i]=collection.getGeometryN(i);
            }
            geoms[id.index]=(Geometry) value;
            geom=collection=builder.factory.createGeometryCollection(geoms);
        }
    }

    class ID {
        int index;
        String id;
        Object data;
        ID( String id, int index, Object data ) {
            this.index = index;
            this.id = id;
            this.data = data;
        }
        ID( String id, int index ) {
            this(id, index, null);
        }
        ID( String id ) {
            this(id, 0, null);
        }
    }

    /**
     * @see org.eclipse.ui.views.properties.IPropertySource2#isPropertyResettable(java.lang.Object)
     */
    public boolean isPropertyResettable( Object id ) {
        //TODO implement method body
        return true;
    }
}
