/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal.render.displayAdapter.impl;

import java.util.Stack;

import net.refractions.udig.project.render.displayAdapter.IMapDisplayListener;
import net.refractions.udig.project.render.displayAdapter.MapDisplayEvent;
import net.refractions.udig.project.ui.internal.Policy;
import net.refractions.udig.project.ui.internal.ProjectUIPlugin;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseEvent;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseListener;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseMotionListener;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseWheelEvent;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseWheelListener;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.util.ListenerList;

class EventJob extends Job {

    /**
     * Construct <code>EventJob</code>.
     */
    public EventJob() {
        super(Policy.bind("EventJob.job.title")); //$NON-NLS-1$
        setSystem(true);
    }

    private ListenerList paneListener = new ListenerList();
    private ListenerList wheel = new ListenerList();
    private ListenerList motion = new ListenerList();
    private ListenerList mouse = new ListenerList();

    static final int DOUBLE_CLICK = 1;
    static final int PRESSED = 2;
    static final int RELEASED = 3;
    static final int MOVED = 4;
    static final int DRAGGED = 5;
    static final int ENTERED = 6;
    static final int EXITED = 7;
    static final int WHEEL = 8;
    static final int RESIZED = 9;
    private Stack<Event> jobs = new Stack<Event>();
    private boolean down;
    /**
     * @see org.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.IProgressMonitor)
     */
    protected IStatus run( IProgressMonitor monitor ) {
        try {
            if (jobs.empty())
                return Status.OK_STATUS;
            Event event = jobs.pop();
            switch( event.type ) {
            case DOUBLE_CLICK: {
                Object[] larray = mouse.getListeners();
                for( int i = 0; i < larray.length; i++ ) {
                    MapMouseListener l = (MapMouseListener) larray[i];
                    try {
                        l.mouseDoubleClicked((MapMouseEvent) event.data);
                    } catch (Throwable t) {
                        ProjectUIPlugin.log( "Error processing a double clicked event", t);//$NON-NLS-1$
                    }
                }
                break;
            }
            case PRESSED: {
                Object[] larray = mouse.getListeners();
                for( int i = 0; i < larray.length; i++ ) {
                    MapMouseListener l = (MapMouseListener) larray[i];
                    try {
                        l.mousePressed((MapMouseEvent) event.data);
                    } catch (Throwable t) {
                        ProjectUIPlugin.log( "Error processing a mouse pressed event", t);//$NON-NLS-1$
                    }
                }
                break;
            }
            case RELEASED: {
                Object[] larray = mouse.getListeners();
                for( int i = 0; i < larray.length; i++ ) {
                    MapMouseListener l = (MapMouseListener) larray[i];
                    try {
                        l.mouseReleased((MapMouseEvent) event.data);
                    } catch (Throwable t) {
                        ProjectUIPlugin.log( "Error processing a mouse released event", t);//$NON-NLS-1$
                    }
                }
                break;
            }
            case MOVED: {
                Object[] larray = motion.getListeners();
                for( int i = 0; i < larray.length; i++ ) {
                    MapMouseMotionListener l = (MapMouseMotionListener) larray[i];
                    try {
                        l.mouseMoved((MapMouseEvent) event.data);
                    } catch (Throwable t) {
                        ProjectUIPlugin.log( "Error processing a mouse moved event", t);//$NON-NLS-1$
                    }
                }
                break;
            }
            case DRAGGED: {
                Object[] larray = motion.getListeners();
                for( int i = 0; i < larray.length; i++ ) {
                    MapMouseMotionListener l = (MapMouseMotionListener) larray[i];
                    try {
                        l.mouseDragged((MapMouseEvent) event.data);
                    } catch (Throwable t) {
                        ProjectUIPlugin.log( "Error processing a mouse dragged event", t);//$NON-NLS-1$
                    }

                }
                break;
            }
            case ENTERED: {
                Object[] larray = mouse.getListeners();
                for( int i = 0; i < larray.length; i++ ) {
                    MapMouseListener l = (MapMouseListener) larray[i];
                    try {
                        l.mouseEntered((MapMouseEvent) event.data);
                    } catch (Throwable t) {
                        ProjectUIPlugin.log(  "Error processing a mouse entered event", t);//$NON-NLS-1$
                    }
                }
            }
            case EXITED: {
                Object[] larray = mouse.getListeners();
                for( int i = 0; i < larray.length; i++ ) {
                    MapMouseListener l = (MapMouseListener) larray[i];
                    try {
                        l.mouseExited((MapMouseEvent) event.data);
                    } catch (Throwable t) {
                        ProjectUIPlugin.log(  "Error processing a moouse exited event", t);//$NON-NLS-1$
                    }
                }
                break;
            }
            case WHEEL: {
                Object[] larray = wheel.getListeners();
                for( int i = 0; i < larray.length; i++ ) {
                    MapMouseWheelListener l = (MapMouseWheelListener) larray[i];
                    try {
                        l.mouseWheelMoved((MapMouseWheelEvent) event.data);
                    } catch (Throwable t) {
                        ProjectUIPlugin.log(  "Error processing a mouse wheel event", t);//$NON-NLS-1$
                    }
                }
                break;
            }
            case RESIZED: {
                Object[] larray = paneListener.getListeners();
                for( int i = 0; i < larray.length; i++ ) {
                    IMapDisplayListener l = (IMapDisplayListener) larray[i];
                    try {
                        l.sizeChanged((MapDisplayEvent) event.data);
                    } catch (Throwable t) {
                        ProjectUIPlugin.log(  "Error processing a resized event", t);//$NON-NLS-1$
                    }
                }
                break;
            }
            default:
                System.err.println("Event requested that does not exist " + event.type); //$NON-NLS-1$
            }
            if (!jobs.empty())
                schedule();
        } catch (Throwable t) {
            ProjectUIPlugin.log( "Error processing a mouse or resize event", t);//$NON-NLS-1$
        }
        return Status.OK_STATUS;
    }

    /**
     * Adds a MapEditorListener
     * 
     * @param l the listener
     */
    public void addMapEditorListener( IMapDisplayListener l ) {
        paneListener.add(l);
    }
    /**
     * Removes a MapEditorListener Listener
     * 
     * @param l the listener
     */
    public void removeMapEditorListener( IMapDisplayListener l ) {
        paneListener.remove(l);
    }
    /**
     * Adds a MapMouseListener Listener
     * 
     * @param l the listener to add.
     */
    public void addMouseListener( MapMouseListener l ) {
        mouse.add(l);
    }
    /**
     * Adds a MapMouseMotionListener Listener
     * 
     * @param l the listener to add.
     */
    public void addMouseMotionListener( MapMouseMotionListener l ) {
        motion.add(l);

    }
    /**
     * Adds a MapMouseWheelListener Listener
     * 
     * @param l the listener to add.
     */
    public void addMouseWheelListener( MapMouseWheelListener l ) {
        wheel.add(l);
    }
    /**
     * Removes a MapMouseListener Listener
     * 
     * @param l the listener to remove.
     */
    public void removeMouseListener( MapMouseListener l ) {
        mouse.remove(l);
    }
    /**
     * Removes a MapMouseMotionListener Listener
     * 
     * @param l the listener to remove.
     */
    public void removeMouseMotionListener( MapMouseMotionListener l ) {
        motion.remove(l);
    }
    /**
     * Removes a MapMouseWheelListener Listener
     * 
     * @param l the listener to remove.
     */
    public void removeMouseWheelListener( MapMouseWheelListener l ) {
        wheel.remove(l);
    }
    /**
     * fires an event to all listeners. If an event is being processed then the new one is ignored
     * because some event take a while to process.
     * 
     * @param type the type of the event
     * @param event the event data object
     */
    public void fire( int type, Object event ) {
        if (jobs.size() > 200
                && !(type == RESIZED || (!down && (type == RELEASED) || type == ENTERED || type == EXITED))) {
        	if( ProjectUIPlugin.isDebugging("debug")) //$NON-NLS-1$
             System.err.println("dropped job: "+type); //$NON-NLS-1$
            return;
        }
        if (type == PRESSED) {
            down = true;
        }
        if (type == RELEASED) {
            down = false;
        }
        jobs.push(new Event(type, event));
        schedule();
    }

    private class Event {
        int type;
        Object data;
        Event( int type, Object event ) {
            this.type = type;
            this.data = event;
        }
    }
}