/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal.tool.display;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import net.refractions.udig.core.internal.ExtensionPointList;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.ProjectPackage;
import net.refractions.udig.project.internal.render.RenderManager;
import net.refractions.udig.project.ui.internal.MapEditor;
import net.refractions.udig.project.ui.internal.Policy;
import net.refractions.udig.project.ui.internal.tool.ToolContext;
import net.refractions.udig.project.ui.internal.tool.ToolFactory;
import net.refractions.udig.project.ui.tool.Tool;
import net.refractions.udig.project.ui.tool.ToolConstants;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.SubCoolBarManager;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPageListener;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.SubActionBars2;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.commands.ICommandService;

/**
 * Manages Edit tools activation and registration.
 * 
 * @author jeichar
 * @since 0.6.0
 */
public class ToolManager {

    private EditorListener editorListener;
    /**
     * This is a list of all tool actions(buttons) that are not part of the editor toolbar. For
     * example the info view may have a tool as part of its toolbar which is a proxy for the real
     * tool on the editor view.
     */
    Set<IAction> registeredToolActions = new HashSet<IAction>();
    MapEditor currentEditor;
    
    List<String> categoryIds=new ArrayList<String>();
    List<ModalToolCategory> modalCategories = new LinkedList<ModalToolCategory>();
    List<ActionToolCategory> actionCategories = new LinkedList<ActionToolCategory>();
    List<MenuToolCategory> menuCategories = new LinkedList<MenuToolCategory>();
    List<ToolProxy> backgroundTools = new LinkedList<ToolProxy>();
    ISharedImages sharedImages = PlatformUI.getWorkbench().getSharedImages();

    Action redoAction;
    Action undoAction;
    Action forwardAction;
    Action backwardAction;
    private AdapterImpl commandListener;

    private boolean initialized = false;

    /**
     * Construct <code>ToolManager</code>.
     */
    public ToolManager() {

        processCategories();
        processTools();
        removeEmptyCategories();
        Collections.sort(categoryIds, new CategorySorter());
        setCommandHandlers();
        
        watchEditorsPlease(); // listen for map editors
    }

    /**
     *
     */
    private void watchEditorsPlease() {
        IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        IWorkbenchPage page = window.getActivePage();
        if( page != null ) {
            watchEditorsPlease( page  );    
        }
        else {    // wait for page
            PlatformUI.getWorkbench().getActiveWorkbenchWindow().addPageListener(
              new IPageListener() {
                public void pageActivated( IWorkbenchPage page ) {
                    // hope opened will work?
                }
                public void pageClosed( IWorkbenchPage page ) {
                    // should not get here
                }
                public void pageOpened( IWorkbenchPage page ) {
                    watchEditorsPlease( page  );
                    PlatformUI.getWorkbench().getActiveWorkbenchWindow().removePageListener( this );                    
                }
              }
            );
        }
    }
    void watchEditorsPlease( IWorkbenchPage page ) {
        if( page == null )
        	throw new AssertionError();
        EditorListener watch =  getEditorListener();
        page.addPartListener( watch );
        if (page.getActiveEditor() instanceof MapEditor) {
            currentEditor = (MapEditor) page.getActiveEditor();
        }                                
    }

    /**
     * Populates the categories with their associated tools
     */
    private void processTools() {
        ExtensionPointList extensionList = new ExtensionPointList(Tool.EXTENSION_ID);
        for( IConfigurationElement element : extensionList ) {
            IExtension extension=element.getDeclaringExtension();
            String type = element.getName();
            
            if (type.equals("backgroundTool")){ //$NON-NLS-1$

                ToolProxy proxy = new ToolProxy(extension, element);
            	backgroundTools.add(proxy);
            }else if( type.equals("modalTool")){ //$NON-NLS-1$
	            String categoryId=element.getAttribute("categoryId"); //$NON-NLS-1$
	            ToolProxy proxy = new ToolProxy(extension, element);
	            
	            addToModalCategory(categoryId, proxy);
	            addToMenuCategory(categoryId, proxy);
            } else if( type.equals("actionTool")){ //$NON-NLS-1$
	            String categoryId=element.getAttribute("categoryId"); //$NON-NLS-1$
	            ToolProxy proxy = new ToolProxy(extension, element);
	            
	            addToActionCategory(categoryId, proxy);
	            addToMenuCategory(categoryId, proxy);
            } 
        }
    }

	private void addToModalCategory(String categoryId, ToolProxy proxy) {
		ModalToolCategory modalCategory = findModalCategory(categoryId);
		if (modalCategory == null) {
			modalCategory = findModalCategory(Policy.bind("ToolManager.other")); //$NON-NLS-1$
			if (modalCategory == null) {
				modalCategory = new ModalToolCategory(this);
				modalCategories.add(modalCategory);
			}
		}
		if (proxy.isOnToolbar()) {
			modalCategory.add(proxy);
		}
	}

	private void addToMenuCategory(String categoryId, ToolProxy proxy) {
		MenuToolCategory category = findMenuCategory(categoryId);
		if (category == null) {
			category = findMenuCategory(Policy.bind("ToolManager.other")); //$NON-NLS-1$
			if (category == null) {
				category = new MenuToolCategory(this);
				menuCategories.add(category);
			}
		}
		if (proxy.isOnToolbar()) {
			category.add(proxy);
		}
	}

	private void addToActionCategory(String categoryId, ToolProxy proxy) {
		ActionToolCategory category = findActionCategory(categoryId);
		if (category == null) {
			category = findActionCategory(Policy.bind("ToolManager.other")); //$NON-NLS-1$
			if (category == null) {
				category = new ActionToolCategory(this);
				actionCategories.add(category);
			}
		}
		if (proxy.isOnToolbar()) {
			category.add(proxy);
		}
	}


    /**
	 * Processes the extension point and creates all the categories.
	 */
    private void processCategories() {
        ExtensionPointList extension = new ExtensionPointList(Tool.EXTENSION_ID);
        for( IConfigurationElement element : extension ) {
            if( !element.getName().equals("category") ) //$NON-NLS-1$
                continue;
            ModalToolCategory modalCategory;
            String id = element.getAttribute("id"); //$NON-NLS-1$
            categoryIds.add(id);
            modalCategory = findModalCategory(id);
            if (modalCategory == null) {
            	modalCategory = new ModalToolCategory(element, ToolManager.this);
                modalCategories.add(modalCategory);
            }
            ActionToolCategory actionCategory;
            actionCategory = findActionCategory(id);
            if (actionCategory == null) {
            	actionCategory = new ActionToolCategory(element, ToolManager.this);
                actionCategories.add(actionCategory);
            }
            MenuToolCategory category;
            category = findMenuCategory(id);
            if (category == null) {
                category = new MenuToolCategory(element, ToolManager.this);
                menuCategories.add(category);
            }
        }
    }


    private void removeEmptyCategories() {
        List<ToolCategory> toRemove = new ArrayList<ToolCategory>();
        for( ActionToolCategory category : actionCategories ) {
            if (category.items.size() == 0)
                toRemove.add(category);
        }
        actionCategories.removeAll(toRemove);
        for( ModalToolCategory category : modalCategories ) {
            if (category.items.size() == 0)
                toRemove.add(category);
        }
        modalCategories.removeAll(toRemove);
        for( MenuToolCategory category : menuCategories ) {
            if (category.items.size() == 0)
                toRemove.add(category);
        }
        menuCategories.removeAll(toRemove);
    }

    private void setCommandHandlers() {
        Set<String> ids = new HashSet<String>();
        ICommandService service=(ICommandService) PlatformUI.getWorkbench().getAdapter(ICommandService.class);
        for( ModalToolCategory category : modalCategories ) {
            if (!ids.contains(category.getId())) {
                ids.add(category.getId());
                category.setCommandHandlers(service);
            }
        }
        for( ActionToolCategory category : actionCategories ) {
            if (!ids.contains(category.getId())) {
                ids.add(category.getId());
                category.setCommandHandlers(service);
            }
        }
        for( MenuToolCategory category : menuCategories ) {
            if (!ids.contains(category.getId())) {
                ids.add(category.getId());
                category.setCommandHandlers(service);
            }
        }

    }

    /**
     * @return the current Editor if it is a MapEditor
     */
    public MapEditor getCurrentEditor() {
        return currentEditor;
    }

    public void setCurrentEditor( MapEditor editor ) {
	    deactivateTools(editor);
	
	    currentEditor = editor;
	
	    if (!initialized && currentEditor != null) {
	        contributeGlobalActions();
	        contributeToCoolBar((SubCoolBarManager) getActionBars().getCoolBarManager());
	        contributeToMenu(getActionBars().getMenuManager()); //$NON-NLS-1$
	        getActionBars().updateActionBars();
	        initialized = true;
	    }
	    if (editor != null){
	        setActiveTool(editor);
	    }
	
	}

    void deactivateTools( MapEditor editor ) {
        if (currentEditor == null)
            return;
        currentEditor.getMap().eAdapters().remove(getCommandListener());
        if (editor != null)
            return;
        disableCategory(modalCategories);
        disableCategory(actionCategories);
        disableCategory(menuCategories);
        for( ToolProxy tool: backgroundTools ){
        	tool.setActive(false);
        }
        for( Iterator iter = registeredToolActions.iterator(); iter.hasNext(); )
            ((IAction) iter.next()).setEnabled(false);

    }

    private void disableCategory( List categories ) {
        for( Iterator iter = categories.iterator(); iter.hasNext(); ) {
            ToolCategory category = (ToolCategory) iter.next();
            for( Iterator titer = category.iterator(); titer.hasNext(); ) {
                ToolProxy tool = (ToolProxy) titer.next();
                tool.setActive(false);
            }
        }
    }

    /**
     * Sets the context of the currently active tool and ensures that all tools are enabled. This is
     * a small hack so that tools are active even when the editor is not in focus.
     * 
     * @param editor
     */
    void setActiveTool( MapEditor editor ) {
        editor.getMap().eAdapters().add(getCommandListener());
        RenderManager rm = editor.getMap().getRenderManagerInternal();
        ToolContext tools = ToolFactory.eINSTANCE.createToolContext();
        tools.setMapInternal(rm.getMapInternal());
        tools.setRenderManagerInternal(rm);

        setContext(modalCategories, tools);
        setContext(actionCategories, tools);
        setContext(menuCategories, tools);

        for( ToolProxy tool: backgroundTools){
        	tool.setContext(tools);
        }
        for( Iterator iter = registeredToolActions.iterator(); iter.hasNext(); )
            ((IAction) iter.next()).setEnabled(true);

        setCommandActions(editor.getMap());
    }

    private void setContext( List categories, ToolContext tools ) {
        for( Iterator iter = categories.iterator(); iter.hasNext(); ) {
            ToolCategory category = (ToolCategory) iter.next();
            for( Iterator titer = category.iterator(); titer.hasNext(); ) {
                ToolProxy tool = (ToolProxy) titer.next();
                tool.setContext(tools);
            }
        }
    }

    /**
     * Adds an Action that executes a tool to the toolbar.
     * 
     * @param action
     */
    public void addToolAction( IAction action ) {
        registeredToolActions.add(action);
        action.setEnabled(currentEditor != null);
    }

    /**
     * Creates a action that acts as a proxy for the tool in the editor toolbar.
     * <p>
     * The client code must set the name image descriptor etc... of the Action
     * </p>
     * 
     * @param toolID the id of the tool
     * @param categoryID the category the tool is part of
     * @return a proxy action that can be put in other toolbars
     */
    public IAction createToolAction( final String toolID, final String categoryID ) {
        IAction toolAction = new Action(){
            public void run() {
                IAction action = getTool(toolID, categoryID);
                if (action != null && action.isEnabled()) {
                    action.run();
                }
            }
        };
        addToolAction(toolAction);
        return toolAction;
    }

    /**
     * @return A listener that keeps track of the last active editor and enables actions when the
     *         editor is a MapEditor.
     */
    public EditorListener getEditorListener() {
        if (editorListener == null) {
            synchronized (this) {
                if (editorListener != null)
                    return editorListener;
                editorListener = new EditorListener();
            }
        }
        return editorListener;
    }
    
    ActionToolCategory findActionCategory(String id){
    	for( ActionToolCategory category : actionCategories ) {
          if (category.getId().equals(id))
              return category;
      }
    	return null;
    }
    
    MenuToolCategory findMenuCategory(String id){
    	for( MenuToolCategory category : menuCategories ) {
          if (category.getId().equals(id))
              return category;
      }
    	return null;
    }
    
    ModalToolCategory findModalCategory(String id){
    	for( ModalToolCategory category : modalCategories ) {
          if (category.getId().equals(id))
              return category;
      }
    	return null;
    }

    void contributeToMenu( IMenuManager manager ) {
    	IMenuManager edit = manager.findMenuUsingPath(IWorkbenchActionConstants.M_EDIT);
    	edit.appendToGroup(IWorkbenchActionConstants.EDIT_START, getUNDOAction());
    	edit.appendToGroup(IWorkbenchActionConstants.EDIT_START, getREDOAction());
    	IMenuManager nav=manager.findMenuUsingPath("nav"); //$NON-NLS-1$
    	nav.appendToGroup(IWorkbenchActionConstants.NAV_START, getBACKWARD_HISTORYAction());
    	nav.appendToGroup(IWorkbenchActionConstants.NAV_START, getFORWARD_HISTORYAction());
        if( !manager.isVisible() )
            manager.setVisible(true);
        for( MenuToolCategory category:menuCategories){
            category.contribute(manager);
        }
    }

    private Action getREDOAction() {
        if (redoAction == null) {
            redoAction = new Action(){
                /**
                 * @see org.eclipse.jface.action.Action#run()
                 */
                public void run() {
                    if (getCurrentEditor() != null)
                        getCurrentEditor().getMap().redo();
                }
            };
            redoAction.setImageDescriptor(sharedImages
                    .getImageDescriptor(ISharedImages.IMG_TOOL_REDO));
            redoAction.setText(Policy.bind("ToolManager.redoAction")); //$NON-NLS-1$
        }

        return redoAction;
    }

    private Action getUNDOAction() {
        if (undoAction == null) {
            undoAction = new Action(){
                /**
                 * @see org.eclipse.jface.action.Action#run()
                 */
                public void run() {
                    if (getCurrentEditor() != null)
                        getCurrentEditor().getMap().undo();
                }
            };
            undoAction.setImageDescriptor(sharedImages
                    .getImageDescriptor(ISharedImages.IMG_TOOL_UNDO));
            undoAction.setText(Policy.bind("ToolManager.undoAction")); //$NON-NLS-1$
        }
        return undoAction;
    }

    public Action getFORWARD_HISTORYAction() {
        if (forwardAction == null) {
            forwardAction = new Action(){
                /**
                 * @see org.eclipse.jface.action.Action#run()
                 */
                public void run() {
                    if (getCurrentEditor() != null)
                        getCurrentEditor().getMap().forewardHistory();
                }
            };
            forwardAction.setImageDescriptor(sharedImages
                    .getImageDescriptor(ISharedImages.IMG_TOOL_FORWARD));
            forwardAction.setText(Policy.bind("ToolManager.forward")); //$NON-NLS-1$
            forwardAction.setToolTipText(Policy.bind("ToolManager.forward.tooltip")); //$NON-NLS-1$
        }
        return forwardAction;
    }

    public Action getBACKWARD_HISTORYAction() {
        if (backwardAction == null) {
            backwardAction = new Action(){
                /**
                 * @see org.eclipse.jface.action.Action#run()
                 */
                public void run() {
                    if (getCurrentEditor() != null)
                        getCurrentEditor().getMap().backwardHistory();
                }

            };
            backwardAction.setImageDescriptor(sharedImages
                    .getImageDescriptor(ISharedImages.IMG_TOOL_BACK));
            backwardAction.setText(Policy.bind("ToolManager.back")); //$NON-NLS-1$
            backwardAction.setToolTipText(Policy.bind("ToolManager.back.tooltip")); //$NON-NLS-1$
        }
        return backwardAction;
    }
    //    
    // void contributeToStatusLine( IStatusLineManager statusLineManager ) {
    // for( Iterator iter = categories.iterator(); iter.hasNext(); ) {
    // boolean addCategory = true;
    // ToolCategory category = (ToolCategory) iter.next();
    // if (addCategory) {
    // category.contribute(statusLineManager, true);
    // }
    // }
    // }

    private void createCategoryToolbar( String categoryID, SubCoolBarManager cbmanager ) {
        ToolBarManager manager = new ToolBarManager(SWT.FLAT);

        ActionToolCategory actionCategory=findActionCategory(categoryID);
//        ModalToolCategory modalCategory=findModalCategory(categoryID);
        if( actionCategory!=null )
            actionCategory.contribute(manager);
//        if( modalCategory!=null )
//            modalCategory.contribute(manager);
        
        if( manager!=null && manager.getItems().length>0 )
            cbmanager.add(manager);
    }
    private void createModalToolToolbar( SubCoolBarManager cbmanager ) {
        ToolBarManager manager = new ToolBarManager(SWT.FLAT);

        for( ModalToolCategory modalCategory : modalCategories ) {
            if( modalCategory!=null )
                modalCategory.contribute(manager);
        }
        if( manager!=null && manager.getItems().length>0 )
            cbmanager.add(manager);
    }

    private void createActionToolToolbar( SubCoolBarManager cbmanager ) {
        ToolBarManager manager = new ToolBarManager(SWT.FLAT);

        for( ActionToolCategory category : actionCategories ) {
            if( category!=null )
                category.contribute(manager);
        }
        if( manager!=null && manager.getItems().length>0 )
            cbmanager.add(manager);
    }

    void contributeToCoolBar( SubCoolBarManager cbmanager ) {
        cbmanager.setVisible(true); 

        ToolBarManager manager = new ToolBarManager(SWT.FLAT);
        manager.add(getActionBars().getGlobalActionHandler(ActionFactory.BACKWARD_HISTORY.getId()));
        manager.add(getActionBars().getGlobalActionHandler(ActionFactory.FORWARD_HISTORY.getId()));
        cbmanager.add(manager);
//        for(String category: categoryIds){
//            createCategoryToolbar(category, cbmanager);
//        }
        createActionToolToolbar(cbmanager);
        createModalToolToolbar(cbmanager);
    }

    SubActionBars2 getActionBars() {
        if (currentEditor == null)
            return null;
        return currentEditor.getActionbar();
    }

    private void contributeGlobalActions() {
        getActionBars().setGlobalActionHandler(ActionFactory.BACKWARD_HISTORY.getId(),
                getBACKWARD_HISTORYAction());
        getActionBars().setGlobalActionHandler(ActionFactory.FORWARD_HISTORY.getId(),
                getFORWARD_HISTORYAction());
        getActionBars().setGlobalActionHandler(ActionFactory.UNDO.getId(), getUNDOAction());
        getActionBars().setGlobalActionHandler(ActionFactory.REDO.getId(), getREDOAction());
    }

    void dispose() {
        for( ToolCategory category : modalCategories ) {
            category.dispose(getCurrentEditor().getEditorSite().getActionBars());
        }
        for( ToolCategory category : actionCategories ) {
            category.dispose(getCurrentEditor().getEditorSite().getActionBars());
        }
        for( ToolCategory category : menuCategories ) {
            category.dispose(getCurrentEditor().getEditorSite().getActionBars());
        }
        for( ToolProxy tool: backgroundTools){
        	tool.dispose();
        }
        initialized = false;
    }

    Adapter getCommandListener() {
        if (commandListener == null) {
            commandListener = new AdapterImpl(){
                /**
                 * @see org.eclipse.emf.common.notify.impl.AdapterImpl#notifyChanged(org.eclipse.emf.common.notify.Notification)
                 */
                public void notifyChanged( Notification arg0 ) {
                    Map map = (Map) getTarget();
                    switch( arg0.getFeatureID(arg0.getNotifier().getClass()) ) {
                    case ProjectPackage.MAP__COMMAND_STACK:
                        setCommandActions(map);
                        break;
                    case ProjectPackage.MAP__NAV_COMMAND_STACK:
                        setCommandActions(map);
                        break;
                    }

                }
            };
        }
        return commandListener;
    }

    void setCommandActions( Map map ) {
        if (map.getCommandStack().canRedo())
            redoAction.setEnabled(true);
        else
            redoAction.setEnabled(false);
        if (map.getCommandStack().canUndo())
            undoAction.setEnabled(true);
        else
            undoAction.setEnabled(false);
        if (map.getNavCommandStack().hasBackHistory())
            backwardAction.setEnabled(true);
        else
            backwardAction.setEnabled(false);
        if (map.getNavCommandStack().hasForwardHistory())
            forwardAction.setEnabled(true);
        else
            forwardAction.setEnabled(false);
    }

    /**
     * Returns the tool identified by an id and a category.
     * 
     * @param toolID the id of the tool to find
     * @param categoryID the id of the category the tool is part of
     * @return the tool identified or null if the tool does not exist.
     */
    public IAction getTool( String toolID, String categoryID ) {
        ToolCategory category = findModalCategory(categoryID);

        if( category == null )
        	category=findActionCategory(categoryID);
        
        if( category == null )
        	category=findMenuCategory(categoryID);
        
        if (category != null) {
            for( Iterator iter2 = category.iterator(); iter2.hasNext(); ) {
                ToolProxy tool = (ToolProxy) iter2.next();
                if (tool.getId().equals(toolID))
                    return tool.getAction();
            }
        }

        return null;
    }

    /**
     * Listens to Activity events and keeps track of the current Editor.
     * 
     * @author jeichar
     * @since 0.6.0
     */
    public class EditorListener implements IPartListener2 {

        EditorListener() {
            //PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().addPartListener(this);
//            if (PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
//                    .getActiveEditor() instanceof MapEditor)
//                currentEditor = (MapEditor) PlatformUI.getWorkbench().getActiveWorkbenchWindow()
//                        .getActivePage().getActiveEditor();
        }

        /**
         * @see org.eclipse.ui.IPartListener2#partActivated(org.eclipse.ui.IWorkbenchPartReference)
         */
        public void partActivated( IWorkbenchPartReference partRef ) {
            if (partRef.getPart(false) instanceof MapEditor) {
                setCurrentEditor((MapEditor) partRef.getPart(false));
            } else if (partRef.getPart(false) instanceof IEditorPart)
                setCurrentEditor(null);

            // Ensure that tools are active
            if (getCurrentEditor() != null)
                getActionBars().activate();
        }

        /**
         * @see org.eclipse.ui.IPartListener2#partBroughtToTop(org.eclipse.ui.IWorkbenchPartReference)
         */
        public void partBroughtToTop( IWorkbenchPartReference partRef ) {
            if (partRef.getPart(false) instanceof MapEditor) {
                setCurrentEditor((MapEditor) partRef.getPart(false));
            } else if (partRef.getPart(false) instanceof IEditorPart)
                setCurrentEditor(null);
        }

        /**
         * @see org.eclipse.ui.IPartListener2#partClosed(org.eclipse.ui.IWorkbenchPartReference)
         */
        public void partClosed( IWorkbenchPartReference partRef ) {
            if (partRef.getPart(false) == currentEditor) {
                if (partRef.getPage().getEditorReferences().length == 0) {
                	dispose();
                    setCurrentEditor(null);
                }
            }
        }

        /**
         * @see org.eclipse.ui.IPartListener2#partDeactivated(org.eclipse.ui.IWorkbenchPartReference)
         */
        public void partDeactivated( IWorkbenchPartReference partRef ) {
            // TODO implement method body

        }

        /**
         * @see org.eclipse.ui.IPartListener2#partOpened(org.eclipse.ui.IWorkbenchPartReference)
         */
        public void partOpened( IWorkbenchPartReference partRef ) {
            // TODO implement method body

        }

        /**
         * @see org.eclipse.ui.IPartListener2#partHidden(org.eclipse.ui.IWorkbenchPartReference)
         */
        public void partHidden( IWorkbenchPartReference partRef ) {
            // TODO implement method body
        }

        /**
         * @see org.eclipse.ui.IPartListener2#partVisible(org.eclipse.ui.IWorkbenchPartReference)
         */
        public void partVisible( IWorkbenchPartReference partRef ) {
            // TODO implement method body
        }

        /**
         * @see org.eclipse.ui.IPartListener2#partInputChanged(org.eclipse.ui.IWorkbenchPartReference)
         */
        public void partInputChanged( IWorkbenchPartReference partRef ) {
            // TODO implement method body
        }

    }

    /**
     * Returns the list of categories containing modal tools.
     * @return the list of categories containing modal tools.
     */
    public List<ModalToolCategory> getModalToolCategories() {
        return modalCategories;
    }
    
    /**
     * Returns the tool category that is currently active.
     * @return the tool category that is currently active.
     */
    public ToolCategory getActiveCategory(){
    	return findModalCategory(ToolProxy.activeItem.getCategoryId());
    }
    
    private static class CategorySorter implements Comparator<String>{

        private final static java.util.Map<String, Integer> values=new HashMap<String, Integer>();
        static {
            values.put(ToolConstants.RENDER_CA, 0);
            values.put(ToolConstants.ZOOM_CA, 1);
            values.put(ToolConstants.PAN_CA, 2); 
            values.put(ToolConstants.SELECTION_CA, 3);
            values.put(ToolConstants.INFO_CA, 4); 
            values.put(ToolConstants.EDIT_CA, 5); 
        }
        
        private static final int max=-1;
        private static final int min=-1;
        /**
         * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
         */
        public int compare( String arg0, String arg1 ) {
            Integer value0=values.get(arg0);
            Integer value1=values.get(arg1);
            if( value0==null && value1==null )
                return 0;
            if( value0!=null && value1==null)
                return max;
            if( value0==null && value1!=null)
                return min;
            if( value0==value1 )
                return 0;
            
            return value0>value1?max:min;
        }
        
    }

    /**
     * This method enables the context that allows tool keybinding to work.
     * 
     * @param site
     */
    public static void addToolScope( IWorkbenchPartSite site ) {
        String[] scopes=site.getKeyBindingService().getScopes();
        String[] newScopes=new String[scopes.length+1];
        System.arraycopy(scopes,0,newScopes,1,scopes.length);
        newScopes[0]="net.refractions.udig.project.ui.tool"; //$NON-NLS-1$
        site.getKeyBindingService().setScopes(newScopes);
    }
}
