/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.project.ui.internal.tool.display;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

/**
 * Contribution items that represents the current item selected.
 * @author jeichar
 * @since 0.9.0
 */
public abstract class ToolbarCurrentItem extends CurrentContributionItem {

    ToolItem toolItem;
	private ModalItem currentTool;
	private boolean checked;
	
	/**
	 * @see org.eclipse.jface.action.ContributionItem#fill(org.eclipse.swt.widgets.ToolBar, int)
	 */
	public final void fill(final ToolBar parent, int index){
		if (getTools().size() == 0)
			return;

		toolItem=createToolItem(parent, index);
		setCurrentTool(getTools().get(0));

		if( isDefaultItem() ){
			runCurrentTool();
		}
		
		toolItem.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				widgetDefaultSelected(e);

			}

			public void widgetDefaultSelected(SelectionEvent e) {
				if( e.detail==SWT.ARROW){
					final Menu menu = new Menu(parent);
					for (final ModalItem tool : getTools()) {
						MenuItem menuItem = new MenuItem(menu, SWT.PUSH);
						menuItem.setImage(tool.getImage());
						menuItem.setText(tool.getName());
						menuItem.addSelectionListener(new SelectionListener() {

							public void widgetSelected(SelectionEvent e) {
								widgetDefaultSelected(e);
							}

							public void widgetDefaultSelected(SelectionEvent e) {
                                setCurrentTool(tool);
								runCurrentTool();
								menu.dispose();
							}
						});
					}
					menu.setVisible(true);
				}else
					runCurrentTool();
			}

		});

	}
	
	
	/**
	 * If the tool is the default tool then run that tool.
	 * 
	 * @return true if the default tool is in the current category and the default category
	 * should be activated.
	 */
	protected abstract boolean isDefaultItem() ;

	/**
	 * Creates the ToolItem to add to the menu.
	 * 
	 * @param parent
	 * @param index
	 */
	protected abstract ToolItem createToolItem(ToolBar parent, int index);
	
	/**
	 * returns the list of items for the category.
	 * 
	 * @return  
	 */
	protected abstract List<ModalItem> getTools() ;

	/**
	 * Sets the current items from the category as the current items for the tool item
	 * 
	 * @param tool the new current tool. It must be one of the items in the category.
	 */
	protected void setCurrentTool(ModalItem tool) {
        currentTool=tool;
        if( toolItem.getImage()==null )
            toolItem.setImage(tool.getImage());
		toolItem.setToolTipText(tool.getToolTipText());
	}
	
	/**
	 * @see net.refractions.udig.project.ui.internal.tool.display.CurrentContributionItem#setSelection(boolean, net.refractions.udig.project.ui.internal.tool.display.ModalItem)
	 */
	public void setSelection(boolean checked, ModalItem tool) {
        setCurrentTool(tool);
        if( !toolItem.isDisposed()){
			if( checked ){
				toolItem.setImage(tool.getActiveImage());
			}else{
				toolItem.setImage(tool.getImage());
			}
            this.checked=checked;
        }
	}

	/**
	 * @see net.refractions.udig.project.ui.internal.tool.display.CurrentContributionItem#isChecked()
	 */
	protected boolean isChecked() {
        return checked;
	}

    /**
     * Runs the current tool.
     */
    protected void runCurrentTool() {
        if (currentTool != null){
            currentTool.run();
        }
    }

    /**
	 * @see net.refractions.udig.project.ui.internal.tool.display.CurrentContributionItem#isDisposed()
	 */
	public boolean isDisposed() {
		return toolItem==null || toolItem.isDisposed();
	}
    
	/**
	 * Sets the current to too be the next tool in the list.
	 */
	public void incrementSelection() {
		int index=getTools().indexOf(currentTool);
		if( index==getTools().size()-1)
			index=0;
		else
			index++;
		setCurrentTool(getTools().get(index));
	}

}