/*
 * Created on Mar 25, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package net.refractions.udig.project.ui.operations.example;

import java.io.IOException;

import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IGeoResourceInfo;
import net.refractions.udig.project.ui.internal.Policy;
import net.refractions.udig.project.ui.internal.ProjectUIPlugin;
import net.refractions.udig.ui.operations.IOp;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.geotools.data.FeatureReader;
import org.geotools.data.FeatureSource;
import org.geotools.data.FeatureStore;
import org.geotools.data.Query;
import org.geotools.data.wms.WebMapServer;

import com.vividsolutions.jts.geom.Envelope;

/**
 * An Operation that can operate on either GeoResources or on FeatureSources.  If the object is a georesource
 * the operation prints out a summary about the resource.  if the object is a FeatureSource the
 * number of features are printed.
 * 
 * @author jones
 */
public class MultiTargetOp implements IOp {

	/**
	 * @see net.refractions.udig.ui.operations.IOp#op(java.lang.Object)
	 */
	public void op(final Display display, Object target, IProgressMonitor monitor) throws Exception {
		if ( target instanceof IGeoResource ){
			IGeoResource resource=(IGeoResource) target;
			op(display, monitor, resource);
		}else if ( target instanceof FeatureSource ){
			FeatureSource source=(FeatureSource) target;
			op(display, monitor, source);
		} 
	}

	/**
	 * @param display
	 * @param monitor
	 * @param resource
	 * @throws IOException
	 */
	private void op(final Display display, IProgressMonitor monitor, IGeoResource resource) throws IOException {
		IGeoResourceInfo info=resource.getInfo(monitor);
        Envelope bounds=info.getBounds();
        final StringBuffer buffer=new StringBuffer();
        try{
        buffer.append(Policy.bind("MultiTargetOp.name")+info.getName()+"\n"); //$NON-NLS-1$ //$NON-NLS-2$
        buffer.append(Policy.bind("MultiTargetOp.title")+info.getTitle()+"\n"); //$NON-NLS-1$ //$NON-NLS-2$
        buffer.append(Policy.bind("MultiTargetOp.bounds")+bounds.getMinX()+","+bounds.getMinY()+")\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        buffer.append("            ("+bounds.getMaxX()+","+bounds.getMaxY()+"\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        String crs;
        if ( info.getCRS()!=null )
            crs=info.getCRS().getName().toString();
        else
            crs=Policy.bind("MultiTargetOp.unknown");  //$NON-NLS-1$
        buffer.append(Policy.bind("MultiTargetOp.crs")+crs+"\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        buffer.append(Policy.bind("MultiTargetOp.featuresource")+resource.canResolve(FeatureSource.class)+"\n"); //$NON-NLS-1$ //$NON-NLS-2$
        buffer.append(Policy.bind("MultiTargetOp.featurestore")+resource.canResolve(FeatureStore.class)+"\n"); //$NON-NLS-1$ //$NON-NLS-2$
        buffer.append(Policy.bind("MultiTargetOp.wms")+resource.canResolve(WebMapServer.class)+"\n"); //$NON-NLS-1$ //$NON-NLS-2$
        buffer.append(Policy.bind("MultiTargetOp.keywords")); //$NON-NLS-1$
        for( String word:info.getKeywords()){
            buffer.append("\t"+word+"\n"); //$NON-NLS-1$ //$NON-NLS-2$
        }
        }catch(Exception e){
            display.asyncExec(new Runnable() {
                public void run() {
                    MessageDialog.openError(display.getActiveShell(), "Resource Summary", //$NON-NLS-1$
                            Policy.bind("MultiTargetOp.error")); //$NON-NLS-1$
                }
            });
            ProjectUIPlugin.log( null,e );
            
        }
		display.asyncExec(new Runnable() {
			public void run() {
				MessageDialog.openInformation(display.getActiveShell(), "Resource Summary", //$NON-NLS-1$
                        buffer.toString());
			}
		});
	}

	private void op(final Display display, IProgressMonitor monitor, FeatureSource source) throws IOException {
        int tmp=0;
        tmp=source.getCount(Query.ALL);
        if( tmp==-1){
            FeatureReader reader=source.getFeatures().reader();

        	try{
            while( reader.hasNext() ){
                try {
                    reader.next();
                } catch (Exception e) {
                    //do nothing
                }
                tmp++;
            }
        	}finally{
        		reader.close();
        	}
        }
		final int features=tmp;
		display.asyncExec(new Runnable() {
			public void run() {
				MessageDialog.openInformation(display.getActiveShell(), "Number of features",  //$NON-NLS-1$
						Policy.bind("MultiTargetOp.number")+(features==-1?Policy.bind("MultiTargetOp.expensive"):String.valueOf(features))); //$NON-NLS-1$ //$NON-NLS-2$
			}
		});
	}

}
