package net.refractions.udig.render.internal.wms.basic;

import java.util.Arrays;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.internal.render.Renderer;
import net.refractions.udig.project.render.ICompositeRenderContext;
import net.refractions.udig.project.render.IRenderContext;
import net.refractions.udig.project.render.IRenderMetrics;
import net.refractions.udig.project.render.IRenderMetricsFactory;
import net.refractions.udig.style.sld.SLDContent;
import net.refractions.udig.ui.graphics.SLDs;

import org.geotools.styling.Style;


/**
 * Used to describe the rendering abilities of BasicWMSRenderer2.
 * @author Richard Gould
 */
public class BasicWMSMetrics2 implements IRenderMetrics {

	
	private IRenderContext context;
    private BasicWMSMetricsFactory2 factory;
    
	/**
     * Construct <code>BasicWMSMetrics2</code>.
     *
     * @param context2
     * @param factory
     */
    public BasicWMSMetrics2( IRenderContext context2, BasicWMSMetricsFactory2 factory ) {
        this.context=context2;
        this.factory=factory;
    }

    public Renderer createRenderer() {
        Renderer renderer=new BasicWMSRenderer2();
        renderer.setContext(context);
        return renderer;
    }
    
    public IRenderContext getRenderContext() {
        return context;
    }

    /**
     * @see net.refractions.udig.project.render.IRenderMetrics#getRenderMetricsFactory()
     */
    public IRenderMetricsFactory getRenderMetricsFactory() {
        return factory;
    }

    public boolean canAddLayer( ILayer layer ) {
        double opacity = Double.NaN;
        
        ICompositeRenderContext context1= (ICompositeRenderContext) context;
        
        if (context1.getContexts().isEmpty()) {
            return true;
        }
        
        IRenderContext[] contexts=context1.getContexts().toArray(new IRenderContext[context1.getContexts().size()]);
        Arrays.sort(contexts);
        for( IRenderContext renderContext: contexts) {
            ILayer previousLayer = renderContext.getLayer();
            Style style = (Style) previousLayer.getStyleBlackboard().get(SLDContent.ID);
            if (style != null) {
                opacity = SLDs.rasterOpacity(SLDs.rasterSymbolizer(style));
            }
        }
                
        Style style = (Style) layer.getStyleBlackboard().get(SLDContent.ID);
        if (style == null && Double.isNaN(opacity)) {
            return true;
        }        
        
        double result = SLDs.rasterOpacity(SLDs.rasterSymbolizer(style));
        
        if (result == opacity) {
            return true;
        }
        return false;
    }

    // XXX: Consider doing more SLD when WMS post is a go
    public boolean canStyle( String SyleID, Object value ) {
        if( value == null || !(value instanceof Style)) return false;
        
        return !Double.isNaN( SLDs.rasterOpacity( (Style) value ) );       
    }
}
