/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.render.internal.wms.basic;

import java.io.IOException;
import java.util.Iterator;
import java.util.Set;

import net.refractions.udig.project.render.IRenderContext;
import net.refractions.udig.project.render.IRenderMetrics;
import net.refractions.udig.project.render.IRenderMetricsFactory;

import org.geotools.data.wms.WebMapServer;
import org.geotools.referencing.CRS;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.MathTransform;

/**
 * Creates Metrics for the BasicWMSRenderer
 * 
 * @author Richard Gould
 */
public class BasicWMSMetricsFactory2 implements IRenderMetricsFactory {

    public boolean canRender( IRenderContext toolkit ) {
        if( !toolkit.getLayer().isType(WebMapServer.class) ) {
            return false; // not a wms
        }
        CoordinateReferenceSystem crs = toolkit.getViewportModel().getCRS();
        if( crs == null ) {
            return true; // we will assume our default            
        }        
        org.geotools.data.ows.Layer layer;
        try {
            layer = toolkit.getLayer().getResource(org.geotools.data.ows.Layer.class, null);
        } catch (IOException e) {
            return false;
        }
        Set srs =  layer.getSrs();
            
        for( Iterator i=srs.iterator(); i.hasNext();) {
            try {
                CoordinateReferenceSystem rs = CRS.decode( (String) i.next() );
                
                if (rs.equals(crs)) {
                    return true;
                }
                
                MathTransform transform = CRS.transform(crs, rs, true);
                
                if (transform != null) {
                    return true;
                }
            }
            catch( Throwable t ) {
                // could not create a object representation of this code 
            }
        }
		return false; // we cannot handle crs
    }

    /**
     * @see net.refractions.udig.project.render.RenderMetricsFactory#createMetrics(net.refractions.udig.project.render.RenderContext)
     */
    public IRenderMetrics createMetrics( IRenderContext context ) {
        return new BasicWMSMetrics2(context, this);
    }
    /**
     * @see net.refractions.udig.project.render.RenderMetrics#getRendererType()
     */
    public Class getRendererType() {
        return BasicWMSRenderer2.class;
    }

}
