/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.style.sld;

import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.style.sld.simple.RasterViewer;
import net.refractions.udig.ui.graphics.SLDs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;
import org.geotools.data.wms.WebMapServer;
import org.geotools.styling.FeatureTypeStyle;
import org.geotools.styling.RasterSymbolizer;
import org.geotools.styling.Rule;
import org.geotools.styling.Style;
import org.geotools.styling.Symbolizer;
import org.opengis.coverage.grid.GridCoverage;

/**
 * Allow editing of a RasterSymbolizaer.
 * <p>
 * Currently this is focused on opacity.
 * </p>
 * @author mleslie
 * @since 1.0.
 */
public class SimpleRasterConfigurator extends AbstractSimpleConfigurator {
    private RasterViewer raster = new RasterViewer();
    
    SelectionListener synchronize = new SelectionListener(){

        public void widgetSelected( SelectionEvent e ) {
            synchronize();
        }

        public void widgetDefaultSelected( SelectionEvent e ) {
            synchronize();
            // apply(); // and apply?
        }
        
    };

    
    /**
     * Construct <code>SimpleRasterConfigurator</code>.
     *
     */
    public SimpleRasterConfigurator() {
        super();
        this.raster.addListener(this.synchronize);
    }

    @Override
    public boolean canStyle( Layer aLayer ) {
        if (aLayer.isType(GridCoverage.class) 
                || aLayer.isType(WebMapServer.class))
            return true;        
        return false;
    }

    @Override
    protected void refresh() {
        Style style = getSLD();
        RasterSymbolizer sym = SLDs.rasterSymbolizer(style);
        this.raster.set(sym);
    }

    @Override
    public void createControl( Composite parent ) {
        setLayout(parent);
        this.raster.createControl(parent, new KeyAdapter() {
            @SuppressWarnings("syntheticAccess") //$NON-NLS-1$
            public void keyReleased(KeyEvent e) {
                /*
                 * I don't like having different ways of checking for keypad enter
                 * and the normal one.  Using the keyCode would be better, but
                 * I couldn't readily find the value for CR.
                 */
                if(e.keyCode == SWT.KEYPAD_CR 
                        || e.character == SWT.CR) {
                    makeActionDoStuff();
                }
            }
        });
    }

    @Override
    public void synchronize() {
        RasterSymbolizer sym = this.raster.get(this.build);
        Rule rule = this.build.createRule(new Symbolizer[] {sym});
        FeatureTypeStyle featureTypeStyle = this.build.createFeatureTypeStyle( "Feature", rule ); //$NON-NLS-1$
        Style style = this.build.createStyle();
        style.setDefault( true );
        style.setName( "simple" ); //$NON-NLS-1$
        style.setFeatureTypeStyles( new FeatureTypeStyle[]{featureTypeStyle,} );
        
        getStyleBlackboard().put(SLDContent.ID, style);        
    }

}
