/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.tool.info.internal;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.ui.internal.ProjectUIPlugin;
import net.refractions.udig.project.ui.internal.properties.FeaturePropertySource;
import net.refractions.udig.tool.info.InfoPlugin;
import net.refractions.udig.tool.info.InfoTool;
import net.refractions.udig.tool.info.LayerPointInfo;
import net.refractions.udig.tool.info.Policy;
import net.refractions.udig.tool.info.internal.display.BrowserInfoDisplay;
import net.refractions.udig.tool.info.internal.display.TextInfoDisplay;
import net.refractions.udig.ui.SearchPart;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.geotools.data.FeatureSource;
import org.geotools.data.ows.Layer;
import org.geotools.feature.Feature;
import org.geotools.geometry.JTS.ReferencedEnvelope;

/**
 * Info recast as a search part.
 * 
 * @author jgarnett
 * @since 1.0.0
 */
public class InfoView2 extends SearchPart {
    /** <code>VIEW_ID</code> field */
    public static final String VIEW_ID = "net.refractions.udig.tool.info.infoView"; //$NON-NLS-1$
    private Text information;
    private TextInfoDisplay textDisplay;
    private BrowserInfoDisplay browserDisplay;
    //private FeatureDisplay featureDisplay;
    private PropertySheetPage featureDisplay;
    
    public static class InfoRequest {
        public ReferencedEnvelope bbox;
        public List<ILayer> layers;
    };
    
    /**
     * 
     */
    public InfoView2() {
        super( InfoPlugin.getDefault().getDialogSettings() );
    }
    
    @Override
    protected Composite createDetails( SashForm splitter ) {
        PageBook book = new PageBook( splitter, SWT.NONE );        
        splitter.setWeights(new int[]{10, 90});

        {
            information = new Text(book, SWT.WRAP );
            information.setText(Policy.bind("InfoView.instructions.text")); //$NON-NLS-1$
            book.showPage(information);            
        }
        {
            textDisplay = new TextInfoDisplay();
            textDisplay.createDisplay(book);
            //textDisplay.getControl().setVisible(false);
        }
        {
            browserDisplay = new BrowserInfoDisplay();
            browserDisplay.createDisplay(book);
            //browserDisplay.getControl().setVisible(false);
        }
        {
            //featureDisplay = new FeatureDisplay();
            //featureDisplay.createDisplay(book);
            featureDisplay = new PropertySheetPage();
            featureDisplay.createControl(book);
            //featureDisplay.getControl().setVisible(false);
        }              
        return book;
    }
    @Override    
    protected void fillActionBars() {
        IActionBars actionBars = getViewSite().getActionBars();
        IToolBarManager toolBar = actionBars.getToolBarManager();
     
        IAction infoTool = ProjectUIPlugin.getDefault().getToolManager().getTool(InfoTool.ID, "information");
        assert( infoTool != null );
        if( toolBar == null ){
            toolBar.add( infoTool );
        }
        super.fillActionBars();
        
    }
    
    @Override
    protected PageBook getDetails() {
        return (PageBook) super.getDetails();
    }
    @Override
    protected void showDetail( Object selection ) {
        if (selection == null ) {
            getDetails().showPage(information);
            return;
        }
        if( selection instanceof Feature ) {
            getDetails().showPage(featureDisplay.getControl());
            FeaturePropertySource source = new FeaturePropertySource( (Feature)selection);
            StructuredSelection sel = new StructuredSelection(source);
            featureDisplay.selectionChanged(null, sel);
            //featureDisplay.setInfo( (Feature) selection );
        }
        if (selection instanceof LayerPointInfo) {
            LayerPointInfo info = (LayerPointInfo) selection;
            if (info.getMimeType() == null) {
                getDetails().showPage(information);
            } else if (info.getMimeType().startsWith(LayerPointInfo.GML)) {
                getDetails().showPage(featureDisplay.getControl());
                try {
                    Feature feature = (Feature) info.aquireValue();
                    FeaturePropertySource src = new FeaturePropertySource(feature);
                    StructuredSelection sel = new StructuredSelection(src);
                    featureDisplay.selectionChanged(null, sel);
                } catch(IOException ex) {
                    InfoPlugin.log("GML value could not be aquired.\n", ex);
                }
                //featureDisplay.setInfo(info);
            } else if (info.getRequestURL() != null &&
                       info.getMimeType().startsWith(LayerPointInfo.HTML)) {
                getDetails().showPage( browserDisplay.getControl() );
                browserDisplay.setInfo(info);        
            } else if (info.getRequestURL() != null && info.getMimeType().startsWith(LayerPointInfo.XML)) {
                getDetails().showPage( browserDisplay.getControl() );
                browserDisplay.setInfo(info);
            } else if (info.getRequestURL() != null && info.getMimeType().startsWith(LayerPointInfo.ERROR)) {
                getDetails().showPage( browserDisplay.getControl() );
                browserDisplay.setInfo(info);
            } else {
                getDetails().showPage(information);
            }
        }        
    }
    /**
     * Called by search( Object filter ) in a job to get stuff done.
     * <p>
     * Resulting like is provided to the viewer via setInput.
     * </p>
     */
    protected List<Object> searchImplementation( Object filter, IProgressMonitor monitor ) {
        InfoRequest request = (InfoRequest) filter;
        List<Object> list = new ArrayList<Object>();
            
        if( monitor != null ) {
            monitor.beginTask( "Information request", request.layers.size() );
        }
        int work = 0;        
        for( ILayer layer : request.layers ) {
            monitor.subTask( layer.getName() );
            monitor.worked( ++work );
            
            if( !layer.isVisible() ) continue;
            //if( !layer.isApplicable( "information" )) continue;
            
            if( layer.isType( FeatureSource.class ) ) {
                try {
                    List<Feature> more = DataStoreDescribeLayer.info2( layer, request.bbox, monitor );
                    if( !more.isEmpty() ) list.addAll( more );
                }
                catch( Throwable t ) {
                    InfoPlugin.log( "Information request "+layer.getName()+" failed "+t, t );
                }
                continue;
            }            
            if( layer.isType( Layer.class ) ) {
                // TODO: freek out because WMS is so hard to use
                try {
                    LayerPointInfo hit = WMSDescribeLayer.info2( layer, request.bbox );
                    if( hit != null ) list.add( hit );
                }
                catch( Throwable t ) {
                    InfoPlugin.log( "Information request "+layer.getName()+" failed "+t, t );
                }
                continue;
            }
        }
        return list;
    }
    @Override
    protected IBaseLabelProvider createLableProvider() {
        return new LabelProvider(){
            public String getText( Object element ) {
                if( element instanceof Feature ) {
                    return ((Feature) element).getID();
                }
                else if( element instanceof LayerPointInfo ) {
                    LayerPointInfo info = (LayerPointInfo) element;                    
                    return info.getLayer().getName();
                }
                return super.getText(element);
            }
        };
    }
}