/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.tool.select;

import java.io.IOException;

import net.refractions.udig.internal.ui.UiPlugin;
import net.refractions.udig.project.internal.ContextModelListenerAdapter;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.ui.internal.MapEditor;
import net.refractions.udig.project.ui.internal.ProjectUIPlugin;
import net.refractions.udig.ui.FeatureTableControl;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.geotools.data.FeatureSource;
import org.geotools.filter.ExpressionBuilder;
import org.geotools.filter.Filter;
import org.geotools.filter.parser.ParseException;

/**
 * Selection view for a Map.
 * </p>
 * Currently this is a playground using the FeatureTable to look at the 
 * query indicated by the current selection.
 * </p>
 * <p>
 * Long term responsibilities include:
 * <ul>
 * <li>Access to a Filter editor for selection specification
 * <li>Allowing in view edits
 * <li>"Moving Goal Posts" idea to communicate forward only cursor in ui terms
 * <li>Real random access for shapefile allowing the table view everyone expects (tm)
 * </ul>
 * @author dzwiers
 * @since 0.6
 */
public class SelectView extends ViewPart {

    FeatureTableControl table;
    private Map map;
    private Layer layer;
    Text text;
    private SashForm sash;
    SelectTreeViewer selectable;
    private IAction select;
    MapEditor currentEditor;
    
    /**
     * Construct <code>SelectView</code>.
     * <p>
     * Don't do setup here - there is an init method you can override that has
     * access to configuration and stuff.
     * </p>
     */
    public SelectView() {
        super();
    }
    
    public void createPartControl( Composite parent ) {
        Label label = new Label(parent, SWT.NONE);
        label.setText("Filter:"); //$NON-NLS-1$
        
        text = new Text(parent, SWT.NONE );
        text.setText("NONE"); //$NON-NLS-1$
        text.setEditable(true);
        text.addSelectionListener(new SelectionListener(){
            public void widgetDefaultSelected( SelectionEvent e ) {
                filter(text.getText());
            }
            public void widgetSelected( SelectionEvent e ) {
                // no need to update on text selection
            }
        });
        sash = new SashForm(parent, SWT.HORIZONTAL);
        selectable = new SelectTreeViewer( sash ); 
        
        //WorkbenchHelp.setHelp(text, IHelpContextIds.FILTER_FIELD);
        table = new FeatureTableControl();        
        table.createTableControl( sash );
        
        sash.setWeights(new int[]{20, 90});
        //WorkbenchHelp.setHelp( table.getControl(), IHelpContextIds.SELECTION_TABLE);
        
        FormLayout layout = new FormLayout();
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        layout.spacing = 0;
        parent.setLayout(layout);
        
        FormData dLabel = new FormData(); // bind to left & text
        dLabel.left = new FormAttachment(0);
        dLabel.top = new FormAttachment(text, 5, SWT.CENTER);
        label.setLayoutData(dLabel);
        
        FormData dText = new FormData(); // bind to top, label, bbox
        dText.top = new FormAttachment(0);
        dText.left = new FormAttachment(label, 5);
        dText.right = new FormAttachment(100,-5);
        text.setLayoutData(dText);
        
        FormData dContents = new FormData(100, 100); // text & bottom
        dContents.right = new FormAttachment(100); // bind to right of form
        dContents.left = new FormAttachment(0); // bind to left of form
        dContents.top = new FormAttachment(text, 2); // attach with 5 pixel offset
        dContents.bottom = new FormAttachment(100); // bind to bottom of form
        sash.setLayoutData(dContents);
        
        // Create menu and toolbars
        createActions();
        
        createMenu();
        createContextMenu();
        createToolbar();
        
        // hookGlobalActions();

        // restore state (from previous session)
        
        
        IWorkbenchPage page = getSite().getPage(); //PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
        
        if( page.getActiveEditor() instanceof MapEditor ){            
            mapActivated( (MapEditor) page.getActiveEditor() );
        }
        // listen to selections
        page.addSelectionListener(new ISelectionListener(){
            public void selectionChanged( IWorkbenchPart part, ISelection selection ) {
                if (!(selection instanceof IStructuredSelection)) return;
                                
                Object selected = ((IStructuredSelection) selection).getFirstElement();
                if( !(selected instanceof Layer) ) return;
                
                layerSelected( (Layer) selected );
            }
        });
        // listen for editor changes
        page.addPartListener(new IPartListener2(){            
            public void partActivated( IWorkbenchPartReference partRef ) {
                if( partRef.getPart(false) instanceof MapEditor ){           
                    mapActivated( (MapEditor) partRef.getPart(false) );
                }
                else if( partRef.getPart(false) instanceof IEditorPart ){
                    mapActivated( null );                    
                }
            }
            public void partBroughtToTop( IWorkbenchPartReference partRef ) {
                if( partRef.getPart(false) instanceof MapEditor ){           
                    mapActivated( (MapEditor) partRef.getPart(false) );
                }
                else if( partRef.getPart(false) instanceof IEditorPart ){
                    mapActivated( null );                    
                }
            }
            public void partClosed( IWorkbenchPartReference partRef ) {
                if( currentEditor == partRef.getPart(false) ){
                    mapActivated( null );
                }
            }
            public void partDeactivated( IWorkbenchPartReference partRef ) {
//              don't think I care
            }
            public void partOpened( IWorkbenchPartReference partRef ) {
//              don't care hope that part is activated
                if( partRef.getPart(false) instanceof MapEditor ){           
                    mapActivated( (MapEditor) partRef.getPart(false) );
                }
                else if( partRef.getPart(false) instanceof IEditorPart ){
                    mapActivated( null );                    
                }
            }
            public void partHidden( IWorkbenchPartReference partRef ) {
//              don't care
            }
            public void partVisible( IWorkbenchPartReference partRef ) {
//              don't care
            }
            public void partInputChanged( IWorkbenchPartReference partRef ) {
//              don't care as Map Editors are not known to change there content                
            }
        });
        
        //provide workbench selections
        getSite().setSelectionProvider(selectable);
    }
    

    /**
     * Create actions, linking view to current map.
     */
    private void createActions() {
        select=ProjectUIPlugin.getDefault().getToolManager().createToolAction(BBoxSelection.ID, ToolsConstants.SELECTION_CATEGORY);
        ImageDescriptor icon = AbstractUIPlugin.imageDescriptorFromPlugin( SelectPlugin.ID, "icons/eview16/select_view.gif"); //$NON-NLS-1$
        select.setImageDescriptor( icon ); //$NON-NLS-1$
        
    }

    /* Called when a Layer is selected - will need to check if we care */
    void layerSelected( Layer selected ){
        if( layer == selected ) return; // we already know
        
        if( !selected.isSelectable() ||
            !selected.isType( FeatureSource.class )){
            layer = null;
            table.message("To display the selection, select a layer (for example in the Layers view), and select the 'Open Selection' menu option. Alternatively, you can drag and drop a layer onto this view." ); //$NON-NLS-1$
            return;
        }
        layer = selected;
        if( map != null && map.getContextModel().getLayers().contains( layer ) ){
            selectable.setSelection( new StructuredSelection( layer ) );    
        }        
        Filter filter = layer.getFilter();
        
        String out = filter.toString();
        System.out.println( "Current fitler:"+filter); //$NON-NLS-1$
        text.setText( out );
        
        try {
            table.setFeatures( layer.getResource(FeatureSource.class, null).getFeatures( layer.getFilter() ) );
        } catch (IOException e) {
            table.message( e.getMessage() );
        }
        
        //update the deep adapter on the map if necessary
        Map selectedMap = layer.getContextModel().getMap();
        if (!selectedMap.equals(map)) {
            if (map != null)
                map.getContextModel().getDeepAdapters().remove(contextListener);
            
            map = selectedMap;
            map.getContextModel().getDeepAdapters().add(contextListener);
        }
        //else do nothing
    }
    
    private ContextModelListenerAdapter contextListener = new ContextModelListenerAdapter() {        
        /**
         * If the notification indicates a change in the list of layers then update the structure,
         * but not labels, of the tree.
         * If the notification is not a context model change then it must be a layer change, if this is the case then
         * update the labels of the layer changed</p> 
         * @param msg The notification
         */
        public void notifyChanged( Notification msg ) {
        	if( !( msg.getNotifier() instanceof Layer) )
        		return;
            final Layer layer1 = (Layer) msg.getNotifier();
            
            
            Display.getDefault().asyncExec(
                new Runnable() {
                    public void run() {
                        Filter filter = layer1.getFilter();
                        
                        String out = filter.toString();
                        System.out.println( "Current fitler:"+filter); //$NON-NLS-1$
                        text.setText( out );
                        
                        try {
                            table.setFeatures( layer1.getResource(FeatureSource.class, null).getFeatures( layer1.getFilter() ) );
                        } catch (IOException e) {
                            table.message( e.getMessage() );
                        }
                        
                        selectable.setInput(layer1.getContextModel().getMap());
                    } 
                }
            );
        }
    };
    
    /**
     * Watch current editor to indicate current selectable layers.
     * 
     * @param editor
     */
    protected void mapActivated( MapEditor editor ) {
        if( currentEditor == editor ) return;
        
        currentEditor=editor;
        if( editor == null ){
            selectable.setInput( null );
            select.setEnabled(false);
            return;
        }
        selectable.setInput( editor.getMap() );        
        select.setEnabled(true);
    }

    private void createToolbar() {
        IToolBarManager toolbar = getViewSite().getActionBars().getToolBarManager();        
        toolbar.add( table.first );
        toolbar.add( table.prev );
        toolbar.add( table.next );
        toolbar.add( table.last );
        
        toolbar.add( select );
    }
    
    private void createContextMenu() {
        final MenuManager contextMenu = new MenuManager();

        contextMenu.setRemoveAllWhenShown(true);
        contextMenu.addMenuListener(new IMenuListener() {
            
            public void menuAboutToShow(IMenuManager mgr) {
                contextMenu.add(new GroupMarker(
                        IWorkbenchActionConstants.MB_ADDITIONS));
                contextMenu.add(UiPlugin.getDefault().getOperationMenuFactory().getContextMenu(  ));
            }

        });

        // Create menu.
        Menu menu = contextMenu.createContextMenu(selectable.getControl());
        selectable.getControl().setMenu(menu);

        // Register menu for extension.
        getSite().registerContextMenu(contextMenu, selectable);
    }

    private void createMenu() {
        // create view menu, consider sync
    }

    /** User has supplied us with a new filter */
    void filter( String filterText ){
        if( layer == null ){
            return; // could not set as we don't have a filter (should be disabled)
        }
        Filter filter;
        try {
            filter = (Filter) ExpressionBuilder.parse(filterText);
        } catch (ParseException e) {
            String message = ExpressionBuilder.getFormattedErrorMessage( e, layer.getName() );
            table.message( message );
            e.printStackTrace();            
            return;
        }
        layer.setFilter( filter ); // do I need to be in a separate thread?
    }
    public void setFocus() {
        // select your "main" control
        if(table.getControl()!=null)
            table.getControl().setFocus();
    }
    
}