/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.tools.edit;

import java.awt.Point;

import net.refractions.udig.core.internal.ExtensionPointList;
import net.refractions.udig.core.internal.FeatureUtils;
import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.ui.internal.commands.draw.DrawEditFeatureCommand;
import net.refractions.udig.project.ui.internal.tool.display.ToolProxy;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseEvent;
import net.refractions.udig.project.ui.tool.IToolContext;
import net.refractions.udig.project.ui.tool.Tool;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.geotools.feature.Feature;
import org.geotools.geometry.JTS;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.MathTransform;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Polygon;

/**
 * A Tool for manipulating point vertices. This tool adds, moves and deletes vertices.
 * 
 * @author aalam
 * @since 0.3
 */
public class PointEditTool extends AbstractEditTool {
    private int mode = -2;
    private Coordinate lastCoordinate=null;
    private ToolProxy.CursorLoader editCursor;
    private Cursor selectionCursor;
    
    
    /**
     * Creates a new instance of AddVertex
     */
    public PointEditTool() {
        ExtensionPointList list = new ExtensionPointList(Tool.EXTENSION_ID);
        for( IConfigurationElement element : list ) {
            if (element.getAttribute("id").equals("net.refractions.udig.tools.pointEdit")) { //$NON-NLS-1$ //$NON-NLS-2$
                IConfigurationElement[] cursorElement = element.getChildren("cursor"); //$NON-NLS-1$
                this.editCursor = new ToolProxy.CursorLoader(cursorElement[0]);
                break;
            }
        }
        ImageDescriptor desc=AbstractUIPlugin.imageDescriptorFromPlugin("net.refractions.udig.tool.default", //$NON-NLS-1$
                "/icons/pointers/point_select_source.png"); //$NON-NLS-1$
        Image image=desc.createImage();
        selectionCursor=new Cursor(Display.getDefault(), image.getImageData(), 5, 5);
        image.dispose();
    }
    
    @Override
    public void dispose() {
        super.dispose();
        editCursor.dispose();
        selectionCursor.dispose();
    }

    /**
     * @see net.refractions.udig.project.ui.tool.AbstractModalTool#setActive(boolean)
     */
    public void setActive( boolean active ) {
        super.setActive(active);
        if( active )
            refresh();
        else{
            if( featureDrawCommand!=null )
                featureDrawCommand.setValid(false);
            getContext().getViewportPane().repaint();           
        }
        if( !active && editing ){
            this.noContextMouseDoubleClicked(null);
        }
        if( active ){
            if( getContext().getEditManager().getEditFeature()!=null )
                setStatusBarMessage(Policy.bind("FeatureEditTool.moveMouse")); //$NON-NLS-1$
            else
                setStatusBarMessage(Policy.bind("FeatureEditTool.newFeature")); //$NON-NLS-1$
        }
        else
            setStatusBarMessage(""); //$NON-NLS-1$

        editing=false;
    }

    private void setStatusBarMessage(final String message ) {
        if( getContext().getMapLayers().size()==0 )
            return;
        getContext().updateUI(new Runnable() {
            public void run() {
                IStatusLineManager bar = getContext().getStatusBar();
                if( bar!=null ){
                    bar.setMessage(message);
                }
            }
        });
    }

    @Override
    public void setContext( IToolContext context ) {
        super.setContext(context);
    }
    
    DrawEditFeatureCommand featureDrawCommand;
    
    private void refresh() {
         if( featureDrawCommand==null ){
             featureDrawCommand = getContext().getDrawFactory().createEditFeaturesCommand(
                     getContext().getViewportModel());
             featureDrawCommand.setDrawKnots(true);
             getContext().sendASyncCommand(featureDrawCommand);
         }
         if( !featureDrawCommand.isValid() ){
             featureDrawCommand.setValid(true);
             getContext().sendASyncCommand(featureDrawCommand);
         }
         
    }

    
    public void noContextMouseMoved( MapMouseEvent e ) {
        if( e.isControlDown() ){
            setCursor(selectionCursor);
            return;
        }

        if (getEditFeature() == null) {
            mode = -2;
            setCursor(editCursor.getCursor());
            return;
        }

        setMode(e);
        refresh();
    }
    

    @Override
    public void noContextMouseDoubleClicked( MapMouseEvent e ) {
        getContext().sendASyncCommand(getContext().getEditFactory().createWriteEditFeatureCommand());
        editing=false;
        lastCoordinate=null;
    }
    public void noContextMouseDragged( MapMouseEvent e ) {
        if (e.button != MapMouseEvent.BUTTON1) {
            return;
        }
        if (getCoords() == null)
            return;
        if (mode >= 0) {
            Coordinate current = getContext().pixelToWorld(e.x, e.y);
            try {
                JTS.transform(current, current, getContext().getMap().getEditManager()
                        .getEditLayer().mapToLayerTransform());
            } catch (Exception e1) {
                EditPlugin.log( "", e1); //$NON-NLS-1$
            }
            
            getCoords()[mode].x = current.x;
            getCoords()[mode].y = current.y;
            
            // Polygons have the first and last point the same so both must be moved.
            if (getGeometry() instanceof Polygon || getGeometry() instanceof MultiPolygon)
                if( mode==0 ){
                    getCoords()[getCoords().length-1].x = current.x;
                    getCoords()[getCoords().length-1].y = current.y;                    
                }else if ( mode==getCoords().length-1 ){
                    getCoords()[0].x = current.x;
                    getCoords()[0].y = current.y;
                }
            getContext().getViewportPane().repaint();
        }
    }

    public void noContextMouseReleased( MapMouseEvent e ) {
    	if( !runsOnLayer() ){
    		Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					MessageDialog.openWarning(Display.getCurrent().getActiveShell(), Policy.bind("PointEditTool.0"), //$NON-NLS-1$
							Policy.bind("PointEditTool.1")); //$NON-NLS-1$
				}
			});
    		return;
    	}
        if( getContext().getMapLayers().size()==0 )
            return;
            
        if (e.button == MapMouseEvent.BUTTON1  && !e.isControlDown()) {
            editing=true;
            try {
                Coordinate click = getContext().pixelToWorld(e.x, e.y);
                Coordinate transformedClick = getTransformedClick(click);
                // TODO
                if (mode == OVER_EDGE) {
                    int index = getClosest(transformedClick, getCoords());
                    Coordinate newCoord = closestPoint(getCoords()[index],
                            getCoords()[index == getCoords().length - 1 ? 0 : index + 1], transformedClick);
                    Geometry newgeom = createGeometry(getGeometry(), newCoord, getCoords()[index]);
                    getContext().getEditManager().getEditFeature().setDefaultGeometry(newgeom);
                } else if( mode == NO_INTERSECTION){
                    if (getGeometry() != null && lastCoordinate!=null ) {
                        Geometry geometry;
                        if (getGeometry().getClass() == LineString.class
                                || getGeometry().getClass() == MultiLineString.class) {
                            geometry = createGeometry(getGeometry(), transformedClick,
                                    getCoords()[getCoords().length - 1]);
                        } else {
                            geometry = createGeometry(getGeometry(), transformedClick, lastCoordinate);
                            lastCoordinate=transformedClick;
                        }
                        getContext().getEditManager().getEditFeature().setDefaultGeometry(geometry);
                    } else if( getGeometry() == null ){
                        ILayer editLayer = findEditLayer();
                        CoordinateReferenceSystem crs = getContext().getViewportModel()
                                .getCRS();
                        try{
                            transformedClick=JTS.transform(click, new Coordinate(),editLayer.mapToLayerTransform());
                        }catch (Exception e1) {
                            transformedClick=click;
                        }
                        lastCoordinate=transformedClick;
                        Feature newFeature = FeatureUtils.createFeature(crs, crs,
                                editLayer.getSchema(), new Coordinate[]{transformedClick},
                                com.vividsolutions.jts.geom.Point.class);
//                        if( editLayer.getSchema().getDefaultGeometry().getType() == com.vividsolutions.jts.geom.Point.class ){
                            getContext().sendSyncCommand(
                                    getContext().getEditFactory().createSetEditFeatureCommand(
                                            newFeature, editLayer));
                            getContext().sendASyncCommand(getContext().getEditFactory().createWriteEditFeatureCommand());
                            editing=false;
                            lastCoordinate=null;
//                        } else {
//                            getContext().sendSyncCommand(
//                                getContext().getEditFactory().createSetEditFeatureCommand(
//                                        newFeature, editLayer));
//                        }
                    }
                }
                setMode(e);
            } catch (Exception exception) {
                EditPlugin.log( "", exception); //$NON-NLS-1$
            }
        }else if( e.button==MapMouseEvent.BUTTON1 && e.isControlDown() ){
            getContext().sendASyncCommand(
                    getContext().getSelectionFactory().createBBoxSelectionCommand(
                    getContext().getBoundingBox(e.getPoint(), 3)) 
                    );
        }
    }

    

    private void setMode( MapMouseEvent e ) {
        if( getEditFeature()==null){
            mode = NO_INTERSECTION;
            setCursor(editCursor.getCursor());
            setStatusBarMessage(Policy.bind("FeatureEditTool.newFeature")); //$NON-NLS-1$
            return;
        }
        Point p = e.getPoint();
        Envelope env = getContext().getBoundingBox(p, 5);
        try {
            ILayer layer = getContext().getMap().getEditManager().getEditLayer();
            MathTransform mt = layer.mapToLayerTransform();
            env = JTS.transform(env, mt);
        } catch (Exception e1) {
            EditPlugin.log( "", e1); //$NON-NLS-1$
        }
        int hit = overVertext(env);
        if (hit != OVER_EDGE) {
            if (mode < 0) {
                mode = hit;
                setCursor(SWT.CURSOR_SIZEALL);
                setStatusBarMessage(Policy.bind("FeatureEditTool.moveVertex")); //$NON-NLS-1$
            }
        } else if (overFeature(env.getMinX(), env.getMinY(), env.getMaxX(), env.getMaxY())) {
            if (mode != OVER_EDGE) {
                mode = OVER_EDGE;
                setCursor(SWT.CURSOR_CROSS);
                setStatusBarMessage(Policy.bind("FeatureEditTool.addVertex")); //$NON-NLS-1$
            }
        } else {
            if (mode != NO_INTERSECTION) {
                mode = NO_INTERSECTION;
                setCursor(editCursor.getCursor());
                if( editing )
                    setStatusBarMessage(Policy.bind("FeatureEditTool.addVertex")); //$NON-NLS-1$
                else
                    setStatusBarMessage(Policy.bind("FeatureEditTool.moveMouse")); //$NON-NLS-1$
            }
        }

        getContext().getViewportPane().repaint();
    }

    private void setCursor( final int cursor_id ) {
        final Display display = PlatformUI.getWorkbench().getDisplay();
        display.asyncExec(new Runnable(){

            public void run() {
                setCursor(display.getSystemCursor(cursor_id));
            }
        });

    }
    void setCursor( final Cursor cursor ) {
        final Display display = PlatformUI.getWorkbench().getDisplay();
        display.asyncExec(new Runnable(){

            public void run() {
                getContext().getViewportPane().setCursor(cursor);
            }
        });
    }

	public boolean runsOnLayer(){
		ILayer selectedLayer = getContext().getEditManager().getSelectedLayer();
		if( selectedLayer.getSchema()==null 
				|| !com.vividsolutions.jts.geom.Point.class.isAssignableFrom(selectedLayer.getSchema().getDefaultGeometry().getType()) 
				&& !MultiPoint.class.isAssignableFrom(selectedLayer.getSchema().getDefaultGeometry().getType())
				&& !Geometry.class.isAssignableFrom(selectedLayer.getSchema().getDefaultGeometry().getType())  )
			return false;
		return true;
	}
}