/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details. */
package net.refractions.udig.tools.internal;

import java.awt.Point;

import net.refractions.udig.core.internal.ExtensionPointList;
import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.ui.commands.IDrawCommand;
import net.refractions.udig.project.ui.internal.tool.display.ToolProxy;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseEvent;
import net.refractions.udig.project.ui.tool.AbstractModalTool;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;
import org.geotools.feature.Feature;
import org.geotools.geometry.JTS;
import org.opengis.referencing.operation.MathTransform;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.LinearRing;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Polygon;

/**
 * A Tool for manipulating geomtry vertices. This tool adds, moves and deletes vertices.
 * 
 * @author jeichar
 * @since 0.3
 */
public class CopyOfFeatureEditTool extends AbstractModalTool {
    int mode = -2;
    private Coordinate[] coords;
    Geometry geom;
    Feature feature;
    IDrawCommand featureDrawCommand;
    private boolean active;
    private boolean warned=false;
    private ToolProxy.CursorLoader cursor;
    /**
     * Creates a new instance of AddVertex
     */
    public CopyOfFeatureEditTool() {
        super(MOUSE | MOTION);
        ExtensionPointList list=new ExtensionPointList(ToolProxy.EXTENSION_ID);
        for( IConfigurationElement element: list){
            if( element.getAttribute("id").equals("net.refractions.udig.tools.featureEdit")){
                IConfigurationElement[] cursor=element.getChildren("cursor");
                this.cursor=new ToolProxy.CursorLoader(cursor[0]);
                break;
            }
        }
    }

    /**
     * @see net.refractions.udig.project.ui.tool.AbstractModalTool#setActive(boolean)
     */
    public void setActive( boolean active ) {
        super.setActive(active);
        this.active=active;
        sendDrawCommand();
   }
    
    void sendDrawCommand(){
        if( active ){
            if( featureDrawCommand==null ){
                featureDrawCommand = getContext().getDrawFactory().createEditFeaturesCommand(
                        getContext().getViewportModel());
                getContext().sendASyncCommand(featureDrawCommand);
            }
            if( !featureDrawCommand.isValid() ){ 
                featureDrawCommand = getContext().getDrawFactory().createEditFeaturesCommand(
                  getContext().getViewportModel());
              getContext().sendASyncCommand(featureDrawCommand);getContext().sendASyncCommand(featureDrawCommand);
            }
//        if( featureDrawCommand!=null )
//            featureDrawCommand.setValid(false);
//            featureDrawCommand = getContext().getDrawFactory().createEditFeaturesCommand(
//                    getContext().getViewportModel());
//            getContext().sendCommand(featureDrawCommand);
        }else{
            if( featureDrawCommand!=null)
                featureDrawCommand.setValid(false);
        }
        getContext().getViewportPane().repaint();
    }
	/**
	 * @see net.refractions.udig.project.ui.tool.AbstractTool#mousePressed(net.refractions.udig.project.render.displayAdapter.MapMouseEvent)
	 */
	public void mousePressed( MapMouseEvent e ) {
        feature = getContext().getEditManager().getEditFeature();
        sendDrawCommand();        

        if (feature == null){
            geom=null;
            coords=null;
            return;
        }
        geom = feature.getDefaultGeometry();
        coords = geom.getCoordinates();
	}
    
    /**
     * @see net.refractions.udig.project.ui.tool.AbstractTool#mouseMoved(net.refractions.udig.project.render.displayAdapter.MapMouseEvent)
     */
    public void mouseMoved( MapMouseEvent e ) {
        //TODO how to determine which geometry to edit?
        mousePressed(e);
        if (feature == null){
            mode=-2;
            setCursor(SWT.CURSOR_ARROW);
            return;
        }

        Point p = e.getPoint();
        Coordinate min = getContext().pixelToWorld(p.x - 3, p.y - 3);
        Coordinate max = getContext().pixelToWorld(p.x + 3, p.y + 3);
        Envelope env = new Envelope(min, max);
        try {
            ILayer layer=getContext().getMap().getEditManager().getEditLayer();
            MathTransform mt=layer.mapToLayerTransform();
            env=JTS.transform(env, mt);
        } catch (Exception e1) {
            ToolsPlugin.log( null, e1);
        }
        int hit = overVertext(env);
        if (hit != -1) {
            if (mode < 0) {
                mode = hit;
                setCursor(SWT.CURSOR_SIZEALL);
            }
        } else if (overFeature(env.getMinX(), env.getMinY(), env.getMaxX(), env.getMaxY())) {
            if (mode != -1) {
                mode = -1;
                setCursor(SWT.CURSOR_CROSS);
            }
        } else {
            if (mode != -2) {
                mode = -2;
                setCursor(cursor.getCursor());
            }
        }
    }

    private void setCursor( final int cursor_id ) {
        final Display display=PlatformUI.getWorkbench().getDisplay();
        display.asyncExec(new Runnable(){

            public void run() {
                setCursor(display.getSystemCursor(cursor_id));
            }
        });
        
    }
    private void setCursor( final Cursor cursor) {
        final Display display=PlatformUI.getWorkbench().getDisplay();
        display.asyncExec(new Runnable(){

            public void run() {
                getContext().getViewportPane().setCursor(cursor);
            }
        });
    }

    /**
     * @see net.refractions.udig.project.ui.tool.AbstractTool#mouseDragged(net.refractions.udig.project.render.displayAdapter.MapMouseEvent)
     */
    public void mouseDragged( MapMouseEvent e ) {
        if (e.button != MapMouseEvent.BUTTON1){
            return;
        }
        if (coords == null)
            return;
        if (mode > 0) {
            Coordinate current = getContext().pixelToWorld(e.x, e.y);
            try {
                JTS.transform(current,current, getContext().getMap().getEditManager().getEditLayer().mapToLayerTransform());
            } catch (Exception e1) {
                ToolsPlugin.log( null, e1);
            }            
            coords[mode].x = current.x;
            coords[mode].y = current.y;
            getContext().getViewportPane().repaint();
        }
    }
    
    private int overVertext( Envelope env ) {
        for( int i = 0; i < coords.length; i++ ) {
            Coordinate coord = coords[i];
            if (env.contains(coord))
                return i;
        }
        return -1;
    }
    private boolean overFeature( double xmin, double ymin, double xmax, double ymax ) {
        Coordinate coord0 = coords[coords.length - 1];
        for( int i = 0; i < coords.length; i++ ) {
            Coordinate coord1 = coords[i];
            if (overLine(coord0.x, coord0.y, coord1.x, coord1.y, xmin, ymin, xmax, ymax))
                return true;
            coord0 = coord1;
        }
        return false;
    }
    private boolean overLine( double x0, double y0, double x1, double y1, double xmin, double ymin, double xmax,
            double ymax ) {
        boolean accept = false, done = false;
        int outcode0 = compOutCode(x0, y0, xmin, xmax, ymin, ymax);
        int outcode1 = compOutCode(x1, y1, xmin, xmax, ymin, ymax);
        do {
            if (outcode0 == 0 || outcode1 == 0)
                accept = done = true;
            else if ((outcode0 & outcode1) != 0)
                done = true;
            else {
                double x, y;
                int outcodeOut = outcode0 > 0 ? outcode0 : outcode1;
                if ((outcodeOut & TOP) > 0) {
                    x = x0 + (x1 - x0) * (ymax - y0) / (y1 - y0);
                    y = ymax;
                } else if ((outcodeOut & BOTTOM) > 0) {
                    x = x0 + (x1 - x0) * (ymin - y0) / (y1 - y0);
                    y = ymin;
                } else if ((outcodeOut & RIGHT) > 0) {
                    y = y0 + (y1 - y0) * (xmax - x0) / (x1 - x0);
                    x = xmax;
                } else {
                    y = y0 + (y1 - y0) * (xmin - x0) / (x1 - x0);
                    x = xmin;
                }
                if (outcodeOut == outcode0) {
                    x0 = x;
                    y0 = y;
                    outcode0 = compOutCode(x0, y0, xmin, xmax, ymin, ymax);
                } else {
                    x1 = x;
                    y1 = y;
                    outcode1 = compOutCode(x1, y1, xmin, xmax, ymin, ymax);
                }
            }
        } while( done == false );
        return accept;
    }

    int TOP = 0x8, BOTTOM = 0x4, RIGHT = 0x2, LEFT = 0x1;

    private int compOutCode( double x, double y, double xmin, double xmax, double ymin, double ymax ) {
        int outcode = 0;
        if (y > ymax)
            outcode |= TOP;
        if (y < ymin)
            outcode |= BOTTOM;
        if (x > xmax)
            outcode |= RIGHT;
        if (x < xmin)
            outcode |= LEFT;
        return outcode;
    }
    private Coordinate closestPoint( Coordinate end1, Coordinate end2, Coordinate p ) {
        Coordinate v = new Coordinate();
        v.x = end2.x - end1.x;
        v.y = end2.y - end1.y;
        double t = ((p.x - end1.x) * v.x + (p.y - end1.y) * v.y) / (v.x * v.x + v.y * v.y);
        if (t > 1 || t < 0 || Double.isInfinite(t) || Double.isNaN(t))
            return null;
        Coordinate result = new Coordinate();
        result.x = end1.x + t * v.x;
        result.y = end1.y + t * v.y;
        return result;
    }
    
    int getClosest(int s, int t, Coordinate[] coords){
        int prev = 0;
        double mindist = Double.MAX_VALUE;
        Coordinate click = getContext().pixelToWorld(s, t);
        Coordinate coord1 = coords[coords.length - 1];
        for( int i = 0; i < coords.length; i++ ) {
            Coordinate coord2 = coords[i];
            Coordinate point = closestPoint(coord1, coord2, click);
            if (point == null) {
                coord1 = coord2;
                continue;
            }
            double x = click.x - point.x;
            double y = click.y - point.y;
            double dist = Math.sqrt(x * x + y * y);
            if (dist < mindist) {
                mindist = dist;
                prev = i;
            }
            coord1 = coord2;
        }
        if( geom instanceof LinearRing){
                return (prev==0||prev==coords.length)?coords.length-2:prev;
        }
        return prev!=0?prev-1:coords.length-2;
    }
    
    /**
     * @see net.refractions.udig.project.ui.tool.AbstractTool#mouseReleased(net.refractions.udig.project.render.displayAdapter.MapMouseEvent)
     */
    public void mouseReleased( MapMouseEvent e ) {
        Coordinate click = getContext().pixelToWorld(e.x, e.y);
        Coordinate transformedClick=getTransformedClick(click);
        //TODO
        if (mode == -1 && e.button == MapMouseEvent.BUTTON1) {
            int index=getClosest(e.x, e.y, coords);
            Coordinate newCoord = closestPoint(coords[index], coords[index==coords.length-1?0:index+1], transformedClick);
            getContext().sendASyncCommand(getContext().getEditFactory().createSetGeomteryCommand(createGeometry(geom, newCoord, coords[index])));
        } else if (mode > -1) {
            getContext().sendASyncCommand(getContext().getEditFactory().createSetGeomteryCommand(feature.getDefaultGeometry()));
        } else if (e.button == MapMouseEvent.BUTTON1) {
            if( geom != null ){
                Geometry geometry;
                if( geom.getClass() == LineString.class ||
                        geom.getClass() == MultiLineString.class)
                    geometry=createGeometry(geom, transformedClick, coords[coords.length-1]);
                else {
    	            int index=getClosest(e.x, e.y, coords);
    	            Coordinate closest=coords[index];
                    geometry=createGeometry(geom, transformedClick, closest);
                }
	            getContext().sendASyncCommand(getContext().getEditFactory().createSetGeomteryCommand(geometry));
            }else{
                getContext().sendASyncCommand(getContext().getEditFactory().createFeature(new Coordinate[]{click}));
            }
        }

        mode=-2;
    }

    private Coordinate getTransformedClick( Coordinate click ) {

        try {
            MathTransform transform=getContext().getEditManager().getEditLayer().mapToLayerTransform();
            if( transform==null || transform.isIdentity())
                return click;
            return JTS.transform(click, new Coordinate(), transform);
        } catch (Exception e1) {
//            CorePlugin.log(ToolsPlugin.getDefault(), e1);
            return click;
        }
    }


    private Geometry createGeometry( Geometry geom, Coordinate newCoord, Coordinate prev ) {
        assert newCoord != null && prev != null && geom != null;
        GeometryFactory factory = geom.getFactory();
        if (geom.getClass() == MultiPoint.class) {
            return factory.createMultiPoint(coords);
        }
        if (geom.getClass() == MultiPolygon.class) {
            MultiPolygon collection = (MultiPolygon) geom;
            return createMultiPolygon(newCoord, prev, factory, collection);
        }
        if (geom.getClass() == MultiLineString.class) {
            MultiLineString collection = (MultiLineString) geom;
            return createMultiLineString(newCoord, prev, factory, collection);
        }
        if (geom.getClass() == Polygon.class) {
            Polygon polygon = (Polygon) geom;
            return createPolygon(newCoord, prev, factory, polygon);
        }
        if (geom.getClass() == LinearRing.class) {
            LinearRing ring = (LinearRing) geom;
            return createLinearRing(newCoord, prev, factory, ring);
        }
        if (geom.getClass() == LineString.class) {
            LineString line = (LineString) geom;
            return createLineString(newCoord, prev, factory, line);
        }
        return null;
    }

    private MultiLineString createMultiLineString( Coordinate newCoord, Coordinate prev, GeometryFactory factory, MultiLineString collection ) {
        LineString[] subgeoms = new LineString[collection.getNumGeometries()];
        for( int i = 0; i < subgeoms.length; i++ ) {
            subgeoms[i] = (LineString) createGeometry(collection.getGeometryN(i), newCoord, prev);
        }
        return factory.createMultiLineString(subgeoms);
    }

    private LineString createLineString(Coordinate newCoord, Coordinate prev, GeometryFactory factory, LineString line ) {
        Coordinate[] coords = line.getCoordinates();
        Coordinate[] newcoords = null;
        newcoords = createNewCoordinate(newCoord, prev, coords, newcoords);
        if (newcoords == null)
            return line;
        return factory.createLineString(newcoords);
    }

    /**
     * TODO summary sentence for createNewCoordinate ...
     * 
     * @param newCoord
     * @param prev
     * @param coords
     * @param newcoords
     * @return
     */
    private Coordinate[] createNewCoordinate( Coordinate newCoord, Coordinate prev, Coordinate[] coords, Coordinate[] newcoords ) {
        for( int i = 0; i < coords.length; i++ ) {
            if (coords[i] == prev) {
                i++;
                newcoords = new Coordinate[coords.length + 1];
                System.arraycopy(coords, 0, newcoords, 0, i);
                newcoords[i] = newCoord;
                if (i < coords.length)
                    System.arraycopy(coords, i, newcoords, i + 1, coords.length - i);
                break;
            }
        }
        return newcoords;
    }

    private LinearRing createLinearRing(Coordinate newCoord, Coordinate prev, GeometryFactory factory, LinearRing ring ) {
        Coordinate[] newcoords = null;
        newcoords = createNewCoordinate(newCoord, prev, ring.getCoordinates(), newcoords);
        if (newcoords == null)
            return ring;
        return factory.createLinearRing(newcoords);
    }

    private Geometry createPolygon( Coordinate newCoord, Coordinate prev, GeometryFactory factory, Polygon polygon ) {
        LinearRing outer = (LinearRing) polygon.getExteriorRing();
        outer = (LinearRing) createGeometry(outer, newCoord, prev);
        LinearRing[] inner = new LinearRing[polygon.getNumInteriorRing()];
        for( int i = 0; i < inner.length; i++ ) {
            inner[i] = (LinearRing) createGeometry(polygon.getInteriorRingN(i), newCoord, prev);
        }
        return factory.createPolygon(outer, inner);
    }

    private Geometry createMultiPolygon( Coordinate newCoord, Coordinate prev, GeometryFactory factory, MultiPolygon collection ) {
        Polygon[] subgeoms = new Polygon[collection.getNumGeometries()];
        for( int i = 0; i < subgeoms.length; i++ ) {
            subgeoms[i] = (Polygon) createGeometry(collection.getGeometryN(i), newCoord, prev);
        }
        return factory.createMultiPolygon(subgeoms);
    }
}