/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.tools.internal;

import java.awt.Point;

import net.refractions.udig.project.ui.internal.commands.draw.TranslateCommand;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseEvent;
import net.refractions.udig.project.ui.tool.AbstractModalTool;
import net.refractions.udig.project.ui.tool.ModalTool;


/**
 * Provides Pan functionality for MapViewport
 *
 * @author Jesse Eichar
 * @version $Revision: 1.9 $
 */
public class Pan extends AbstractModalTool implements ModalTool {
    private boolean dragging=false;
    private Point start=null;

    private TranslateCommand command;
    private static final int PAN_AMOUNT = 30;
    /**
     * Creates an new instance of Pan
     */
    public Pan() {
        super(MOUSE | MOTION);

        //TODO set Cursor
    }

    /**
     * @see net.refractions.udig.project.ui.tool.AbstractTool#mouseDragged(net.refractions.udig.project.render.displayAdapter.MapMouseEvent)
     */
    public void mouseDragged(MapMouseEvent e) {
        if (dragging) {
            command.setTranslation(e.x- start.x, e.y - start.y);
            context.getViewportPane().repaint();
        }
    }

    /**
     * @see net.refractions.udig.project.ui.tool.AbstractTool#mousePressed(net.refractions.udig.project.render.displayAdapter.MapMouseEvent)
     */
    public void mousePressed(MapMouseEvent e) {
        if (e.state == MapMouseEvent.BUTTON1
                && !(e.state == (MapMouseEvent.ALT_DOWN_MASK))) {
            dragging = true;
            start = e.getPoint();
            command=context.getDrawFactory().createTranslateCommand(0,0);
            context.sendASyncCommand(command);
        }
    }

    /**
     * @see net.refractions.udig.project.ui.tool.AbstractTool#mouseReleased(net.refractions.udig.project.render.displayAdapter.MapMouseEvent)
     */
    public void mouseReleased(MapMouseEvent e) {
        if (dragging) {
            Point end=e.getPoint();
            context.sendASyncCommand(context.getNavigationFactory().createPanCommandUsingScreenCoords(start.x-end.x, start.y-end.y));

            dragging = false;
            command.setTranslation(e.x- start.x, e.y - start.y);
        }
    }
    /**
     * @see net.refractions.udig.project.ui.tool.Tool#dispose()
     */
    public void dispose() {
        super.dispose();
    }

    void right() {
        getContext().sendASyncCommand(getContext().getNavigationFactory().createPanCommandUsingScreenCoords(-PAN_AMOUNT,0));
    }
    void left() {
        getContext().sendASyncCommand(getContext().getNavigationFactory().createPanCommandUsingScreenCoords(PAN_AMOUNT,0));
    }
    void up() {
        getContext().sendASyncCommand(getContext().getNavigationFactory().createPanCommandUsingScreenCoords(0,PAN_AMOUNT));
    }
    void down() {
        getContext().sendASyncCommand(getContext().getNavigationFactory().createPanCommandUsingScreenCoords(0,-PAN_AMOUNT));
    }
}
