/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.tools.internal;

import net.refractions.udig.project.command.NavigationCommandFactory;
import net.refractions.udig.project.ui.commands.DrawCommandFactory;
import net.refractions.udig.project.ui.internal.commands.draw.ZoomDrawCommand;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseEvent;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseWheelEvent;
import net.refractions.udig.project.ui.tool.AbstractTool;
import net.refractions.udig.project.ui.tool.IToolContext;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

/**
 * A tool that enables zooming using the mouse wheel and simulates the mouse
 * wheel zoom when the alt key is held down and the mouse is moved horizontally
 * This tool is always "on"
 * 
 * @author Jesse Eichar
 * @version $Revision: 1.9 $
 */
public class ScrollZoom extends AbstractTool {
    private static final double FACTOR = 1.01;

    private static final int INTERVAL = 50;

    private int distance = 0;

    private int start;

    ZoomThread updater=new ZoomThread();

    int amount = 0;

    boolean in = true;
    
    ZoomDrawCommand command; 

    /**
     * Creates an new instance of ScrollZoom
     */
    public ScrollZoom() {
        super(ON|MOUSE|MOTION|WHEEL);
    }

    /**
     * @see net.refractions.udig.project.ui.tool.AbstractTool#mouseWheelMoved(net.refractions.udig.project.render.displayAdapter.MapMouseWheelEvent)
     */
    public void mouseWheelMoved(MapMouseWheelEvent e) {
        zoom(e.clickCount*3);
    }

    /**
     * @see net.refractions.udig.project.ui.tool.AbstractTool#mousePressed(net.refractions.udig.project.render.displayAdapter.MapMouseEvent)
     */
    public void mousePressed(MapMouseEvent e) {
        if (e.state == (MapMouseEvent.ALT_DOWN_MASK | MapMouseEvent.BUTTON1)) {
            start = e.x;
            amount = 0;
        }
    }

    /**
     * @see net.refractions.udig.project.ui.tool.AbstractTool#mouseDragged(net.refractions.udig.project.render.displayAdapter.MapMouseEvent)
     */
    public void mouseDragged(MapMouseEvent e) {
        if (e.state == (MapMouseEvent.ALT_DOWN_MASK | MapMouseEvent.BUTTON1)) {
            distance += (start - e.x);

            if ((distance >= INTERVAL) || (distance <= -INTERVAL)) {
                zoom(10*(distance > 0?1:-1));
                distance = 0;
                start = e.x;
            }
        }
    }

    /**
     * @see net.refractions.udig.project.ui.tool.AbstractTool#mouseReleased(net.refractions.udig.project.render.displayAdapter.MapMouseEvent)
     */
    public void mouseReleased(MapMouseEvent e) {
        distance = 0;
    }

    private void zoom(int change) {
        this.amount += change;
        
        double zoom=Math.abs(Math.pow(FACTOR,amount));
        
        updater.cancel();
        if( command==null){
            int x=context.getMapDisplay().getWidth()/2;
            int y=context.getMapDisplay().getHeight()/2;
            command=DrawCommandFactory.getInstance().createZoomDrawCommand(x,y,zoom);
            context.sendASyncCommand(command);
        }else{
            command.setZoom(zoom);
        }
        
        updater.setContext(context);
        updater.schedule(1000);
    }
    
    class ZoomThread extends Job{
        NavigationCommandFactory factory=NavigationCommandFactory.getInstance();
        IToolContext threadContext;
        
        ZoomThread( ){
            super(Policy.bind("ScrollZoom.scroll.zoom")); //$NON-NLS-1$
            setSystem(true);
            setPriority(INTERACTIVE);
        }
        
        boolean stopped = false;

        void setContext( IToolContext context){
            this.threadContext=context;
        }
        
        void stop() {
            stopped = true;
        }


        /**
         * @see org.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.IProgressMonitor)
         */
        public IStatus run(IProgressMonitor monitor) {
            double zoom = Math.abs(Math.pow(FACTOR, amount));

//            if (amount<0) {
//                zoom = 1 / zoom;
//            }
            command=null;
            threadContext.sendASyncCommand(factory.createZoomCommand(zoom));
            amount = 0;
            return Status.OK_STATUS;
        }
    }

}