/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.ui.graphics;

import java.awt.Color;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.RenderedImage;


/**
 * TODO provide type description
 * 
 * @author jeichar
 * @since TODO provide version
 */
public interface ViewportGraphics{
	/**
	 * Line drawing style for solid lines  (value is 1).
	 */
	public static final int LINE_SOLID = 1;
		
	/**
	 * Line drawing style for dashed lines (value is 2).
	 */
	public static final int LINE_DASH = 2;
		
	/**
	 * Line drawing style for dotted lines (value is 3).
	 */
	public static final int LINE_DOT = 3;
		
	/**
	 * Line drawing style for alternating dash-dot lines (value is 4).
	 */
	public static final int LINE_DASHDOT = 4;
		
	/**
	 * Line drawing style for dash-dot-dot lines (value is 5).
	 */
	public static final int LINE_DASHDOTDOT = 5;
    
    /** <code>ALIGN_TOP</code> field used to align text */
    public static final int ALIGN_TOP = 1;
    /** <code>ALIGN_MIDDLE</code> field used to align text */
    public static final int ALIGN_MIDDLE = 0;
    /** <code>ALIGN_BOTTOM</code> field used to align text */
    public static final int ALIGN_BOTTOM = -1;
    /** <code>ALIGN_LEFT</code> field used to align text */
    public static final int ALIGN_LEFT = -1;
    /** <code>ALIGN_RIGHT</code> field used to align text */
    public static final int ALIGN_RIGHT = 1;

    /**
     * Draws the outline of <code>shape</code> using the color, clip & transform.
     * <p>
     * Reference description from Graphics2d:
     * <bq>
     * Strokes the outline of a <code>Shape</code> using the settings of the 
     * current <code>Graphics2D</code> context.  The rendering attributes
     * applied include the <code>Clip</code>, <code>Transform</code>,
     * <code>Paint</code>, <code>Composite</code> and 
     * <code>Stroke</code> attributes.
     * <bq>
     * </p>
     * @param s the <code>Shape</code> to be rendered
     * @see fill
     */
    public void draw(Shape s);
    
    /**
     * Fills the interior of a <code>Shape</code> using the foreground color, clip & transform.
     * <p>
     * Reference description from Graphics2d:
     * <bq>
     * Fills the interior of a <code>Shape</code> using the settings of the   
     * <code>Graphics2D</code> context. The rendering attributes applied 
     * include the <code>Clip</code>, <code>Transform</code>,
     * <code>Paint</code>, and <code>Composite</code>.
     * @param s the <code>Shape</code> to be filled
     * <bq>
     * </p>
     * @param s the <code>Shape</code> to be rendered
     * @see fill
     */
    public void fill(Shape s);
    
    /**
     * Fills a rectangle.
     * 
     * @param x the starting x coordinate
     * @param y the starting y coordinate
     * @param width the width of the rectangle.
     * @param height the height of the rectangle.
     */
    public void fillRect(int x,int y,int width,int height);
    
    /**
     * Sets the foreground color to draw with.
     * 
     * @param c The new color.
     */
    public void setColor(Color c);
    /**
     * Sets the background color to draw with.
     * 
     * @param c The new color.
     */
    public void setBackground(Color c);
    /**
     * Sets the stroke color to draw with.
     * 
     * @param strokeStyle The style of line to draw.
     * @param strokeWidth the width, in pixels, to draw lines with.
     */
    public void setStroke(int strokeStyle, int strokeWidth);
    /**
     * Sets the clip.  
     * 
     * @param r the rectangle to clip to.
     */
    public void setClip(Rectangle r);
    /**
     * Sets the draw offset.
     * 
     * @param offset The amount the draw is offset in the graphics.
     */
    public void translate( Point offset );
    
    /**
     * Fills the specified rectangle with the background color.
     * 
     * @param x The starting corner's x-coordinate.
     * @param y The starting corner's y-coordinate.
     * @param width the width of the rectangle
     * @param height the height of the rectangle
     */
    public void clearRect( int x, int y, int width, int height );
    /**
     * Draws an image.
     * 
     * @param image The image to draw.
     * @param x The x coordinate of the image top left corner of the image.
     * @param y The y coordinate of the image top left corner of the image.
     */
    public void drawImage( RenderedImage image, int x, int y );    
    /**
     * Draws an image.
     * 
     * @param image The image to draw.
     * @param x The x coordinate of the image top left corner of the image.
     * @param y The y coordinate of the image top left corner of the image.
     */
    public void drawImage( Image image, int x, int y );
    /**
     * 
     * Draws a string. Alignment paramaters specify where the string should be 
     * located relative to coordinate (x,y)
     * 
     * @param string The string to draw. 
     * @param x the x coordinate of the location where the string should be
     * rendered.
     * @param y the y coordinate of the location where the string should be 
     * rendered.
     * @param alignx horizontal alignment, ALIGN_LEFT, ALIGN_MIDDLE or ALIGN_RIGHT
     * @param aligny vertical alignment, ALIGN_BOTTOM, ALIGN_MIDDLE or ALIGN_TOP
     */      
    public void drawString( String string, int x, int y, int alignx, int aligny);
    /**
     * Draws a line from x1,y1 to x2,y2
     * 
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     */
    public void drawLine(int x1, int y1, int x2, int y2);
    /**
     *  Modifies the graphics so that further draws us minX,minY as the origin
     *  and maxX and maxY as the width and height of the display area.
     * @param minX The x-coord that will be used as the origin.
     * @param minY The y-coord that will be used as the origin.
     * @param width The width that will be used to draws.
     * @param height The height that will be used to draws.
     */
    public void setTransform( AffineTransform transform );

    /**
	 * Draws a portion of the image to the target location on the viewport graphics.
     * 
     * @param image image to draw
     * @param dx1 - the x coordinate of the first corner of the destination rectangle.
     * @param dy1 - the y coordinate of the first corner of the destination rectangle.
     * @param dx2 - the x coordinate of the second corner of the destination rectangle.
     * @param dy2 - the y coordinate of the second corner of the destination rectangle.
     * @param sx1 - the x coordinate of the first corner of the source rectangle.
     * @param sy1 - the y coordinate of the first corner of the source rectangle.
     * @param sx2 - the x coordinate of the second corner of the source rectangle.
     * @param sy2 - the y coordinate of the second corner of the source rectangle.
     */
	public void drawImage(Image image, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1, int sx2, int sy2);
    
    /**
     * Gets the height of the current font
     * 
     * TODO at some point maybe this could be broken out to getFontMetrics(),
     * and a create FontMetrics object that maps between SWT and AWT.
     * 
     * @return the height of the current font
     */
    public int getFontHeight();

    /**
     * Returns the length in pixels of the given string, or -1 if this operation
     * is not available.
     * 
     * @param str
     * @return
     */
    public int stringWidth( String str );
    
    /**
     * Gets the <code>ascent</code> of the current font, which is the distance
     * the font rises above its <code>baseline</code>.
     * 
     * @return
     */
    public int getFontAscent();

    /**
     * Returns the bounds of a String, or null if this operation is not 
     * available.
     * 
     * @param str
     * @return
     */
    public Rectangle2D getStringBounds( String str );

	public void drawImage(org.eclipse.swt.graphics.Image image, int i, int j, int minWidth, int minHeight, int k, int l, int minWidth2, int minHeight2);

	public AffineTransform getTransform();
}
